/**
 * \file gre/layersection.h
 * \brief GRE::LAYERSECTION definitions
 *
 * \if NODOC
 * $Id: layersection.h_v 1.15 2012/01/19 22:15:53 mju Exp $
 *
 * $Log: layersection.h_v $
 * Revision 1.15  2012/01/19 22:15:53  mju
 * Add parm to getZView so can skip setup.
 *
 * Revision 1.14  2011/12/01 22:29:31  mju
 * Add getZView and virtual overridable.
 *
 * Revision 1.13  2011/10/19 20:39:35  mju
 * Remove unused overlayRectIfRendered.
 *
 * Revision 1.12  2011/03/21 16:44:44  mju
 * Remove virtual tiling support.
 *
 * Revision 1.11  2010/04/23 16:54:22  mju
 * Use unsignedset.
 *
 * Revision 1.10  2008/08/08 16:49:58  mju
 * Add option to erase renderbuffer when setup.
 *
 * Revision 1.9  2008/08/07 21:21:48  mju
 * Add setupRenderBuffer.
 *
 * Revision 1.8  2008/06/26 17:28:25  mju
 * Add computeRegion.
 *
 * Revision 1.7  2008/06/19 21:45:10  mju
 * Add rect variant for tile overlay.
 *
 * Revision 1.6  2008/06/16 20:47:54  mju
 * Keep track of job so can selectively cancel
 *
 * Revision 1.2  2008/04/11 16:19:48  mju
 * Make overlay methods const.
 *
 * Revision 1.1  2008/03/17 21:33:00  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LAYERSECTION_H
#define	INC_GRE_LAYERSECTION_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_GRE_LABELPLACEMENT_H
	#include <gre/labelplacement.h>
#endif

#ifndef	INC_MI32_RENDERBUFFER_H
	#include <mi32/renderbuffer.h>
#endif

#ifndef	INC_MI32_TRANS2DMAPGEN_H
	#include <mi32/trans2dmapgen.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class UNSIGNEDSET;
class ZVIEW;
namespace MITHREAD {
	class QUEUE;
	}
#endif


namespace GRE {

//===================================================================================================================
//! Container for layer information retained for a section of layer.
class LAYERSECTION {
	public:

		//! Constructor.
		explicit LAYERSECTION (
			GRE::LAYER *layer,
			GRE::VIEW *view
			);

		//! Clear label placement if any.
		void ClearLabelPlacement ();

		//! Close Z view surface objects if any.
		void CloseZView ();

		//! Compute clip region in buffer device coordinates
		void ComputeClipRegion (
			REGION2D& ClipRegion										//!< Clipping region returned
			);

		//! Get set for elements drawn in view (const).
		//! @return NULL if no elements of specified type supported by layer.
		//! The set returned by this method only contains the elements which are actually visible in the view.
		//! Normally when checking for highlighted elements, the GRE::LAYER::GetElemHighlightSet method should be used.
		const UNSIGNEDSET* GetElemDrawnSet (
			ELEMTYPE ElemType,										//! Element type
			bool UseHighlighted										//! True to return set for visible elements drawn as 'highlighted'
			) const { return (const_cast<LAYERSECTION*>(this)->v_GetElemDrawnSet(ElemType,UseHighlighted)); }

		//! Get set for elements drawn in view (non-const).
		//! @return NULL if no elements of specified type supported by layer.
		//! The set returned by this method should only contain the elements which are actually visible in the view.
		//! Normally when checking for highlighted elements, the GRE::LAYER::GetElemHighlightSet method should be used.
		UNSIGNEDSET* GetElemDrawnSet (
			ELEMTYPE ElemType,										//! Element type
			bool UseHighlighted										//! True to return set for visible elements drawn as 'highlighted'
			) { return (v_GetElemDrawnSet(ElemType,UseHighlighted)); }

		//! Get pointer to label placement instance.
		//! @return LABELPLACEMENT pointer or NULL if not supported.
		LABELPLACEMENT* GetLabelPlacement (
			bool ReturnNullIfEmpty = true							//!< Return NULL pointer if no labels
			);

		//! Get associated GRE::LAYER.
		GRE::LAYER* GetLayer (
			) const { return (m_layer); }

		//! Get rendering buffer for specified rendering target (const).
		const RENDERBUFFER& GetRenderBuffer (
			GRE::RENDERTARGET content								//!< Rendering content
			) const;

		//! Get rendering buffer for specified rendering target (non-const).
		RENDERBUFFER& GetRenderBuffer (
			GRE::RENDERTARGET content								//!< Rendering content
			);

		//! Get transformation from layer to buffer.
		const TRANS2D_MAPGEN& GetTransLayerBuffer (
			) const { return (m_TransLayerBuffer); }

		//! Get assigned view.
		GRE::VIEW* GetView (
			) const { return (m_view); }

		//! Get ZVIEW for surface value access and stereo drawing.
		//! @return Pointer to ZVIEW if available, NULL if not.
		const ZVIEW* GetZView (bool DoSetup = true)
			{ return (v_GetZView(DoSetup)); }

		//! Determine if has rendered all of specified content types.
		bool HasRendered (
			GRE::RENDERTARGET content								//!< Rendering content
			) const;

		//! Overlay rendering buffer on device if it has been rendered.
		//! @return true if did overlay, false if not.
		bool OverlayBufferIfRendered (
			GRE::RENDERTARGET content,								//!< Rendering content
			MGD::DEVICE *pDevTarget,								//!< Target device
			const LRECT2D *pClipRect = 0							//!< Optional clip rectangle
			) const;

		//! Create and queue job for rendering section.
		ERRVALUE QueueJobRender (
			MITHREAD::QUEUE& Queue									//! Queue to place job in
			);

		//! Safely release and destroy class instance, supporting multithreading.
		void Release (
			);

		//! Reset all 'drawn element' sets to have no entries.
		void ResetAllDrawnSets (
			);

		//! Reset rendered status so redraws.
		void ResetRendered (
			GRE::RENDERTARGET content = GRE::RENDERTARGET_LayerAll
			);

		//! Set transformation from layer (object) to buffer (device).
		void SetTransLayerBuffer (
			const TRANS2D_MAPGEN& TransLayerBuffer
			) { m_TransLayerBuffer = TransLayerBuffer; }

		//! Setup render buffer.
		ERRVALUE SetupRenderBuffer (
			RENDERBUFFER& RenderBuffer,
			bool EraseExisting = false
			) const;

	protected:

		virtual ~LAYERSECTION ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class JOB_RENDER;

		LAYER *m_layer;
		VIEW *m_view;
		JOB_RENDER *m_pJob;
		RENDERBUFFER m_RenderBufferNormal;
		RENDERBUFFER m_RenderBufferHighlight;
		RENDERBUFFER m_RenderBufferLabels;
		TRANS2D_MAPGEN m_TransLayerBuffer;

		// Unimplemented
		LAYERSECTION (const LAYERSECTION&);
		LAYERSECTION& operator= (const LAYERSECTION&);

		#endif // GENERATING_DOXYGEN_OUTPUT

		// LAYERSECTION Overridables.

		//! Get set for elements drawn in view.
		//! @return NULL if no elements of specified type supported by layer.
		virtual UNSIGNEDSET* v_GetElemDrawnSet (
			ELEMTYPE ElemType,
			bool UseHighlighted
			);

		//! Get pointer label placement manager.
		//! @return NULL if labels not supported.
		virtual LABELPLACEMENT* v_GetLabelPlacement ();

		//! Get ZVIEW for surface value access and stereo drawing.
		//! Default implementation returns NULL.
		virtual ZVIEW* v_GetZView (bool DoSetup);

	};


//===================================================================================================================
//! Layer section supporting geometric element drawn sets.
class LAYERSECTION_GEOMETRIC : public LAYERSECTION {
	public:

		//! Constructor.
		LAYERSECTION_GEOMETRIC (
			GRE::LAYER *layer,
			GRE::VIEW *view
			);

	protected:

		virtual ~LAYERSECTION_GEOMETRIC ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		unsigned m_NumComponents;
		UNSIGNEDSET *m_ElemDrawnSetsNormal;
		UNSIGNEDSET *m_ElemDrawnSetsHighlighted;

		virtual UNSIGNEDSET* v_GetElemDrawnSet (ELEMTYPE ElemType, bool UseHighlighted);

		// Unimplemented
		LAYERSECTION_GEOMETRIC (const LAYERSECTION_GEOMETRIC&);
		LAYERSECTION_GEOMETRIC& operator= (const LAYERSECTION_GEOMETRIC&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
//! Layer section supporting label placement.
class LAYERSECTION_LABELED : public LAYERSECTION_GEOMETRIC {
	public:

		//! Constructor.
		LAYERSECTION_LABELED (
			GRE::LAYER *layer,
			GRE::VIEW *view
			);

	protected:

		virtual ~LAYERSECTION_LABELED ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LABELPLACEMENT m_LabelPlacement;

		virtual LABELPLACEMENT* v_GetLabelPlacement ();

		// Unimplemented
		LAYERSECTION_LABELED (const LAYERSECTION_LABELED&);
		LAYERSECTION_LABELED& operator= (const LAYERSECTION_LABELED&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LAYERSECTION_H
