/**
 * \file gre/rendertotileset.h
 * \brief GRE render to tileset definitions.
 *
 * \if NODOC
 * $Id: rendertotileset.h_v 1.15 2011/10/20 13:10:02 mju Exp $
 *
 * $Log: rendertotileset.h_v $
 * Revision 1.15  2011/10/20 13:10:02  mju
 * Remove redundant hdrs.
 *
 * Revision 1.14  2011/07/13 20:56:59  bcolombe
 * Remove full structure
 *
 * Revision 1.13  2011/06/21 22:47:38  bcolombe
 * Add create full structure
 *
 * Revision 1.9  2010/10/04 18:29:28  bcolombe
 * Use block of sections
 *
 * Revision 1.8  2010/09/20 18:17:42  bcolombe
 * Changes for multiple blocks per job
 *
 * Revision 1.7  2010/07/20 21:34:30  bcolombe
 * Add doBranchKML to parameters
 *
 * Revision 1.3  2010/02/18 17:16:35  bcolombe
 * Support resample method and img format in parameters
 *
 * Revision 1.1  2010/01/27 18:17:17  bcolombe
 * Initial revision
 * \endif
**/

#ifndef INC_GRE_RENDERTOTILESET_H
#define	INC_GRE_RENDERTOTILESET_H

#ifndef INC_GRE_GROUP_H
#include <gre/group.h>
#endif

#ifndef INC_SPATMOD_IMAGETILESET_H
#include <spatmod/imagetileset.h>
#endif

#ifndef INC_MI32_MIJOB_H
#include <mi32/mijob.h>
#endif

namespace GRE {

//=====================================================================================================================
//! Rendering to tileset.
class RENDERTOTILESET {
	public:

		//! Parameters class passed to DoRender
		class PARAMETERS {
			public:
				enum TARGET {
					TARGET_Google_Earth = 1,
					TARGET_Google_Maps = 2,
					TARGET_NASA_WorldWind = 3,
					TARGET_Microsoft_BingMaps = 4,
					};

				//! Section to process definition
				class SECTION {
					public:
						SECTION () {}
						~SECTION () {}

						//! Create section
						ERRVALUE Create(
							TARGET target,			//!< Tileset Target
							INT32 level,			//!< Zoom level for section to render
							const DRECT2D& extents, //!< Extents of section in tileset CRS
							INT32 tilesize = 256,	//!< Tilesize
							INT32 buffersize = 16	//!< Buffer around section
							);

						//! Read from job
						ERRVALUE ReadFromJob (
							const MIJOB& mijob,
							const XMLNODE* ParentNode
							);

						//! Write to job
						void WriteToJob (
							MIJOB& mijob
							);

						const SPATMOD::IMAGE::TILESET& GetTileSet() const {return m_Tileset;}
						INT32 GetBufferSize() const {return m_buffersize;}
						INT32 GetLevel() const {return m_Tileset.GetMaxLevel();}
						DRECT2D GetExtents() const {return m_Tileset.GetExtentsInCRS();}
						double GetCellSize() const {return m_Tileset.ComputeCellSize(GetLevel());}
						INT32 GetTotalRows() const;
						INT32 GetTotalColumns() const;

					private:
						#ifndef GENERATING_DOXYGEN_OUTPUT
						INT32 m_buffersize;
						SPATMOD::IMAGE::TILESET m_Tileset;
						#endif
					};

				class SECTIONBLOCK {
					public:
						SECTIONBLOCK () {}
						~SECTIONBLOCK () {}

						//! Create section
						ERRVALUE Create(
							INT32 level,			//!< Zoom level for section to render
							const DRECT2D& blockextents, //!< Extents of sectionblock in tileset CRS
							INT32 SectionSize,		//!< Size of each section
							INT32 tilesize = 256,	//!< Tilesize
							INT32 buffersize = 16	//!< Buffer around section
							);

						//! Read from job
						ERRVALUE ReadFromJob (
							const MIJOB& mijob,
							const XMLNODE* ParentNode,
							MILIST<SECTION>& Sections
							);

						//! Write to job
						void WriteToJob (
							MIJOB& mijob
							);

					private:
						#ifndef GENERATING_DOXYGEN_OUTPUT
						INT32 m_level;
						DRECT2D m_blockextents;
						INT32 m_SectionSize;
						INT32 m_tilesize;
						INT32 m_buffersize;
						#endif
					};

				PARAMETERS () {}
				~PARAMETERS () {}

				//! Create parameters
				ERRVALUE Create(
					TARGET target,						//!< Tileset Target
					INT32 level,						//!< Zoom level for section to render
					bool bDoBranchKML,					//!< process the kmls as branch kmls
					const DRECT2D& extents,				//!< Extents of section in tileset CRS
					INT32 format,						//!< Image format to use
					INT32 tilesize = 256,				//!< Tilesize
					INT32 buffersize = 16				//!< Buffer around section
					);

				//! Create parameters
				ERRVALUE Create(
					TARGET target,						//!< Tileset Target
					bool bDoBranchKML,					//!< process the kmls as branch kmls
					INT32 format						//!< Image format to use
					);
		
				//! Read from job
				ERRVALUE ReadFromJob (
					const MIJOB& mijob
					);

				//! Write to job
				void WriteToJob (
					MIJOB& mijob
					);

				MILIST<SECTION>& GetSections() {return m_sections;}
				TARGET GetTarget() const {return m_target;}
				INT32 GetFormat() const {return m_ImgFormat;}
				bool GetDoBranchKML() const {return m_bDoBranchKML;}

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MILIST<SECTION> m_sections;
				TARGET m_target;
				INT32 m_ImgFormat;
				bool m_bDoBranchKML;
				#endif
			};

		//! Do rendering of section to tileset
		static ERRVALUE DoRender (
			GRE::GROUP* group,						//!< Viewable to render
			PARAMETERS& parameters,
			const FILEPATH& TilesePath				//!< Filepath of tileset
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		#endif
	};


//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_DLGLAYERMGR_H
