/**
 * \file gre/uiviewcanvas.h
 * \brief GRE::UI::VIEW_CANVAS class definition
 *
 * \if NODOC
 * $Id: uiviewcanvas.h_v 1.31 2013/03/12 14:57:50 mju Exp $
 *
 * $Log: uiviewcanvas.h_v $
 * Revision 1.31  2013/03/12 14:57:50  mju
 * Use form_view in toolAddHyperNavigate.
 *
 * Revision 1.30  2012/12/06 15:43:14  mju
 * Add icid parms to most toolAdd fns.
 *
 * Revision 1.29  2012/08/08 22:54:03  mju
 * Add v_cursorTrackDraw/Restore.
 *
 * Revision 1.28  2012/02/20 20:07:37  mju
 * Use device_mem32rgb for viewinview.
 *
 * Revision 1.26  2011/10/31 17:34:42  mju
 * Add useStereo parm to v_getDrawingDeviceMemory.
 *
 * Revision 1.25  2011/10/04 21:39:31  mju
 * Add toolAddViewPos3D.
 *
 * Revision 1.24  2011/08/17 19:26:35  mju
 * Add toolAddSketch.
 *
 * Revision 1.22  2011/03/01 22:43:56  bcolombe
 * Add OnPointerButtonDown
 *
 * Revision 1.21  2011/02/27 19:24:23  bcolombe
 * Add v_OnPointerButtonUp
 *
 * Revision 1.20  2008/06/03 21:55:12  bcolombe
 * Added toolAddPointProfile
 *
 * Revision 1.19  2008/05/15 17:55:24  mju
 * Chg v_onTilesRendered.
 *
 * Revision 1.18  2008/05/09 17:44:13  mju
 * Add v_onLayerRendered.
 *
 * Revision 1.17  2008/04/23 21:46:34  mju
 * Make toolcanvas a pointer as has private override.
 *
 * Revision 1.16  2008/04/16 13:54:19  mju
 * Add toolAddDragZoom.
 *
 * Revision 1.11  2008/02/07 15:24:47  mju
 * Add toolAddScript fn.
 *
 * Revision 1.8  2007/12/10 21:06:15  mju
 * Add getToolBarPane and corresponding overridable.
 *
 * Revision 1.6  2007/12/03 18:57:41  mju
 * Add overrides for pointer messages.
 *
 * Revision 1.5  2007/11/30 17:28:35  mju
 * Add delegates to handle canvas events.
 *
 * Revision 1.4  2007/11/26 16:34:22  mju
 * Add toolGetActiveID.
 *
 * Revision 1.1  2007/10/08 20:36:10  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIVIEWCANVAS_H
#define	INC_GRE_UIVIEWCANVAS_H

#ifndef	INC_GRE_VIEW_H
#include <gre/view.h>
#endif

#ifndef	INC_GRE_UIVIEWTOOLID_H
#include <gre/uiviewtoolid.h>
#endif

#ifndef	INC_MGUI_TOOLCANVAS_H
#include <mgui/toolcanvas.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
// Forward declarations
namespace GRE {
	class TOOLSCRIPT;
	namespace UI {
		class VIEWTOOL;
		class VIEWTOOLMGR;
		class FORM_VIEW;
		}
	}
namespace MGD {
	class DEVICE_MEM32RGB;
	}
#endif


namespace GRE {
namespace UI {

//===================================================================================================================

class VIEW_CANVAS : public GRE::VIEW {
	public:

		//! Default constructor.
		VIEW_CANVAS ();

		//! Destructor.
		virtual ~VIEW_CANVAS ();

		//! Create canvas and view.
		ERRVALUE CreateCanvas (
			MGUI::LAYOUT_PANE& ParentPane,
			VIEWABLE *viewable,
			int iwidth,
			int iheight,
			const char *sizefield = 0,				//!< Field from .ini for retrieving window size
			CREATEFLAGS createflags = CREATEFLAG_None,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get canvas.
		MGUI::TOOLCANVAS& GetCanvas (
			) { return (*m_pCanvas); }

		//! Get MGD::DEVICE for 'view-in-view'.
		MGD::DEVICE_MEM32RGB* GetDrawingDeviceViewInView (
			) const { return (m_pDeviceViewInView); }

		//! Get pointer to toolbar pane.
		//! @return NULL if no toolbar pane.
		MGUI::LAYOUT_PANE* GetToolBarPane (
			) { return (v_GetToolBarPane()); }

		//! Get tool manager.
		//! Intended for internal use only.
		VIEWTOOLMGR* GetToolMgr (
			) const { return (m_pViewToolMgr); }

		//! Pan canvas by specified amount.
		//! If possible, existing rendering will be copied and exposed area filled in.
		ERRVALUE Pan (
			INT16 dx,									//!< Amount to pan left(-) or right(+) in display pixels
			INT16 dy										//!< Amount to pan up(-) down(+) in display pixels
			);

		//! Find tool using builtin ID.
		VIEWTOOL* ToolFind (
			VIEWTOOLID ViewToolID
			) const;

		//! Get the active tool.
		//! @return Pointer to active tool or NULL if no tool active.
		VIEWTOOL* ToolGetActive (
			) const;

		//! Get ID of active tool.
		//! @return Active tool VIEWTOOLID, VIEWTOOLID_None if no tool active.
		VIEWTOOLID ToolGetActiveID (
			) const;

		//! Set active tool for view (by tool).
		void ToolSet (
			VIEWTOOL* tool								//!< Tool
			);

		//! Set active tool for view (by tool ID).
		void ToolSet (
			VIEWTOOLID ViewToolID					//!< Builtin tool id
			);

		//! Set tool to default.
		void ToolSetDefault (
			);

		//! Zoom view by specified amount.
		//! If possible, existing rendering will be used for initial display before redraw.
		ERRVALUE Zoom (
			double dz,									//!< Change in zoom, > 1 to increase, < 1 to decrease
			INT16 cx,									//!< Center X coordinate to zoom around in device coordinates
			INT16 cy										//!< Center Y coordinate to zoom around in device coordinates
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Install SML class, called by internal SML function.
		static void SML_ClassInstall (
			SML::CONTEXT *context
			);
	#endif

	protected:

		//! \name Overridables
		//!@{

		//! Called when key "pressed".
		//! @return True if key handled, false if not.
		virtual bool v_OnKey (
			MGUI::POINT point,					//!< Point in canvas coordinates where button event occurred
			MGUI::KEYCODE keycode,				//!< Processed key code, equivalent to MIUNICODE for alphanumeric keys.
			MGUI::KEYSTATE keystate				//!< Modifier key state
			);

		//! Called when pointer button pressed.
		//! Override must call base for proper behavior.
		virtual void v_OnPointerButtonDown (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);

		//! Called when pointer button released.
		//! Override must call base for proper behavior.
		virtual void v_OnPointerButtonUp (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);

		//! Called when pointer enters canvas area.
		//! Override must call base for proper behavior.
		virtual void v_OnPointerEnter ();

		//! Called when pointer leaves canvas area.
		//! Override must call base for proper behavior.
		virtual void v_OnPointerLeave ();

		//! Called when pointer is moved.
		//! Override must call base for proper behavior.
		virtual void v_OnPointerMove (
			MGUI::POINT point,					//!< Point in canvas coordinates where button event occurred
			MGUI::KEYSTATE keystate				//!< Modifier key state at time of event
			);

		//!@}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef GRE::VIEW BASECLASS;
		typedef VIEW_CANVAS THISCLASS;
		friend class GRE::UI::VIEWTOOLMGR;
		class PRIVCANVAS;
		friend class GRE::UI::VIEW_CANVAS::PRIVCANVAS;

		MGUI::TOOLCANVAS *m_pCanvas;
		VIEWTOOLMGR *m_pViewToolMgr;
		MGD::DEVICE_MEM32RGB *m_pDeviceViewInView;
		MGUI::CTRL_TOOLTIP *m_pDataTip;

		bool NotifyCanvasKey (MGUI::POINT point, MGUI::KEYCODE keycode, MGUI::KEYSTATE keystate);
		void NotifyCanvasPaint (MGD::CONTEXT *gc);
		void NotifyCanvasPointerButtonDown (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void NotifyCanvasPointerButtonUp (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void NotifyCanvasPointerEnter ();
		void NotifyCanvasPointerLeave ();
		void NotifyCanvasPointerMove (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void NotifyCanvasSize (int width, int height);

		// VIEW overrides.
		virtual void v_CursorTrackDraw ();
		virtual void v_CursorTrackRestore ();
		virtual void v_DataTipSetDelay (INT32 delay);
		virtual void v_DataTipSetPixelDelta (int PixelDelta);
		virtual ERRVALUE v_Draw ();
		virtual ERRVALUE v_DrawThisOnly (GRE::DRAWFLAGS drawflags);
		virtual MDLGPARENT v_GetDlgParent () const;
		virtual MGD::DEVICE* v_GetDrawingDeviceMemory (bool UseStereo) const;
		virtual MGD::DEVICE* v_GetDrawingDeviceScreen () const;
		virtual int v_GetViewNum () const;
		virtual void v_OnLayerRendered (GRE::LAYER *layer);
		virtual void v_RestoreFromMemory (const LRECT2D *rect, bool UseScreenBuffer);
		virtual ERRVALUE v_TestCancel ();
		virtual void v_UpdateMessage (const MISTRING& string);
		virtual void v_UpdateUI ();

		// Unimplemented.
		VIEW_CANVAS (const VIEW_CANVAS&);
		VIEW_CANVAS& operator= (const VIEW_CANVAS&);
		#endif

		// VIEW_CANVAS overridables.

		//! Get pointer to toolbar pane.
		//! Default implementation returns NULL.
		virtual MGUI::LAYOUT_PANE* v_GetToolBarPane ();
	};

//===================================================================================================================

VIEWTOOL* ToolAddDragZoom (VIEW_CANVAS *view);
VIEWTOOL* ToolAddExamineRaster (VIEW_CANVAS *view, ICID icidMenu = 0, ICID icidToolBar = 0);
VIEWTOOL* ToolAddGeoToolbox (VIEW_CANVAS *view, ICID icidMenu = 0, ICID icidToolBar = 0);
VIEWTOOL* ToolAddHyperEdit (VIEW_CANVAS *view);
VIEWTOOL* ToolAddHyperNavigate (FORM_VIEW& FormView);
VIEWTOOL* ToolAddPlacement (VIEW_CANVAS *view);
VIEWTOOL* ToolAddPointProfile (VIEW_CANVAS *view, ICID icidMenu = 0, ICID icidToolBar = 0);
VIEWTOOL* ToolAddScript (VIEW_CANVAS *view, const TOOLSCRIPT& script);
VIEWTOOL* ToolAddSelectPoint (VIEW_CANVAS *view, ICID icidMenu = 0, ICID icidToolBar = 0);
VIEWTOOL* ToolAddSketch (VIEW_CANVAS *view, ICID icidMenu = 0, ICID icidToolBar = 0);
VIEWTOOL* ToolAddViewInView (VIEW_CANVAS *view, ICID icidMenu = 0, ICID icidToolBar = 0);
VIEWTOOL* ToolAddViewPos3D (VIEW_CANVAS *view);
VIEWTOOL* ToolAddZoom (VIEW_CANVAS *view);

//===================================================================================================================
}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIVIEWCANVAS_H
