/**
 * \file mgui/colorbtn.h
 * \brief MGUI CTRL_COLORBUTTON and FORM_COLORBUTTON class definitions.
 *
 * \if NODOC
 * $Id: colorbtn.h_v 1.21 2011/11/09 16:00:02 mju Exp $
 *
 * $Log: colorbtn.h_v $
 * Revision 1.21  2011/11/09 16:00:02  mju
 * Remove mfc defns.
 *
 * Revision 1.20  2011/09/01 17:58:31  mju
 * Exclude macro from dox.
 *
 * Revision 1.19  2007/10/02 15:52:07  mju
 * Add style_AllowTransparency.
 *
 * Revision 1.1  2001/12/13 16:58:26  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_COLORBTN_H
#define	INC_MGUI_COLORBTN_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

namespace MGUI {

//===================================================================================================================
//! PushButton control for showing and selecting a "color".
//! The button will be shown in the selected color and contains no visible text.
class CTRL_COLORBUTTON : public MGUI::CTRL {
	public:

		enum STYLE {
			STYLE_Default =				0x0000,
			STYLE_OutputOnly =			0x0001,		//!< User cannot change value
			STYLE_AllowTransparent =	0x0002,		//!< Allow transparent settings
			STYLE_AllowTransparency =	0x0004,		//!< Allow transparency (and transparent) settings
			};

		//! Constructor.
		CTRL_COLORBUTTON (
			);

		//! Destructor.
		virtual ~CTRL_COLORBUTTON (
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const COLOR& dftcolor,						//!< Default color
			STYLE style = STYLE_Default,				//!< Control style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get current color.
		const COLOR& GetColor (
			) const { return (m_color); }

		//! Get current color.
		COLORREF GetColorRef (
			) const { return (m_color.GetColorRef()); }

		//! Set delegate to call when color is changed.
		void SetDelegateOnChangeColor (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeColor = delegate; }

		//! Set color.
		void SetColor (
			const COLOR& color,						//!< New color value to set
			bool notify = true						//!< Call OnChangeColor() if 'true'
			);

		//! Set color.
		void SetColor (
			const COLORREF color,					//!< New color value to set
			bool notify = true						//!< Call OnChangeColor() if 'true'
			) { COLOR c; c.SetColorRef(color); SetColor(c, notify); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MPIXEL m_pixel;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeColor;
		COLOR m_color;
		STYLE m_style;

		static void CCB_ChangeColorMap (void*, UINT8*);
		static void CB_PushButton (Widget, CTRL_COLORBUTTON*, void*);
		void UpdateButtonColor ();
		void OnChangeColor ();

		CTRL_COLORBUTTON (const CTRL_COLORBUTTON&);
		CTRL_COLORBUTTON& operator= (const CTRL_COLORBUTTON&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_COLORBUTTON::STYLE)
#endif

//===================================================================================================================
//! Form containing label and color pushbutton for showing and selecting a "color".
//! The button will be shown in the selected color and contains no visible text.
class FORM_COLORBUTTON : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_COLORBUTTON (
			);

		//! Destructor.
		virtual ~FORM_COLORBUTTON (
			);

		//! Create form with MISTRING or TEXTID label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			const COLOR& dftcolor,						//!< Default color
			MGUI::CTRL_COLORBUTTON::STYLE buttonstyle = MGUI::CTRL_COLORBUTTON::STYLE_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Get button control.
		MGUI::CTRL_COLORBUTTON& GetButton (
			) { return (m_button); }

		//! Get current color.
		const COLOR& GetColor (
			) const { return (m_button.GetColor()); }

		//! Get current color.
		COLORREF GetColorRef (
			) const { return (m_button.GetColorRef()); }

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_label); }

		//! Set color.
		void SetColor (
			const COLOR& color,						//!< New color value to set
			bool notify = true						//!< Call OnChangeColor() if 'true'
			) { m_button.SetColor(color,notify); }

		//! Set color.
		void SetColor (
			const COLORREF color,					//!< New color value to set
			bool notify = true						//!< Call OnChangeColor() if 'true'
			) { m_button.SetColor(color,notify); }

		//! Set delegate to call when color is changed.
		void SetDelegateOnChangeColor (
			DELEGATE_VOID_NOPARMS delegate
			) { m_button.SetDelegateOnChangeColor(delegate); }

		//! Set label text from MISTRING or TEXTID.
		void SetLabel (
			const MISTRING& label						//!< New label text
			) { m_label.SetLabel(label); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_COLORBUTTON m_button;
		MGUI::CTRL_LABEL m_label;

		FORM_COLORBUTTON (const FORM_COLORBUTTON&);
		FORM_COLORBUTTON& operator= (const FORM_COLORBUTTON&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================

}	// End of MGUI namespace

#endif	// INC_MGUI_COLORBTN_H
