/**
 * \file mgui/dlggetobjectfilter.h
 * \brief MGUI::DLG_GETOBJECT_FILTER definition.
 *
 * \if NODOC
 * $Id: dlggetobjectfilter.h_v 1.7 2011/02/07 15:41:02 bcolombe Exp $
 *
 * $Log: dlggetobjectfilter.h_v $
 * Revision 1.7  2011/02/07 15:41:02  bcolombe
 * Add IgnoreDelegates to IsObjectSelectable
 *
 * Revision 1.6  2009/11/06 22:04:05  mju
 * Use const filter so can use implicit ctors.
 *
 * Revision 1.5  2009/11/05 16:13:53  mju
 * Add ctors using extensions and allow multiple extensions in addFileTypes.
 *
 * Revision 1.4  2009/11/04 22:57:58  mju
 * Use filetype/set classes.
 *
 * Revision 1.3  2009/10/26 20:52:04  mju
 * Add filetype support.
 *
 * Revision 1.2  2009/10/23 21:23:07  mju
 * Add automatic determination of valid parent types if not specified in ctor.
 *
 * Revision 1.1  2009/10/22 21:50:46  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#define	INC_MGUI_DLGGETOBJECTFILTER_H

#ifndef	INC_MI32_DELEGATEFILTER_H
#include <mi32/delegatefilter.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_RVC_OBJTYPESET_H
#include <rvc/otypeset.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
class FILETYPE;
class FILETYPESET;
namespace RVC {
	class OBJITEM;
	class OBJITEMLIST;
	}
#endif

namespace MGUI {

//=====================================================================================================================

//! Filter to determine if objects and files are allowed for selection.
class DLG_GETOBJECT_FILTER {
	public:

		//! Default constructor.
		DLG_GETOBJECT_FILTER (
			);

		//! Implicit constructor using char* file extensions for general file selection.
		DLG_GETOBJECT_FILTER (
			const char *extns					//!< Extensions without wildcard (*) or period (.), may use "*" for all, separate multiple by space or comma, eg "txt,htm"
			);

		//! Implicit constructor using MISTRING file extensions for general file selection.
		DLG_GETOBJECT_FILTER (
			const MISTRING& extns			//!< Extensions without wildcard (*) or period (.), separated by space or comma
			);

		//! Implicit constructor using RVC::OBJTYPESET (or RVC::OBJTYPE).
		DLG_GETOBJECT_FILTER (
			const RVC::OBJTYPESET& ObjTypeSetSelectable
			);

		//! Construct with selectable and valid parent object type sets.
		//! Use of this constructor implies allowing browsing if is valid parent.
		DLG_GETOBJECT_FILTER (
			const RVC::OBJTYPESET& ObjTypeSetSelectable,
			const RVC::OBJTYPESET& ObjTypeSetValidParent
			);

		// Destructor.
		virtual ~DLG_GETOBJECT_FILTER ();

		//! Add delegate to determine if general file is selectable.
		//! If multiple delegates are added the file is considered selectable if v_IsFileSelectable or ANY of the delegates returns true.
		void AddDelegateIsFileSelectable (
			DELEGATE_FILTER_FILEPATH delegate
			);

		//! Add delegate to determine if folder is browsable.
		//! If multiple delegates are added the folder is considered browsable v_IsFolderBrowsable and ALL of the delegates returns true.
		void AddDelegateIsFolderBrowsable (
			DELEGATE_FILTER_FILEPATH delegate
			);

		//! Add delegate to determine if folder is selectable.
		//! If multiple delegates are added the folder is considered selectable if v_IsFolderSelectable or ANY of the delegates returns true.
		void AddDelegateIsFolderSelectable (
			DELEGATE_FILTER_FILEPATH delegate
			);

		//! Add delegate to determine if folder is valid as parent for New File.
		//! If multiple delegates are added the folder is considered valid if v_IsFolderValidParent and ALL of the delegates returns true.
		void AddDelegateIsFolderValidParent (
			DELEGATE_FILTER_FILEPATH delegate
			);

		//! Add delegate to determine if list is valid.
		//! The list is only considered valid if v_IsListValid and ALL of the delegates returns true.
		void AddDelegateIsListValid (
			DELEGATE_FILTER_OBJITEMLIST delegate
			);

		//! Add delegate to determine if object is browsable (ie. has selectable child object).
		//! The object is considered browsable if v_IsObjectBrowsable or ANY of the delegates returns true.
		void AddDelegateIsObjectBrowsable (
			DELEGATE_FILTER_OBJITEM delegate
			);

		//! Add delegate to determine if object is selectable.
		//! The object is considered selectable if ALL "exclusive" delegates return true and ANY "inclusive" delegate returns true.
		//! If no delegate of a particular type is added that check will not be done.
		void AddDelegateIsObjectSelectable (
			DELEGATE_FILTER_OBJITEM delegate,
			bool inclusive = true					//!< True if "inclusive", false if "exclusive"
			);

		//! Add delegate to determine if object is valid as a parent for a New Object.
		//! The object is only considered valid if v_IsObjectValidParent and ALL of the delegates returns true.
		void AddDelegateIsObjectValidParent (
			DELEGATE_FILTER_OBJITEM delegate
			);

		//! Add file type for general file selection.
		void AddFileType (
			const FILETYPE& filetype						//!< File type definition
			);

		//! Add char* file extension(s) for general file selection.
		//! Will lookup extension(s) in registered common file type set to determine description, icon etc.
		void AddFileTypes (
			const char *extn									//!< Single file extension (case-insensitive), do not use ".", may use "*" for all.
			);

		//! Add MISTRING file extension(s) for general file selection.
		//! Will lookup extension(s) in registered common file type set to determine description, icon etc.
		void AddFileTypes (
			const MISTRING& extn								//!< File extension (case-insensitive), do not use "."
			);

		//! Add selectable object type.
		void AddObjTypeSelectable (
			RVC::OBJTYPE ObjTypeSelectable
			);

		//! Add selectable object type set.
		void AddObjTypeSetSelectable (
			const RVC::OBJTYPESET& ObjTypeSetSelectable
			);

		//! Clear current message and error value settings.
		void Clear (
			) const { m_Message.Clear(); m_ErrValue = 0; }

		//! Get whether to allow all file extensions.
		bool GetAllowAllFileExtns (
			) const { return (m_AllowAllFileExtns); }

		//! Determine if allowing all object types.
		bool GetAllowAllObjTypes (
			) const { return (m_AllowAllObjTypes); }

		//! Get current error value.
		ERRVALUE GetErrValue (
			) const { return (m_ErrValue); }

		//! Get set of allowed file types.
		const FILETYPESET& GetFileTypeSet (
			) const { return (*m_pFileTypeSet); }
			
		//! Get message based on stored error value or message string.
		//! Error value takes precedence over any message set.
		MISTRING GetMessage (
			TEXTID TextID = TEXTID__None		//!< Text ID to use for message if none previously set
			) const;

		//! Get set of selectable object types.
		const RVC::OBJTYPESET& GetObjTypeSetSelectable (
			) const { return (m_ObjTypeSetSelectable); }
			
		//! Get set of valid parent object types.
		const RVC::OBJTYPESET& GetObjTypeSetValidParent (
			) const { return (m_ObjTypeSetValidParent); }
			
		//! Determine if general file is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Calls v_IsFileSelectable().
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsFileSelectable (
			const FILEPATH& filepath,							//!< Path to file
			int position = -1,									//!< Position of the object in the set for DLG_GETOBJECT_SET.
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Determine if folder is browsable.
		//! @return 'true' if browsable, 'false' if not.
		//! Calls v_IsFolderBrowsable() and delegates added via AddDelegateIsFolderBrowsable.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsFolderBrowsable (
			const FILEPATH& filepath,							//!< Path to folder
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Determine if folder is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Calls v_IsFolderSelectable() and delegates added via AddDelegateIsFolderSelectable.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsFolderSelectable (
			const FILEPATH& filepath,							//!< Path to folder
			int position = -1,									//!< Position of the object in the set for DLG_GETOBJECT_SET.
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Determine if New File should be enabled.
		//! @return 'true' if valid, 'false' if not.
		//! Calls v_IsFolderValidParent() and delegates added via AddDelegateIsFolderValidParent.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsFolderValidParent (
			const FILEPATH& filepath,							//!< Path to folder
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Determine if list of objects is valid and usable together.
		//! @return 'true' if list is valid, 'false' if not.
		//! Calls v_IsListValid which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsListValid (
			const RVC::OBJITEMLIST& objlist,					//!< Object list to check
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Determine can browse "into" object (ie. has selectable child object).
		//! Calls v_IsObjectBrowsable which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsObjectBrowsable (
			const RVC::OBJITEM& objitem,						//!< Object to check
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Determine if object is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Calls v_IsObjectSelectable() which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsObjectSelectable (
			const RVC::OBJITEM& objitem,						//!< Object to check
			int position = -1,									//!< Position of the object in the set for DLG_GETOBJECT_SET.
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0,		//!< Filter this is a proxy for
			bool bIgnoreDelegates = false					//!< Set true to ignore overrides and delegates
			) const;

		//! Determine if the "New Object" button should be enabled.
		//! @return 'true' if parent is valid parent for creating object under.
		//! Returns false if file is read-only.
		//! Returns false if object type not in set of allowed parent object types.
		//! Calls v_IsObjectValidParent which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsObjectValidParent (
			const RVC::OBJITEM& parent,						//!< Parent object item
			DLG_GETOBJECT_FILTER *pFilterProxyFor = 0		//!< Filter this is a proxy for
			) const;

		//! Set whether to allow all file extensions.
		void SetAllowAllFileExtns (
			bool allow = true
			) { m_AllowAllFileExtns = allow; }

		//! Set whether isObjectBrowsable should check for selectable child/descendent.
		void SetAllowBrowseIfHasSelectableChild (
			bool allow = true
			) const { m_AllowBrowseIfHasSelectableChild = allow; }

		//! Set error value.
		void SetErrValue (
			ERRVALUE err
			) const { m_ErrValue = err; }
			
		//! Set the current message for this filter
		void SetMessage (
			const MISTRING& message
			) const { m_Message = message; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef MILIST<DELEGATE_FILTER_FILEPATH> DELEGATELIST_FILE;
		typedef MILIST<DELEGATE_FILTER_OBJITEMLIST> DELEGATELIST_LIST;
		typedef MILIST<DELEGATE_FILTER_OBJITEM> DELEGATELIST_OBJECT;

		RVC::OBJTYPESET m_ObjTypeSetSelectable;
		mutable RVC::OBJTYPESET m_ObjTypeSetValidParent;
		DELEGATELIST_FILE m_DelegateListIsFileSelectable;
		DELEGATELIST_FILE m_DelegateListIsFolderBrowsable;
		DELEGATELIST_FILE m_DelegateListIsFolderSelectable;
		DELEGATELIST_FILE m_DelegateListIsFolderValidParent;
		DELEGATELIST_OBJECT m_DelegateListIsObjectBrowsable;
		DELEGATELIST_OBJECT m_DelegateListIsObjectSelectableExclusive;
		DELEGATELIST_OBJECT m_DelegateListIsObjectSelectableInclusive;
		DELEGATELIST_OBJECT m_DelegateListIsObjectValidParent;
		DELEGATELIST_LIST m_DelegateListIsListValid;
		FILETYPESET *m_pFileTypeSet;
		bool m_AllowAllFileExtns;
		bool m_AllowAllObjTypes;
		mutable bool m_AllowBrowseIfHasSelectableChild;
		bool m_AllowBrowseIfIsValidParent;
		mutable bool m_NeedUpdateValidParentSet;
		mutable MISTRING m_Message;
		mutable ERRVALUE m_ErrValue;

		void AddFileExtn (const MISTRING& extn);
		void UpdateValidParentSet () const;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Called by IsFileSelectable() to determine if file is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsFileSelectable (
			const FILEPATH& filepath,			//!< File to check
			int position							//!< Position of the file in the set for DLG_GETOBJECT_SET.
			) const;

		//! Called by IsFolderBrowsable() to determine if folder is browsable.
		//! @return 'true' if browsable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsFolderBrowsable (
			const FILEPATH& filepath			//!< Folder to check
			) const;

		//! Called by IsFolderSelectable() to determine if folder is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Default implementation always returns 'false'.
		virtual bool v_IsFolderSelectable (
			const FILEPATH& filepath,			//!< Folder to check
			int position							//!< Position of the file in the set for DLG_GETOBJECT_SET.
			) const;

		//! Called by IsFolderValidParent() to determine if folder is valid for New File.
		//! @return 'true' if browsable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsFolderValidParent (
			const FILEPATH& filepath			//!< Folder to check
			) const;

		//! Called by IsListValid() to determine if list of objects is valid and usable together.
		//! @return 'true' if list is valid, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsListValid (
			const RVC::OBJITEMLIST& objlist
			) const;

		//! Called by IsObjectBrowsable() to determine if has selectable child objects.
		//! Default implementation always returns 'false'.
		virtual bool v_IsObjectBrowsable (
			const RVC::OBJITEM& objitem		//!< Object to check
			) const;

		//! Called by IsObjectSelectable() to determine if object is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsObjectSelectable (
			const RVC::OBJITEM& objitem,		//!< Object to check
			int position							//!< Position of the object in the set for DLG_GETOBJECT_SET.
			) const;

		//! Called by IsObjectValidParent() to determine if "New Object" should be enabled.
		//! @return 'true' if parent is valid parent for creating object under.
		//! Default implementation always returns 'true'.
		virtual bool v_IsObjectValidParent (
			const RVC::OBJITEM& parent
			) const;

	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGGETOBJECTFILTER_H

