/**
 * \file mgui/toolbase.h
 * \brief MGUI::TOOL_BASE class definitions.
 *
 * \if NODOC
 * $Id: toolbase.h_v 1.48 2013/06/05 17:20:52 mju Exp $
 *
 * $Log: toolbase.h_v $
 * Revision 1.48  2013/06/05 17:20:52  mju
 * Add readPenColors.
 *
 * Revision 1.47  2012/12/18 15:40:42  mju
 * Add createControlDialog.
 *
 * Revision 1.46  2012/11/30 22:03:48  mju
 * Add changetype_ScrollUp/Down.
 *
 * Revision 1.45  2012/03/13 20:08:18  mju
 * Add m_inDrawTool.
 *
 * Revision 1.44  2012/01/12 18:11:37  mju
 * Add getCanvasExtent using lrect2d.
 *
 * Revision 1.43  2012/01/09 21:20:48  mju
 * Add usePixmap parm to restoreBackground.
 *
 * Revision 1.42  2011/12/30 17:17:06  mju
 * Add usePixmap parm to drawing and other methods.
 *
 * Revision 1.41  2011/12/22 18:00:15  mju
 * Make getPenWidth non-inline.
 *
 * Revision 1.40  2011/12/21 22:29:57  mju
 * Add drawing methods so can handle stereo.
 * Add set3D and v_onSet3D.
 *
 * Revision 1.39  2011/12/09 20:22:40  mju
 * More stereo support.
 *
 * Revision 1.38  2011/12/08 22:23:47  mju
 * Add getDrawingContext with stereoview and new drawMarker methods without gc.
 *
 * Revision 1.37  2011/12/07 22:36:33  mju
 * Add members for stereo and make tooldlg and snapgrid use pointers as rarely used.
 *
 * Revision 1.36  2011/08/23 22:36:20  mju
 * Add getKeyState.
 *
 * Revision 1.33  2010/11/23 15:19:46  mju
 * Add destroyDrawingDevice.
 *
 * Revision 1.32  2010/07/06 13:55:15  mju
 * Add manual_base::m_userSetCRS.
 *
 * Revision 1.31  2010/02/01 21:34:26  mju
 * Add drawMarker using dpoint2d.
 *
 * Revision 1.29  2008/05/15 19:56:49  mju
 * Call getPenWidthShared from getPenWidth so reads from ini file if needed.
 *
 * Revision 1.27  2008/05/15 17:33:15  mju
 * Add pen width get/set methods.
 *
 * Revision 1.26  2008/05/14 15:19:40  mju
 * Add more diamond marker types.
 *
 * Revision 1.23  2008/04/28 23:04:08  scowan
 * Added virtual method for tool to CRS conversion changes.
 *
 * Revision 1.22  2008/04/23 20:14:03  mju
 * Add isUserModifying.
 *
 * Revision 1.20  2008/02/15 23:37:04  scowan
 * Added manual entry base class methods/definitions and support.
 *
 * Revision 1.19  2008/02/15 18:58:46  mju
 * Add setDelegateOnDestroy.
 * Add GPS methods.
 *
 * Revision 1.18  2008/02/14 23:58:14  scowan
 * Added trans2d mapgen member for manual entry controls.
 *
 * Revision 1.13  2007/12/26 18:30:21  mju
 * Add changetype_Other, make notifyChange public.
 *
 * Revision 1.12  2007/12/19 21:01:44  mju
 * Add more ctrlopt values.
 *
 * Revision 1.1  2003/01/08 22:50:23  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOOLBASE_H
#define	INC_MGUI_TOOLBASE_H

#ifndef	INC_MGUI_TOOLCANVAS_H
	#include <mgui/toolcanvas.h>
#endif

#ifndef	INC_MGUI_FORMBTXT_H
	#include <mgui/formbtxt.h>
#endif

#ifndef	INC_MI32_TRANS2AFFINE_H
	#include <mi32/trans2daffine.h>
#endif

#ifndef	INC_MI32_TRANS2MAPGEN_H
	#include <mi32/trans2dmapgen.h>
#endif

#ifndef	INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class REGION2D;
namespace MGD {
	class CONTEXT;
	}
#endif


namespace MGUI {

//=====================================================================================================================

//! Base class for graphic tools.
//! Derived tool clients should only override the v_OnNotifyChange virtual method to receive tool change notifications.
//! Tool implementations should NOT override the v_OnNotifyChange virtual methods (but may define new v_OnNotify...
//! methods) as this avoids the need for derived tool client classes to call base methods.
class TOOL_BASE {
	public:

		//! Adjustment flags for Move() and Resize() methods.
		enum ADJUSTFLAGS {
			ADJUST_Default =			0x00,			//!< Values are relative tool coordinates
			ADJUST_Absolute =			0x01,			//!< Adjustment values are absolute
			ADJUST_ScreenCoords =	0x02,			//!< Adjustment values are in screen pixels
			ADJUST_Notify =			0x80			//!< Send notification message
			};

		//! Options for standard and manual control forms.
		enum CTRLOPT {
			CTRLOPT_Default =			0x0000,
			CTRLOPT_Manual =			0x0001,	//!< Include "manual" controls if available
			CTRLOPT_GPS =				0x0002,	//!< Include "gps" controls if supported
			CTRLOPT_SnapGrid =		0x0004,	//!< Include "snap to grid" toggle if supported
			CTRLOPT_NoManualProj =	0x0008,	//!< Don't allow manual projection / distance controls
			CTRLOPT_NoEditZ =			0x0010,	//!< Don't include Z value editing controls
			CTRLOPT_NoBtnClear =		0x0020,	//!< Don't include "Clear" button
			};

		//! sequence
		enum CHANGESEQ {
			CHANGESEQ_None =		0,
			CHANGESEQ_Begin =		1,
			CHANGESEQ_Continue =	2,
			CHANGESEQ_End =		3,
			CHANGESEQ_Cancel =	4
			};

		enum CHANGETYPE {
			CHANGETYPE_None =				0,
			CHANGETYPE_Shift =			1,		//!< Overall position was shifted
			CHANGETYPE_Resize =			2,		//!< Tool was resized
			CHANGETYPE_Rotate =			3,		//!< Tool was rotated
			CHANGETYPE_Shear =			4,		//!< Shear was applied
			CHANGETYPE_Edit =				5,		//!< General edit was made
			CHANGETYPE_Create =			6,		//!< New tool was started
			CHANGETYPE_Clear =			7,		//!< Tool was cleared
			CHANGETYPE_ActiveElement =	8,		//!< Active element was selected
			CHANGETYPE_ScrollDown =		9,		//!< Scroll down via pointer wheel (requires SetNotifyScrollWheel)
			CHANGETYPE_ScrollUp =		10,	//!< Scroll up via pointer wheel (requires SetNotifyScrollWheel)
			CHANGETYPE_Other =			20		//!< Other change not listed above
			};

		//! Marker types, not all types are supported by all tools.
		enum MARKER {
			MARKER_X =					0,		//!< Diagonal cross
			MARKER_Plus =				1,		//!< Orthogonal cross
			MARKER_ArrowIn =			2,		//!< Arrow pointing in for line endpoint
			MARKER_ArrowOut =			3,		//!< Arrow pointing out for line endpoint
			MARKER_None =				4,		//!< No marker
			MARKER_Circle =			5,		//!< Circle
			MARKER_Flat =				6,		//!< Flat line endpoint perpendicular to direction
			MARKER_Square =			7,		//!< Square
			MARKER_Diamond =			8,		//!< Square rotated 45 degrees
			MARKER_Diamond_Tall =	9,		//!< Square rotated 45 degrees, height > width
			MARKER_Diamond_Wide =	10,	//!< Square rotated 45 degrees, width > height
			};

		enum PENCOLOR {
			PENCOLOR_Primary,				//!< Primary tool drawing color
			PENCOLOR_Inactive,			//!< Color for inactive elements (eg in POLYLINE tool)
			PENCOLOR_Highlight,			//!< Color for highlighting
			PENCOLOR_COUNT					//!< Number of pens, always last
			};

		//! Snap grid definition.
		class SNAPGRID {
			public:

				//! Default constructor.
				SNAPGRID (
					);

				//! Construct with specified interval.
				explicit SNAPGRID (
					const DPOINT2D& interval
					);

				//! Construct with specified interval and transformation.
				SNAPGRID (
					const DPOINT2D& interval,
					const TRANS2D_AFFINE& trans		//!< Transformation from tool to snap grid coordinates
					);

				//! Adjust specified point to snap grid
				void Adjust (
					DPOINT2D& pt							//!< Point in "tool" coordinates
					) const;

				//! Get snap interval.
				const DPOINT2D& GetInterval (
					) const { return (m_interval); }

				//! Get transformation from tool to snapgrid coordinates.
				const TRANS2D_AFFINE& GetTrans (
					) const { return (m_trans); }

				//! Determine if snapgrid is valid.
				bool IsValid (
					) const { return (m_interval.x > 0 && m_interval.y > 0); }

				//! Set snap interval.
				void SetInterval (
					const DPOINT2D& interval
					) { m_interval = interval; }

				//! Set transformation from tool to snapgrid coordinates.
				void SetTrans (
					const TRANS2D_AFFINE& trans		//!< Transformation from tool to snap grid coordinates
					) { m_trans = trans; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DPOINT2D m_interval;
				TRANS2D_AFFINE m_trans;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Delegate for change notification.
		//! Parameters are: sequence, type.
		typedef fastdelegate::FastDelegate<void(CHANGESEQ, CHANGETYPE)> DELEGATE_ONCHANGE;

		//! Delegate for pointer notification.
		//! Parameters are: canvas point, key state.  Return true if handled, false if not.
		typedef fastdelegate::FastDelegate<bool(MGUI::POINT, MGUI::KEYSTATE)> DELEGATE_ONPOINTER;

		//! Get global tool pen color.
		//! @return Color.
		static const COLOR& GetPenColor (
			PENCOLOR pencolor = PENCOLOR_Primary
			);

		//! Get shared pen width.
		//! @return Pen width in pixels.
		static INT8 GetPenWidthShared ();

		//! Read or re-read pen colors.
		static void ReadPenColors ();

		//! Set global tool pen color.
		static void SetPenColor (
			PENCOLOR pencolor,
			const COLOR& color
			);

		//! Set shared pen width.
		static void SetPenWidthShared (
			INT8 PenWidth
			);

		//! Destructor
		virtual ~TOOL_BASE () = 0;

		//! Activate the tool.
		//! If another tool is currently active on the canvas it will be automatically deactivated.
		void Activate ();

		//! Clear current tool position, erasing all graphics.
		void ClearPosition ();

		//! Create control dialog.
		//! Dialog is managed by tool, will be shown/hidden when tool is activated/deactivated.
		void CreateControlDialog (
			const char *PlacementKey = 0			//!< Dialog placement key
			);

		//! Create "form" with tool-specific controls.
		//! @return Pointer to FORM or NULL if tool has no "control" form.
		MGUI::FORM* CreateControlForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			CTRLOPT ctrlopt							//!< Control options
			) { return (v_CreateControlForm(ParentPane,ctrlopt)); }

		//! Create "form" with tool-specific manual-entry controls.
		//! @return Pointer to FORM or NULL if tool has no "manual entry" form.
		MGUI::FORM* CreateManualForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			CTRLOPT ctrlopt							//!< Control options
			);

		//! Deactivate this tool.
		void Deactivate ();

		//! Generate region from tool.
		ERRVALUE GenerateRegion (
			REGION2D& region						//!< Region in 'tool' coordinates returned
			) const { return (v_GenerateRegion(region)); }

		//! Get movement values associated with arrow keys.
		//! @return Values in "tool" coordinates.
		const DPOINT2D& GetArrowKeyDeltas (
			) const { return (m_ArrowDelta); }

		//! Get current control options settings.
		CTRLOPT GetControlOptions (
			) const { return (m_ctrlopt); }

		//! Get "canvas" associated with tool.
		TOOLCANVAS* GetCanvas (
			) const { return (m_pCanvas); }

		//! Get extents of canvas in DRECT2D.
		void GetCanvasExtents (
			DRECT2D& extents						//!< Extents in canvas coordinates returned
			) const;

		//! Get extents of canvas in LRECT2D.
		void GetCanvasExtents (
			LRECT2D& extents						//!< Extents in canvas coordinates returned
			) const;

		//! Get modifier key state (for use in OnPointer delegate).
		KEYSTATE GetKeyState () const
			{ return (m_KeyState); }

		//! Get transformation between "tool" and current "map" coordinates.
		const TRANS2D_MAPGEN& GetTransToMap (
			) const { return (m_ToolToMapCRS); }

		//! Get transformation between "tool" and "window" coordinates.
		const TRANS2D_AFFINE& GetTransToWindow (
			) const { return (m_ToolToWindow); }

		//! Determine if tool controls available.
		bool HasControls (
			) const { return (m_ControlNeed != CONTROLNEED_None); }

		//! Determine if has manual data entry controls.
		bool HasManualControls (
			) const { return ((m_capability & CAPABILITY_ManualControls) != 0); };

		//! Determine if tool set to be 3D.
		bool Is3D () const
			{ return (m_Is3D); }

		//! Check if tool is currently "active".
		bool IsActive (
			) const { return (m_IsActive); }

		//! Determine if tool is "resizable".
		bool IsResizable (
			) const { return (m_Resizable); }

		//! Determine if tool is "rotatable".
		bool IsRotatable (
			) const { return (m_Rotatable); }

		//! Check if tool is "suspended"
		bool IsSuspended (
			) const { return (m_SuspendCount > 0); }

		//! Check if tool has a valid position set
		bool HasPosition (
			) const { return (m_PosnSet); }

		//! Move the tool by the specified amount.
		void Move (
			double dx,
			double dy,
			ADJUSTFLAGS flags
			);

		//! Send notification of "change".
		//! This should be called by tool implementation for any actual change to tool.
		//! This is public for cases where tool client may also send notifications for convenience.
		void NotifyChange (
			CHANGESEQ changeseq,						//!< Sequence
			CHANGETYPE changetype					//!< Type
			);

		//! Resize the tool.
		void Resize (
			double dx,
			double dy,
			ADJUSTFLAGS flags
			);

		//! Resume the tool after performing drawing operation on the canvas.
		void Resume ();

		//! Set whether tool contains 3D values.
		void Set3D (
			bool is3D = true
			);

		//! Set movement amounts associated with arrow keys.
		void SetArrowKeyDeltas (
			double dx,
			double dy
			);

		//! Set whether user can start drawing a "new" tool.
		void SetCanStartNew (
			bool canstartnew
			) { m_CanStartNew = canstartnew; }

		//! Set controls to appear on dialog when shown.
		//! If used, this must be called before tool is first activated.
		void SetControlOptions (
			CTRLOPT ctrlopt								//!< Control settings
			) { m_ctrlopt = ctrlopt; }

		//! Set delegate to call when tool changes.
		void SetDelegateOnChange (
			DELEGATE_ONCHANGE delegate
			) { m_DelegateOnChange = delegate; }

		//! Set delegate to call when tool destroyed.
		//! Note, this is called from base class destructor after derived class destructor
		//! has already been called.  Should generally only be used to "detach" tool to prevent
		//! further references.  Do not call any tool methods from the delegate.
		void SetDelegateOnDestroy (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnDestroy = delegate; }

		//! Set delegate for right button press.
		void SetDelegateOnRightButtonDown (
			DELEGATE_ONPOINTER delegate
			) { m_DelegateOnRightButtonDown = delegate; }

		//! Set delegate for right button release.
		void SetDelegateOnRightButtonUp (
			DELEGATE_ONPOINTER delegate
			) { m_DelegateOnRightButtonUp = delegate; }

		//! Set whether to send notification for scroll wheel.
		//! Default is to not send notification.
		void SetNotifyScrollWheel (
			bool notify = true
			) { m_NotifyScrollWheel = notify; }

		//! Set individual tool pen width.
		void SetPenWidth (
			INT8 PenWidth								//!< Pen width in pixels, < 0 to use shared setting
			);

		//! Set whether tool is resizable.
		void SetResizable (
			bool resizable
			) { m_Resizable = resizable; }

		//! Set whether tool is rotatable.
		void SetRotatable (
			bool rotatable
			) { m_Rotatable = rotatable; }

		//! Set transformation between "tool" and "map" coordinates.
		void SetTransToMap (
			const TRANS2D_MAPGEN& TransToMap		//!< Transformation from "tool" to "map" coordinates
			);

		//! Set transformation between "tool" and "window" coordinates.
		void SetTransToWindow (
			const TRANS2D_AFFINE& trans
			);

		//! Determine if snap grid is currently in use.
		bool SnapGridIsActive (
			) const { return (m_UseSnapGrid && m_pSnapGrid != 0); }

		//! Set snap grid parameters.
		void SnapGridSet (
			const SNAPGRID& snapgrid
			);

		//! Set whether to use snap grid or not.
		void SnapGridSetActive (
			bool active
			) { m_UseSnapGrid = active; }

		//! Enable or disable stereo mode.
		//! Note that the TOOLCANVAS::v_GetZView method must also be overridden and not return NULL for stereo.
		void StereoEnable (
			bool enable					//!< true to enable, false to disable
			);

		//! Determine if stereo is enabled.
		//! Includes check whether enabled for CANVAS and for non-null CANVAS::GetZView result.
		bool StereoIsEnabled () const;

		//! Suspend tool prior to performing a drawing operation on the canvas.
		void Suspend ();

	protected:

		enum CONTROLNEED {
			CONTROLNEED_None,
			CONTROLNEED_Optional,
			CONTROLNEED_Required
			};

		enum CAPABILITY {
			CAPABILITY_Default =				0,
			CAPABILITY_ManualControls =	0x01,		//!< Manual controls available
			CAPABILITY_Stereo =				0x02,		//!< Stereo allowed
			CAPABILITY_UsePixmap =			0x04,		//!< Use canvas Pixmap buffer if available
			};

		//! Base class for manual entry classes implemented by tools based on TOOL_BASE.
		class MANUAL_BASE : public MGUI::FORM_COMPOSITE {
			public:
				//! Constructor
				MANUAL_BASE ();

				//! Destructor
				virtual ~MANUAL_BASE () = 0;

				//! Create the content for the manual entry form
				void Create (
					MGUI::LAYOUT_PANE_BASE &ParentPane,
					TOOL_BASE::CTRLOPT ctrlopt
					);

				//! Send notification that "change" action has or will occur.
				void NotifyChange (
					CHANGESEQ changeseq,						//!< Sequence
					CHANGETYPE changetype					//!< Type
					) { v_OnNotifyChange(changeseq, changetype); }
					
				//! Set the transformation between Tool and a Map Coordinate Reference System (CRS)
				//! The 'Output' side CRS will be used as the initial display CRS for manual entry fields
				//! if not already set
				void SetTransToolToMapCRS (
					const TRANS2D_MAPGEN& ToolToMapCRS
					);

			protected:
				//! Get the current Tool to manual entry Display Coordinate Reference System (CRS).
				const TRANS2D_MAPGEN& GetToolToDisplayCRS (
					) const { return (m_ToolToDisplayCRS); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				TRANS2D_MAPGEN m_ToolToDisplayCRS;
				FORM_BUTTONTEXT m_BtxCoordRefSys;
				bool m_UserSetCRS;

				void OnBtnCoordRefSys ();
				#endif 		// GENERATING_DOXYGEN_OUTPUT

				//! Virtual method to create the content of the manual entry form specific to each tool.
				//! The CRS selection form will be created AFTER this method is called.
				virtual void v_CreateContent (
					MGUI::LAYOUT_PANE_BASE &ParentPane
					) = 0;

				//! Called when the user selects a different Display CRS than is currently used.
				//! Allows updating tool coordinates when Display CRS is changed.
				virtual void v_OnDisplayCoordRefSysChange () = 0;

				//! Called when change made to tool position, shape, size, etc.
				virtual void v_OnNotifyChange (
					CHANGESEQ changeseq,						//!< Sequence
					CHANGETYPE changetype					//!< Type
					);
			};

		//! Constructor
		explicit TOOL_BASE (
			TOOLCANVAS *pCanvas,									//!< Canvas to contain tool
			CONTROLNEED ControlNeed = CONTROLNEED_None,	//!< Need for control dialog or form
			CAPABILITY capability = CAPABILITY_Default	//!< Additional capabilities
			);

		void AdjustBegin (CHANGETYPE changetype, bool redraw = true);

		void AdjustEnd (bool releasecapture = true, bool cancel = false);

		void ApplyMoveDeltas (
			DPOINT2D& point,
			double dx,
			double dy,
			ADJUSTFLAGS flags
			) const;

		//! Snap point to current snap grid, if no snapping enabled point unchanged.
		void ApplySnap (
			DPOINT2D& point							//!< Point to adjust
			) const;

		//! Calculate rectangle covered by marker, accounting for stereo mode if used.
		void CalcMarkerRectTool (
			const DPOINT2D& ptTool,					//!< Point in tool coordinates
			UINT16 size,								//!< Marker size
			LRECT2D& rectCanvas,						//!< Rectangle on canvas returned.
			bool AddLineWidth = true				//!< Include current pen width in rectangle
			) const;

		//! Calculate rectangle covered by marker, accounting for stereo mode if used.
		void CalcMarkerRectTool (
			const DPOINT3D& ptTool,					//!< Point in tool coordinates
			UINT16 size,								//!< Marker size
			LRECT2D& rectCanvas,						//!< Rectangle on canvas returned.
			bool AddLineWidth = true				//!< Include current pen width in rectangle
			) const;

		//! Determine if can use canvas pixmap.
		//! @return true if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers.
		bool CanUsePixmap () const;

		bool CanStartNew () const
			{ return (m_CanStartNew); }

		//! Draw single line in "tool" coordinates.
		void DrawLineTool (
			const POLYLINE& plTool,						//!< Line in "tool" coordinates
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			const double *DashArray = 0,				//!< Dash array for drawing "closing" line
			int DashArraySize = 0,						//!< Number of dash array entries
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Draw a marker in the position and size specified.
		void DrawMarker (
			const DPOINT2D& ptCanvasCenter,			//!< Point in canvas coordinates for "center" of marker
			const DPOINT2D& ptCanvasDir,				//!< Point in canvas coordinates used to determine direction
			MARKER marker,									//!< Type of marker to draw
			UINT16 size,									//!< Size in pixels
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Draw marker (MARKER...) in the position and size specified.
		void DrawMarker (
			const LPOINT2D& ptCanvasCenter,			//!< Point in canvas coordinates for "center" of marker
			const LPOINT2D& ptCanvasDir,				//!< Point in canvas coordinates used to determine direction
			MARKER marker,									//!< Type of marker to draw
			UINT16 size,									//!< Size in pixels
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Draw non-directional marker using specified "tool" coordinates.
		void DrawMarkerTool (
			const DPOINT3D& ptTool,						//!< Point in tool coordinates for center of marker
			MARKER marker,									//!< Type of marker to draw
			UINT16 size,									//!< Size in pixels
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Draw optionally directional marker using specified "tool" coordinates.
		void DrawMarkerTool (
			const DPOINT3D& ptToolCenter,				//!< Point in tool coordinates for center of marker
			const DPOINT2D& ptToolDir,					//!< Point in tool coordinates used to determine direction
			MARKER marker,									//!< Type of marker to draw
			UINT16 size,									//!< Size in pixels
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Draw segment in "tool" coordinates.
		void DrawSegmentTool (
			const DPOINT3D& ptTool0,					//!< Starting point in "tool" coordinates
			const DPOINT3D& ptTool1,					//!< Ending point in "tool" coordinates
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			const double *DashArray = 0,				//!< Dash array if drawing dashed line
			int DashArraySize = 0,						//!< Number of dash array entries
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Draw segment in "window" coordinates.
		void DrawSegmentWindow (
			const DPOINT3D& ptTool0,					//!< Starting point in "tool" coordinates
			const DPOINT3D& ptTool1,					//!< Ending point in "tool" coordinates
			const COLOR *pColor = 0,					//!< Color, if 0 will use primary pen color
			const double *DashArray = 0,				//!< Dash array if drawing dashed line
			int DashArraySize = 0,						//!< Number of dash array entries
			bool UsePixmap = false						//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		bool GetArrowDeltas (KEYCODE, double*, double*);

		//! Get shared drawing context with option to set color.
		//! Will set PixelFunction to Invert if modifying, Copy if not.
		MGD::CONTEXT* GetDrawingContext (
			STEREOVIEW StereoView,							//!< Stereo view, ignored unless stereo enabled for both canvas and tool
			const COLOR *pColor,								//!< Color, NULL to leave unchanged, not used if user is "modifying" tool
			bool UsePixmap = false							//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Get shared drawing context.
		//! Will set PixelFunction to Invert if modifying, Copy if not.
		MGD::CONTEXT* GetDrawingContext (
			STEREOVIEW StereoView = STEREOVIEW_None,	//!< Stereo view, ignored unless stereo enabled for both canvas and tool
			bool UsePixmap = false							//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		CHANGETYPE GetChangeType (
			) const { return (m_ChangeType); }
			
		//! Get the tool's manual base pointer.
		MANUAL_BASE* GetManualBase (
			) { return (m_pManualBase); }

		//! Get pen width used.
		INT8 GetPenWidth () const;

		//! Determine if tool's canvas has the pointer "captured".
		bool IsPointerCaptured () const;

		//! Determine if user is modifying tool.
		bool IsUserModifying (
			) const { return (m_IsUserModifying); }

		//! Restore background for entire tool canvas window.
		void RestoreBackground (
			bool UsePixmap = false					//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Restore background for specified rectangle.
		void RestoreBackground (
			const LRECT2D& rect,						//!< Rectangle in 'canvas' coordinates to restore
			bool AddLineWidth = true,				//!< Expand rectangle by 'LineWidth+1' to allow for wide lines
			bool UsePixmap = false					//!< Use canvas Pixmap if has CAPABILITY_UsePixmap and canvas created with both Memory and Pixmap buffers
			) const;

		//! Set whether to automatically set PixelFunction in GetDrawingContext.
		//! Default is to automatically set.
		void SetAutoPixelFunction (
			bool AutoPixelFunction
			) { m_AutoPixelFunction = AutoPixelFunction; }

		//! Set mouse cursor.
		void SetCursor (
			CURSORID cursorid
			) const { m_pCanvas->SetCursor(cursorid); }

		//! Set whether current position values are valid.
		void SetHasPosition (
			bool PosnValid = true					//!< True if current position is valid, false if not
			) { m_PosnSet = PosnValid; }

		//! Set pixel function on drawing context(s).
		//! Used if tool manages pixel function itself.
		void SetPixelFunction (
			MGD::PIXELFUNC PixelFunc				//!< Pixel function to set
			);

		//! Transform DPOINT2D from "tool" to "window" coordinates.
		LPOINT2D TransToolToWindow (
			const DPOINT2D& ptTool
			) const;

		//! Transform DPOINT2D from "tool" to "window" coordinates with no Z value.
		void TransToolToWindow (
			const DPOINT2D& ptTool,
			DPOINT2D& ptCanvas
			) const { ptCanvas = m_ToolToWindow.ConvertForward(ptTool); }

		//! Transform DPOINT2D from "tool" to "window" coordinates with Z value.
		//! @return true if obtained Z value from Z-view, false if not.
		bool TransToolToWindow (
			const DPOINT2D& ptTool,					//!< Point in tool coordinates
			DPOINT3D& ptCanvas						//!< Point in canvas coordinates returned
			) const;

		//! Transform DPOINT3D from "tool" to "window" coordinates with Z value.
		//! @return false if using Z value from Z-view and no value available, otherwise returns true.
		bool TransToolToWindow (
			const DPOINT3D& ptTool,					//!< Point in tool coordinates
			DPOINT3D& ptCanvas						//!< Point in canvas coordinates returned
			) const;

		//! Transform MGUI::POINT point in "window" to "tool" coordinates.
		DPOINT2D TransWindowToTool (
			MGUI::POINT point
			) const { return (m_ToolToWindow.ConvertInverse(point.m_x,point.m_y)); }

		//! Transform LPOINT2D point in "window" to "tool" coordinates.
		DPOINT2D TransWindowToTool (
			const LPOINT2D& point
			) const { return (m_ToolToWindow.ConvertInverse(point.x,point.y)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class TOOLDLG;

		// Static members
		static COLOR s_PenColor[PENCOLOR_COUNT];
		static INT8 s_PenWidthShared;

		// Explicitly initialized
		TOOLCANVAS *m_pCanvas;
		MGD::CONTEXT *XXXdelete_dc;
		MANUAL_BASE *m_pManualBase;
		TOOLDLG *m_pToolDlg;
		SNAPGRID *m_pSnapGrid;					// Snap grid information structure
		DPOINT2D m_ArrowDelta;					// Increments for arrow key movement, tool coords
		INT32 m_SuspendCount;					// Number of times that this tool has been suspended
		CONTROLNEED m_ControlNeed;				// Need for controls
		CHANGETYPE m_ChangeType;				// Change type for use with button-up
		KEYSTATE m_KeyState;						// Key state for access in OnChange
		CTRLOPT m_ctrlopt;
		CAPABILITY m_capability;
		INT8 m_PenWidth;							// Tool pen width, < 0 to use shared value (default)
		mutable bool m_InRestoreBG;
		bool m_InDeactivateFunc;				// Deactivate() is called, avoid doing redraws
		bool m_IsActive;							// Tool is in use, Activate() has been called
		bool m_PosnSet;							// Tool graphic has a position and is viewable
		bool m_UseSnapGrid;						// Use snap grid
		bool m_HasArrowDeltas;					// Deltas for arrow key movement is set
		bool m_IsUserModifying;					// Is user modifying tool position
		bool m_CanStartNew;						// User can start drawing new tool, default=true
		bool m_Resizable;							// Tool is resizable, default=true
		bool m_Rotatable;							// Tool is rotatable, default=true
		bool m_StereoEnabled;
		bool m_Is3D;								// Tool has own 3D coordinates
		bool m_AutoPixelFunction;
		bool m_InDrawTool;
		bool m_NotifyScrollWheel;
		// Self initializing
		DELEGATE_VOID_NOPARMS m_DelegateOnDestroy;
		DELEGATE_ONCHANGE m_DelegateOnChange;
		DELEGATE_ONPOINTER m_DelegateOnRightButtonDown;
		DELEGATE_ONPOINTER m_DelegateOnRightButtonUp;
		TRANS2D_AFFINE m_ToolToWindow;		// Tool \<--\> Window transformation
		TRANS2D_MAPGEN m_ToolToMapCRS;		// Tool \<--\> MapCRS transformation

		void CalcMarkerRectCanvas (const DPOINT3D& ptCanvas, UINT16 size, LRECT2D& rectCanvas, bool AddLineWidth, bool HasValidZ) const;
		void DrawMarkerLow (MGD::CONTEXT*, const DPOINT2D& ptCanvasCenter, const DPOINT2D& ptCanvasDir, MARKER marker, UINT16 size) const;
		void DrawTool ();
		void DrawTool (UINT32);
		bool OnKey (MGUI::POINT point, MGUI::KEYCODE, KEYSTATE);
		bool OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		bool OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		void SetCanvas (TOOLCANVAS *canvas) { m_pCanvas = canvas; }

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Create "form" with tool-specific controls.
		//! @return Pointer to FORM or NULL if tool has no "control" form.
		virtual MGUI::FORM* v_CreateControlForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			CTRLOPT ctrlopt							//!< Control options
			);

		//! Create "MGUI::TOOL_BASE::MANUAL_BASE" with tool-specific manual-entry controls.
		//! @return Pointer to MANUAL_BASE or NULL if tool has no "manual entry" form.
		virtual MANUAL_BASE* v_CreateManualBase (
			);

		//! Do actual drawing of tool graphic.
		virtual void v_DoDraw ();

		//! Move the tool.
		virtual void v_DoMove (double x, double y, ADJUSTFLAGS);

		//! Size the tool.
		virtual void v_DoSize (double x, double y, ADJUSTFLAGS);

		//! Generate region from tool.
		//! @return TRUE if region generated, FALSE if not, < 0 if error.
		virtual int v_GenerateRegion (
			REGION2D& region							//!< Generated region in 'tool' coordinates returned
			) const;

		//! Called to get title of tool control dialog.
		virtual MISTRING v_GetDialogTitle (
			) const;

		//! Called when tool is "activated".
		virtual void v_OnActivate ();

		//! Called when tool is cleared.
		virtual void v_OnClear ();

		//! Called when tool is deactivated.
		virtual void v_OnDeactivate ();

		//! Called when key pressed/released.
		//! @return 'true' if key handled, 'false' if not.
		//! Should not be called directly, called by OnKey.
		virtual bool v_OnKey (MGUI::POINT point, MGUI::KEYCODE, KEYSTATE);

		//! Called when change made to tool position, shape, size, etc.
		virtual void v_OnNotifyChange (
			CHANGESEQ changeseq,						//!< Sequence
			CHANGETYPE changetype					//!< Type
			);

		//! Called when pointer button pressed.
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate) = 0;

		//! Called when pointer button released.
		virtual bool v_OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);

		//! Called when pointer enters tool canvas.
		virtual void v_OnPointerEnter ();

		//! Called when pointer leaves tool canvas.
		virtual void v_OnPointerLeave ();

		//! Called when pointer is moved.
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);

		//! Called when tool is resumed, just before drawing.
		virtual void v_OnResume ();

		//! Called when 3D setting changed.
		virtual void v_OnSet3D ();

		//! Called when tool to map transformation is set.
		virtual void v_OnSetTransToMap ();

		//! Called when tool to window transformation is set.
		virtual void v_OnSetTransToWindow ();

		//! Called when canvas is resized.
		virtual void v_OnSizeCanvas ();

		//! Called when tool is suspended, just after erasing.
		virtual void v_OnSuspend ();

	friend class TOOLCANVAS::PRIV;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(TOOL_BASE::ADJUSTFLAGS)
DEFINE_ENUM_OP_BITWISE(TOOL_BASE::CTRLOPT)
#endif

//=====================================================================================================================

//! Tool with extents limit support.
class TOOL_LIMITED : public TOOL_BASE {
	public:

		//! Destructor
		virtual ~TOOL_LIMITED ();

		//! Set coordinate limits.
		void SetLimits (
			const DRECT2D& rect							//!< Limits to set
			);

		//! Set whether limits are enforced.
		//! If SetLimits() has not been called then the tool will be limited to the visible canvas extent.
		void SetLimitsEnforced (
			bool enforced = true							//!< Set if limits are enforced
			);

	protected:

		//! Constructor
		explicit TOOL_LIMITED (
			TOOLCANVAS *canvas,
			CONTROLNEED ControlNeed = CONTROLNEED_None,	//!< Need for control dialog or form
			CAPABILITY capability = CAPABILITY_Default	//!< Additional capabilities
			);

		//! Get limits.
		const DRECT2D& GetLimits (
			) const { return (m_Limits); }

		//! Determine if limits are enforced.
		bool IsLimitsEnforced (
			) const { return (m_LimitsEnforced); }

		//! Called when tool is "activated".
		//! Derived class must call base class method BEFORE performing own processing.
		virtual void v_OnActivate ();

		//! Called when tool to window transformation is set.
		//! Derived class must call base class method BEFORE performing own processing.
		virtual void v_OnSetTransToWindow ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef TOOL_BASE BASECLASS;

		bool m_LimitsEnforced;					//!< Limits are enforced
		bool m_LimitsSet;							//!< Limits have been set, otherwise defaults to window extents
		DRECT2D m_Limits;							//!< Position ranges, default to entire parent window range

		// TOOL_BASE overrides.
		virtual void v_OnSizeCanvas ();

		// Called when limits are changed.
		virtual void v_OnChangeLimits ();

		void UpdateLimits ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLBASE_H

