/**
 * \file <mi32/color.h>
 * \brief Definitions for COLOR structure and functions
 *
 * \if NODOC
 * $Id: color.h_v 1.74 2012/03/05 21:12:47 mju Exp $
 *
 * $Log: color.h_v $
 * Revision 1.74  2012/03/05 21:12:47  mju
 * Add getRGBA.
 *
 * Revision 1.73  2011/11/14 13:47:23  mju
 * Add setColorABGR/RGB32.
 *
 * Revision 1.72  2011/08/15 19:49:41  mju
 * Add model enum and getModel method.
 *
 * Revision 1.71  2011/08/15 19:12:01  mju
 * Add setColor and deprecate corresponding setRGB method.
 *
 * Revision 1.70  2011/08/12 21:26:45  mju
 * Generalize member names and use union so can access in different ways based on color model.
 * Remove getBlue/Green/Red methods.
 * Add getColor0/ABGR/RGB and setColorXBGR/RGB.
 * Change "invisible" flag to legacy and remove specific checks for in get... methods.
 *
 * Revision 1.67  2011/07/08 21:36:20  mju
 * Make most methods non-inline in preparation for supporting other color models.
 * Separate status/transparency members.
 *
 * Revision 1.66  2011/06/23 20:17:24  mju
 * Add getCMY/K.
 *
 * Revision 1.65  2010/06/02 14:59:02  mju
 * Add capability enum and iniRead/Write methods.
 *
 * Revision 1.64  2009/10/30 18:32:43  bcolombe
 * Added INTERFACE colors RED, YELLOW, GREEN, CYAN
 *
 * Revision 1.63  2009/09/02 20:45:41  mju
 * Optimizatino to check status_invisible before status_NonRGB as usually not set.
 *
 * Revision 1.62  2007/09/27 20:42:45  mju
 * Fix brown, had red/blue swapped.
 *
 * Revision 1.57  2006/10/05 20:36:58  mju
 * Move iniRead/Write here from inidefns.
 *
 * Revision 1.55  2004/09/14 16:31:32  dwilliss
 * Define some new color names (darker versions)
 *
 * Revision 1.53  2004/03/03 15:56:38  mju
 * Use specified range in setcmyk when eventually set RGB.
 *
 * Revision 1.47  2003/05/22 21:36:10  scowan
 * Removed assignment operator from name string.
 *
 * Revision 1.44  2003/05/21 22:39:44  dwilliss
 * Added support for non-rgb colorspace storage
 *
 * Revision 1.34  2002/11/14 23:02:23  mju
 * Change SetRGB from COLOR to use Get methods so will do colorspace conversione when implemented.
 *
 * Revision 1.33  2002/11/14 22:58:13  mju
 * Make GetRed/Green/Blue have separate implemention for non-range for efficiency.
 * Add STATUS_NonRGB for non-RGB colorspace storage.
 *
 * Revision 1.28  2002/10/23 17:48:39  mju
 * Add IsReserved/SetReserved methods.
 *
 * Revision 1.22  2002/10/21 20:46:16  mju
 * Add Clear() and HasValue() methods.
 * Check appropriate flags in IsEqualTo method.
 *
 * Revision 1.21  2002/10/21 20:24:44  mju
 * Add SetByName and use in ctor/op=.
 *
 * Revision 1.20  2002/10/21 16:53:35  mju
 * Deprecate access to COLOR data members.
 * Add various Get methods for accessing data members.
 * Add SetRGB method using range.
 *
 * Revision 1.1  1999/04/28  14:08:58  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_COLOR_H
#define	INC_MI32_COLOR_H

#ifndef	INC_MI32_INIBASE_H
#include <mi32/inibase.h>
#endif

#ifndef	INC_MI32_COLORSPC_H
#include <mi32/colorspc.h>
#endif

#ifndef	INC_MI32_MEMBUF_H
#include <mi32/membuf.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class COLOR_ARRAY;
#endif

//----------------------------------------------------------------------------
//		Constants
//----------------------------------------------------------------------------

#define	STD64STEP		21845		//!< Step value for creating 64 "standard" colors (65535 / 3)
#define	NUMSTDCOLORS	80			//!< 64 color + 16 gray

//===================================================================================================================
//! Color "model" enumeration.
// Values are passed to DLLs and stored in files and cannot be changed.
// Values must be assigned contiguously and may not exceed 127
enum COLORMODEL {
	COLORMODEL_RGB =				0,		//!< Red-Green-Blue
	COLORMODEL_CMY =				1,		//!< Cyan-Magenta-Yellow
	COLORMODEL_CMYK =				2,		//!< Cyan-Magenta-Yellow-Black
	COLORMODEL_COUNT
	};

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//===================================================================================================================

//! Structure to contain single color specification with possible transparency.
//!
//! This structure is stored in files and will never change in size.
struct CLASSLIBEXPORT COLOR {
	public:

		//! Enumerated color names.
		// Note, names are equal to COLORREF values to simplify assignment.
		enum NAME {
			//             		  xxBBGGRR
			BLACK =					0x00000000,
			RED =						0x000000FF,
			GREEN =					0x0000FF00,
			BLUE =					0x00FF0000,
			CYAN =					0x00FFFF00,
			MAGENTA =				0x00FF00FF,
			YELLOW =					0x0000FFFF,
			GRAY25 =					0x003F3F3F,
			GRAY50 =					0x007F7F7F,
			GRAY75 =					0x00BFBFBF,
			GRAY =					GRAY50,
			ORANGE =					0x0000A5FF,
			BROWN =					0x002A2AA5,
			PURPLE =					0x00A020F0,
			DARKRED =				0x00000080,
			DARKGREEN =				0x00008000,
			DARKBLUE =				0x00800000,
			DARKCYAN =				0x00808000,
			DARKMAGENTA =			0x00A000A0,
			DARKYELLOW =			0x00008080,
			FORESTGREEN =			0x00228B22,
			WHITE =					0x00FFFFFF,
			INTERFACE_RED =		0x009C9CFF,
			INTERFACE_YELLOW =	0x009CFFFF,
			INTERFACE_GREEN =		0x009CFF9C,
			INTERFACE_CYAN =		0x00FFFF9C
			};

		//! Color "model" enumeration.
		// Values are stored in files and cannot be changed.
		// Values must be assigned contiguously and may not exceed 127
		enum MODEL {
			MODEL_RGB =				0,		//!< Red-Green-Blue
			MODEL_CMY =				1,		//!< Cyan-Magenta-Yellow
			MODEL_CMYK =			2,		//!< Cyan-Magenta-Yellow-Black
			MODEL_COUNT
			};

		//! Color capabilities.
		enum CAPABILITY {
			CAPABILITY_NoTransparency =		0,		//!< Don't allow any transparency
			CAPABILITY_AllowTransparent =		1,		//!< Allow fully transparent, but not partial
			CAPABILITY_AllowTransparency =	2,		//!< Allow both full and partial transparency
			CAPABILITY_AsPrevious =				3		//!< Use previously specified capability
			};

		//! Default constructor.
		COLOR (
			) : 
			m_u64(0)
			{ }

		//! Construct with specified red, green, blue values.
		COLOR (
			UINT16 r16,
			UINT16 g16,
			UINT16 b16
			):
			m_u64(0)
			{ SetRGB(r16,g16,b16); }

		//! Construct by color reference.
		explicit COLOR (
			UINT32 ColorRef
			):
			m_u64(0)
			{ SetColorRef(ColorRef); }

		//! Construct by enumerated name.
		COLOR (
			NAME name
			) :
			m_u64(0)
			{ SetColorXBGR32(name); }

		//! Assign by enumerated name.
		//! Original transparency values are retained.
		COLOR& operator= (
			COLOR::NAME name
			) { SetColorXBGR32(name); return (*this); }

		//! Blend this color with another color.
		//! The range of values for alpha is 0-255.  
		void Blend (
			const COLOR& other,
			UINT8 alpha = 128		//!< 0 == all "this", 255 = all "other"
			);
		
		//! Clear current color setting.
		void Clear (
			) { m_u64 = 0; }

		//! Get values in specified COLORSPACE.
		//! For colorspaces containing an "alpha" component, "alpha" is considered to be "opacity".
		//! Binary, grayscale and indexed colorspaces are not supported and an error will be returned for such.
		ERRVALUE GetByColorSpace (
			COLORSPACE colorspace,				//!< Color space
			void *values							//!< Buffer to hold returned values, size depends on colorspace
			) const;

		//! Get as Cyan-Magenta-Yellow 8-bit values in range (0-255).
		void GetCMY (
			UINT8& c8,			//!< Cyan returned (0-255)
			UINT8& m8,			//!< Magenta returned (0-255)
			UINT8& y8			//!< Yellow returned (0-255)
			) const;

		//! Get as Cyan-Magenta-Yellow 16-bit values in range (0-UINT16_MAX).
		void GetCMY (
			UINT16& c16,		//!< Cyan returned
			UINT16& m16,		//!< Magenta returned
			UINT16& y16			//!< Yellow returned
			) const;

		//! Get as Cyan-Magenta-Yellow-Black 8-bit values in range (0-255).
		void GetCMYK (
			UINT8& c8,			//!< Cyan returned (0-255)
			UINT8& m8,			//!< Magenta returned (0-255)
			UINT8& y8,			//!< Yellow returned (0-255)
			UINT8& k8			//!< Black returned (0-255)
			) const;

		//! Get as Cyan-Magenta-Yellow-Black 16-bit values in range (0-UINT16_MAX).
		void GetCMYK (
			UINT16& c16,		//!< Cyan returned
			UINT16& m16,		//!< Magenta returned
			UINT16& y16,		//!< Yellow returned
			UINT16& k16			//!< Black returned
			) const;

		//! Get color as (0x00BBGGRR) UINT32.
		//! @return Color value with red in lowest bits.
		UINT32 GetColor0BGR32 (
			) const;

		//! Get color as (0x00RRGGBB) UINT32.
		//! @return Color value with blue in lowest bits.
		UINT32 GetColor0RGB32 (
			) const;

		//! Get color as (0xAABBGGRR) UINT32.
		//! @return Color value with red in lowest bits.
		UINT32 GetColorABGR32 (
			) const;

		//! Get color as (0xAARRGGBB) UINT32.
		//! @return Color value with blue in lowest bits.
		UINT32 GetColorARGB32 (
			) const;

		//! Get color model.
		DEPRECATED COLORMODEL GetColorModel (
			) const;

		//! Get WIN32 color reference (0x00BBGGRR).
		UINT32 GetColorRef (
			) const { return (GetColor0BGR32()); }

		//! Get array of values to store in Ini file.
		void GetIniValues (
			UINT16 values[4]
			) const;

		//! Get color model.
		MODEL GetModel (
			) const;

		//! Get opacity value (0-255).
		UINT8 GetOpacity (
			) const { return (UINT8_MAX - GetTransparency()); }

		//! Get Red, Green, Blue values as UINT8 values.
		void GetRGB (
			UINT8& r8,
			UINT8& g8,
			UINT8& b8
			) const;

		//! Get Red, Green, Blue values as UINT8 array.
		void GetRGB (
			UINT8 *rgb		//<! Red/Green/Blue values returned
			) const { GetRGB(rgb[0],rgb[1],rgb[2]); }

		//! Get Red, Green, Blue values as UINT16 values.
		void GetRGB (
			UINT16& r16,
			UINT16& g16,
			UINT16& b16
			) const;

		//! Get Red, Green, Blue values as UINT16 array.
		void GetRGB (
			UINT16 *rgb		//<! Red/Green/Blue values returned
			) const { GetRGB(rgb[0],rgb[1],rgb[2]); }

		//! Get Red, Green, Blue, Alpha values as float values in range (0-1).
		void GetRGBA (
			float& r,		//!< Red returned in range 0-1
			float& g,		//!< Green returned in range 0-1
			float& b,		//!< Blue returned in range 0-1
			float& a			//!< Alpha returned in range 0-1
			) const;

		//! Retrieve transparency value.
		//! @return Transparency (0 - 255), 0 = opaque, 255 = 100% transparent.
		UINT8 GetTransparency (
			) const { return (m_u8[I_transparency]); }

		//! Determine if has any transparency value.
		bool HasTransparency (
			) const { return (m_u8[I_transparency] != 0); }

		//! Determine if a color value has been set.
		bool HasValue (
			) const { return (m_u64 != 0); }

		//! Read value from INI file.
		bool IniRead (
			INIHANDLE hdl,
			const char *group,
			const char *field
			);

		//! Write value to INI file.
		void IniWrite (
			INIHANDLE hdl,
			const char *group,
			const char *field
			) const;

		//! Compare colors for equality, excluding transparency settings.
		bool IsEqualNoTransp (
			const COLOR& rhs		
			) const;

		//! Compare colors for equality, including transparency, etc.
		bool IsEqualTo (
			const COLOR& rhs		
			) const { return (m_u64 == rhs.m_u64); }

		//! Determine if the color has been run through an ICM transformation.
		//! This is used to insure that the same color isn't translated more
		//! than once.
		bool IsICMCorrected (
			) const {
			return ((m_u8[I_status] & (STATUS_NonRGB | STATUS_ICMCorrected)) != 0);
			}

		//! Determine if color is 'invisible' (100% transparent).
		bool IsInvisible (
			) const {
			return (m_u8[I_transparency] == UINT8_MAX);
			}

		//! Determine if color is 'reserved' for application-specific use.
		//! In color search and allocation operations 'reserved' colors are not used or changed.
		bool IsReserved (
			) const { 
			return ((m_u8[I_status] & STATUS_Reserved) && !(m_u8[I_status] & STATUS_NonRGB));
			}

		//! Get as string for serializer writing.
		void SerialGetString (
			char *string							//!< String to contain values, should be at least 32 bytes
			) const;

		//! Set from string for serializer reading.
		void SerialSetString (
			const char *string					//!< String to set from
			);

		//! Set values using specified COLORSPACE.
		//! For colorspaces containing an "alpha" component, "alpha" is considered to be "opacity".
		//! Binary, grayscale and indexed colorspaces are not supported and an error will be returned for such.
		ERRVALUE SetByColorSpace (
			COLORSPACE colorspace,				//!< Color space
			const void *values					//!< Buffer holding values to set, size depends on colorspace
			);

		//! Set color by name string.
		//! @return 'true' if name found, 'false' if not.
		//! Also supports strings in forms "redpct greenpct bluepct" and "redpct,greenpct,bluepct".
		bool SetByName (
			const char *string					//!< Color name, case-insensitive
			);

		//! Set from Cyan, Magenta, Yellow in specified range.
		void SetCMY (
			int cyan,
			int magenta,
			int yellow,
			int range								//!< Range, 1-65535
			);

		//! Set from Cyan, Magenta, Yellow as UINT8.
		void SetCMY8 (
			UINT8 c8,								//!< Cyan 0-255
			UINT8 m8,								//!< Magenta 0-255
			UINT8 y8									//!< Yellow 0-255
			);

		//! Set from Cyan, Magenta, Yellow, Black in specified range.
		void SetCMYK (
			int cyan,
			int magenta,
			int yellow,
			int black,
			int range								//!< Range, 1-65535
			);

		//! Set from Cyan, Magenta, Yellow, Black as UINT8.
		void SetCMYK8 (
			UINT8 c8,								//!< Cyan 0-255
			UINT8 m8,								//!< Magenta 0-255
			UINT8 y8,								//!< Yellow 0-255
			UINT8 k8									//!< Black 0-255
			);

		//! Set from COLOR, retaining original transparency value.
		//! Model will be changed to match specified color.
		void SetColor (
			const COLOR& color					//!< Color value to set
			);

		//! Set from color value as (0xAABBGGRR) UINT32.
		//! Original flag values are retained, model will be changed to RGB.
		void SetColorABGR32 (
			UINT32 abgr32							//!< Color and opacity value to set
			);

		//! Set from color value as (0xAARRGGBB) UINT32.
		//! Original flag values are retained, model will be changed to RGB.
		void SetColorARGB32 (
			UINT32 argb32							//!< Color and opacity value to set
			);

		//! Set from color value as (0x??BBGGRR) UINT32.
		//! Original flag and transparency values are retained, model will be changed to RGB.
		void SetColorXBGR32 (
			UINT32 xbgr32							//!< Color value to set
			);

		//! Set from color value as (0x??RRGGBB) UINT32.
		//! Original flag and transparency values are retained, model will be changed to RGB.
		void SetColorXRGB32 (
			UINT32 xrgb32							//!< Color value to set
			);

		//! Set red, green, blue values from WIN32 color reference (0x00BBGGRR).
		//! Original flag and transparency values are retained, model will be changed to RGB.
		void SetColorRef (
			UINT32 ColorRef
			) { SetColorXBGR32(ColorRef); }

		//! Set from array of values read from Ini file.
		void SetIniValues (
			const UINT16 values[4]
			);

		//! Specify whether the color has been run through an ICM transformation.
		//! This is used to insure that the same color isn't translated more
		//! than once.
		//! Of course there's no way to specify which target profile it's been
		//! corrected to.  ICM-corrected colors should never be stored in files
		//! unless the whole file is corrected to some target device profile.
		void SetIsICMCorrected (
			bool bCorrected
			);

		//! Set opacity value.
		void SetOpacity (
			UINT8 opacity							//!< Value to set, 0 = invisible, 255 = 100% opaque
			);

		//! Set whether color is 'reserved' or not.
		void SetReserved (
			bool reserved = true					//!< 'true' to make reserved, 'false' to not
			);

		//! Set red, green, blue values in range 0-65535.
		//! Original flag and transparency values are retained.
		void SetRGB (
			UINT16 red,
			UINT16 green,
			UINT16 blue
			);

		//! Set red, green, blue values from COLOR.
		//! Original flag and transparency values are retained.
		DEPRECATED_MSG("Deprecated, use SetColor method instead.") void SetRGB (
			const COLOR& color
			);

		//! Set from Red, Green, Blue in specified range.
		//! Values outside the range will be limited to the range specified.
		void SetRGB (
			int red,								//!< Red in range specified
			int green,							//!< Green in range specified
			int blue,							//!< Blue in range specified
			int range							//!< Range, 1-65535
			);

		//! Set red, green, blue values from UINT8.
		//! Original flag and transparency values are retained.
		void SetRGB8 (
			UINT8 r8,
			UINT8 g8,
			UINT8 b8
			);

		//! Set transparency value.
		void SetTransparency (
			UINT8 transparency					//!< Value to set, 0 = opaque, 255 = 100% transparent
			);

		//! Perform byte-swapping.
		void SwapBytes ()
			{ ::SwapBytes(m_u16, 4); }

		//! Validate status for some legacy stored color instances.
		void Validate (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Index into u8 array overlapped onto u16 array.
		enum I_U8 {
	#ifdef BYTEORDER_LoHi
			I_Lo0 =				0,
			I_Hi0 =				1,
			I_Lo1 =				2,
			I_Hi1 =				3,
			I_Lo2 =				4,
			I_Hi2 =				5,
			I_status =			6,
			I_transparency =	7,
	#else
			I_Hi0 =				0,
			I_Lo0 =				1,
			I_Hi1 =				2,
			I_Lo1 =				3,
			I_Hi2 =				4,
			I_Lo2 =				5,
			I_transparency =	6,
			I_status =			7,
	#endif
			// RGB16 color model
			RGB16_R8 = I_Hi0,
			RGB16_G8 = I_Hi1,
			RGB16_B8 = I_Hi2,
			RGB16_R8L = I_Lo0,
			RGB16_G8L = I_Lo1,
			RGB16_B8L = I_Lo2,
			// CMY16 color model
			CMY16_C8 = I_Hi0,
			CMY16_M8 = I_Hi1,
			CMY16_Y8 = I_Hi2,
			CMY16_C8L = I_Lo0,
			CMY16_M8L = I_Lo1,
			CMY16_Y8L = I_Lo2,
			// CMYK12 color model
			CMYK12_C8 = I_Lo0,
			CMYK12_M8 = I_Hi0,
			CMYK12_Y8 = I_Lo1,
			CMYK12_K8 = I_Hi1,
			CMYK12_CM4L = I_Lo2,
			CMYK12_YK4L = I_Hi2,
			};

		// Index values for m_u16 array.
		enum I_U16 {
			RGB16_R = 0,
			RGB16_G = 1,
			RGB16_B = 2,
			CMY16_C = 0,
			CMY16_M = 1,
			CMY16_Y = 2,
			};

		enum STATUS {
			STATUS_ValueSet =			0x01,
			LEGACY_Writable =			0x02,
			STATUS_Reserved =			0x04,
			LEGACY_Invisible =		0x08,
			LEGACY_Stored =			0x10,
			STATUS_ICMCorrected =	0x20,	// ICM correction already done. Note, NonRGB implies ICM's already been done too
			STATUS_NonRGB =			0x80	// If set, all other bits are used to hold MODEL
			};

		union {
			UINT64 m_u64;
			UINT16 m_u16[4];
			UINT8 m_u8[8];
			};

		CHECKSIZE(8);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Compare COLORs for equality.
inline bool operator== (
	const COLOR& lhs,
	const COLOR& rhs
	) {
	return (lhs.IsEqualTo(rhs));
	}


//! Compare COLORs for inequality.
inline bool operator!= (
	const COLOR& lhs,
	const COLOR& rhs
	) {
	return (!lhs.IsEqualTo(rhs));
	}


//! Read 'COLOR' value from INI file.
inline int IniRead (INIHANDLE hdl, const char *group, const char *field, COLOR& value) { return (_iniRead(hdl,group,field,INITYPE_COLOR,&value,1)); }

//! Read 'COLOR' value from INI file with default.
inline int IniRead (INIHANDLE hdl, const char *group, const char *field, COLOR& value, const COLOR& dft) { value = dft; return (IniRead(hdl,group,field,value)); }

//! Write 'COLOR' value to INI file.
inline int IniWrite (INIHANDLE hdl, const char *group, const char *field, const COLOR& value) { return (_iniWrite(hdl,group,field,INITYPE_COLOR,&value,1)); }

//! Perform byte-swapping on COLOR.
inline void SwapBytes (
	COLOR& color
	) {
	color.SwapBytes();
	}

//===================================================================================================================

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_COLOR_H
