/**
 * \file colorbal.h <mi32/colorbal.h>
 * \brief Header file for color balance tools
 *
 * \if NODOC
 * $Id: colorbal.h_v 1.43 2012/01/11 21:33:01 mju Exp $
 *
 * $Log: colorbal.h_v $
 * Revision 1.43  2012/01/11 21:33:01  mju
 * Remove unused prototypes.
 *
 * Revision 1.42  2011/10/21 20:56:36  mju
 * Remove redundant hdr.
 *
 * Revision 1.41  2007/04/09 17:48:12  dwilliss
 * Changed printer gamma structure
 *
 * Revision 1.40  2006/09/21 17:47:53  mju
 * Don't use mxdialog hdr.
 *
 * Revision 1.39  2006/09/15 22:04:54  dwilliss
 * include memory.h to get memset()
 *
 * Revision 1.38  2006/09/15 15:25:08  mju
 * Serializer changes.
 *
 * Revision 1.37  2006/07/12 19:58:05  mju
 * Fix doxy comment.
 *
 * Revision 1.36  2005/07/07 16:18:05  scowan
 * Removed forward decl of textinfo.
 *
 * Revision 1.35  2005/07/06 16:44:47  scowan
 * Removed read and write color bal functions, now handled by interface class.
 *
 * Revision 1.34  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.33  2003/08/05 15:21:18  dwilliss
 * Gave CBPARMS a constructor/destructor
 *
 * Revision 1.32  2002/06/10 17:19:37  dwilliss
 * Changed dialog function to take an MxRELPOSN instead of int
 *
 * Revision 1.31  2000/05/19 19:28:43  scowan
 * *** empty log message ***
 *
 * Revision 1.30  2000/05/19 17:12:44  sparsons
 * Documentation.
 *
 * Revision 1.28  1999/12/02 23:24:18  mju
 * Add GetSerialItemDef().
 *
 * Revision 1.26  1999/08/26 13:19:29  mju
 * Use INT8 instead of "SINT8" so it compiles.
 *
 * Revision 1.25  1999/08/25  22:41:06  dwilliss
 * Added a ForceGray flag to CBPARMS
 *
 * Revision 1.24  1999/08/25 22:00:10  dwilliss
 * Added a ForceGray flag
 *
 * Revision 1.20  1999/05/26 13:45:33  mju
 * Don't define Mxd prototypes in win32-native mode.
 *
 * Revision 1.19  1999/05/07  21:15:41  mju
 * Header restruct.
 *
 * Revision 1.18  1999/01/28  17:07:07  dwilliss
 * Found out how to declare a pointer to a member function
 *
 * Revision 1.17  1999/01/26 18:12:55  dwilliss
 * Added COLORBALANCE class
 * \endif
**/

#ifndef INC_MI32_COLORBAL_H
#define INC_MI32_COLORBAL_H

#ifndef	INC_MI32_XBASE_H
	#include <mi32/xbase.h>
#endif

#ifndef INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif

#ifndef INC_MI32_SERIALIZERBASE_H
	#include <mi32/serializerbase.h>
#endif

#ifndef	INC_MEMORY_H
#define	INC_MEMORY_H
#include <memory.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct COLOR;
struct PRINTGAMMA;
#endif // GENERATING_DOXYGEN_OUTPUT

//----------------------------------------------------------------------------
//		CBPARMS Structure
//----------------------------------------------------------------------------

struct CBPARMS {

	CBPARMS() {
		memset(this, 0, sizeof(CBPARMS));
		}
	
	~CBPARMS() {}

	//! MEMBERS
	double spval[6];							//!<  color sliders (phase) 
	double saval[6];							//!<  color sliders (relative brightness) 
	double ssval[6];							//!<  color sliders (relative saturation) 
	double hsval[6];							//!<  color sliders (hue shift) 
	double int_val;							//!<  intensity 
	double con_val;							//!<  contrast 
	double sat_val;							//!<  saturation 
	UINT32 flags;

	//! Determine if is "identity".
	bool IsIdentity (void) const;

	//! Get serial item definitions.
	static const SERIALIZERITEM* GetSerialItemDef();
	};

//----------------------------------------------------------------------------
//		Constants for CBPARMS
//----------------------------------------------------------------------------

/* These are the flags for CBPARMS.  For the most part, you can */
/*	ignore them.  They tell what the 6 sliders at the top should */
/* be adjusting initially (for default) and also if the values  */
/* in the structure have been initialized. */

#define CBAL_ValsSet				0x00000001
#define CBAL_AdjustHue			0x00000002
#define CBAL_AdjustInt			0x00000004
#define CBAL_AdjustSat			0x00000008
#define CBAL_AdjustHueShift	0x00000010
#define CBAL_AdjustMask  (CBAL_AdjustHue|CBAL_AdjustInt|CBAL_AdjustSat|CBAL_AdjustHueShift)
#define CBAL_ForceGray			0x00000020

//----------------------------------------------------------------------------
//!		MCBALCALLBACKSTRUCT Structure
//----------------------------------------------------------------------------

struct MCBALCALLBACKSTRUCT : MICALLBACKLIST::ANYCALLBACKSTRUCT {
	MCBALCALLBACKSTRUCT(int reason);

	//! MEMBERS
	MXDHANDLE dhandle;
	Widget ColorBalanceForm; 
	Widget retdialog;							//!<  So callback can tell us who they are 
	CBPARMS *cbal;			
	PRINTGAMMA *cont;
	int mode;
	};

class COLORBALANCE {
	public:
		
		// CONSTRUCTION / DESTRUCTION
		//! Default constructor.  Do nothing.
		COLORBALANCE (
			) {
			m_is_identity = 1; 
			}							

		//! Constructor with a CBPARMS.
		COLORBALANCE (
			const CBPARMS &p
			) { 
			SetParms(p); 
			}

		//! Simple destructor
		~COLORBALANCE() {}

		// METHODS

		//! Do color balancing on RGB buffers (3 buffers).
		void TransBuffers (
			UINT8 *rbuf,					 	//!< Buffer of red values
			UINT8 *gbuf, 						//!<	Buffer of green values
			UINT8 *bbuf, 						//!<	Buffer of blue values
			int count							//!<	Number of cells to convert
			) const;

		//! Do color balancing on RGB buffers (3 buffers).
		//!
		//! Overloaded version of this function which takes (int*) buffers but
		//!	still assumes values in range of 0-255, less efficient than above.
		void TransBuffers (
			int *rbuf, 							//!<	Buffer of red values
			int *gbuf, 							//!<	Buffer of green values
			int *bbuf, 							//!<	Buffer of blue values
			int count							//!< Number of cells to convert
			) const;	

		//! Translate one buffer with RGBRGBRGB... or RGBxRGBxRGBx...
		void TransRGB (
			UINT8 *buf, 
			int count, 
			int BytesPerPixel = 3
			) const; 	

		//! Translate an array or COLOR structures.
		void TransCOLORs (
			COLOR *buf, 
			int count
			) const;

		//! Set parameters.
		void SetParms (
			const CBPARMS &
			);

		//! Determine if is "identity".
		bool IsIdentity (
			) const { 
			return (m_is_identity); 
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// MEMBERS

		double m_htrans[360], m_div_255[256];
		UINT8 m_igamma[256]; 	//!< Gamma table to run intensity values through
		UINT8 m_sgamma[256];		//!< Gamma table to run saturation values through
		INT8 m_hi_adj[360];		//!< delta intensity for a given hue
		INT8 m_hs_adj[360];		//!< delta saturation for a given hue
		bool m_need_h;
		bool m_need_i;
		bool m_need_s;
		bool m_no_sat;
		bool m_is_identity;

		//! Should actually be a pointer to a member function either TransZeroSat or TransComplete.
		void (COLORBALANCE::*TransFunc) (
			UINT8 *rbuf,					 	//!< Buffer of red values
			UINT8 *gbuf, 						//!<	Buffer of green values
			UINT8 *bbuf, 						//!<	Buffer of blue values
			int count
			) const;

		//! Simplified (faster) translation if saturation is 0 (gray scale).
		void TransZeroSat (
			UINT8 *rbuf,					 	//!< Buffer of red values
			UINT8 *gbuf, 						//!<	Buffer of green values
			UINT8 *bbuf, 						//!<	Buffer of blue values
			int count
			) const;

		//! Complete method for color translation. 
		void TransComplete (
			UINT8 *rbuf,					 	//!< Buffer of red values
			UINT8 *gbuf, 						//!<	Buffer of green values
			UINT8 *bbuf, 						//!<	Buffer of blue values
			int count
			) const;

		//! Complete color translation which forces gray output.
		void TransForceGray (
			UINT8 *rbuf,					 	//!< Buffer of red values
			UINT8 *gbuf, 						//!<	Buffer of green values
			UINT8 *bbuf, 						//!<	Buffer of blue values
			int count
			) const;

		//! Helper function to compute gamma table.
		void ComputeGamma (
			UINT8 *table, 
			double gamma
			) const;

	#endif // GENERATING_DOXYGEN_OUTPUT
	};

//----------------------------------------------------------------------------
//!		Function prototypes
//----------------------------------------------------------------------------

extern "C" {

#ifdef X_NATIVE

//!	Create color balance form.
int MxColorBalanceForm (
	Widget parent, 
	CBPARMS *cp, 
	PRINTGAMMA *gamma, 
	Widget *pForm, 
	XTARGS& args,
	UINT32 flags
	);

//!	Set color balance form parameters.
int MxColorBalanceFormSetParms (
	Widget form, 
	CBPARMS *cp, 
	PRINTGAMMA *gamma
	);

//!	Set test callback for color balance form.
void MxColorBalanceFormSetTestCallback (
	Widget form,
	MICALLBACKLIST::CALLBACKFUNC func,
	void *cbdata
	);

//! Create color balance dialog.
//!
//! The dialog keeps track of the pointer you give it and modifies the structure
//!	directly.  If you pass NULL, it will edit an internal cbparms and you can use
//!	MxdColorBalanceGetCBParms()/MxdColorBalanceSetCBParms() to access it.
//!	The dialog always gets an "Apply..." button, which is automatically enabled/
//!	disabled by adding/removing an MxdCR_Apply callback.	Callbacks supported:
//!		MxdCR_Open		
//!		MxdCR_Close		
//!		MxdCR_Destroy		
//!		MxdCR_Apply			User hit apply button
//!		MxdCR_ChangeContinue		Called for drag callback on slider
//!		MxdCR_ChangeEnd			Slider change done.
int MxdColorBalanceCreate (
	Widget pwidget,							//!< parent widget 
	MXDHANDLE *dhandle, 						//!< Dialog handle returned
	char *dialogname,							//!< Dialog name returned 
	CBPARMS *cbparms,							//!< Parms to edit 
	UINT32 dialogflags						//!< MxdFLAG_... flags
	);

//!	Open previously created color balance dialog.
//!
//! If the MxdFLAG_ReturnWhenClose flag was specified when the dialog was created, this function will
//!	not return until the user closes the dialog.
//!	Using this function will call any callbacks selected by the MxdCR_Open reason.
int MxdColorBalanceOpen (
	MXDHANDLE dhandle							//!< Handle created by MxdColorBalanceCreate()
	);

//!	Close previously opened color balance dialog.
//!
//! Using this function will call any callbacks selected by the MxdCR_Close reason.
int MxdColorBalanceClose (
	MXDHANDLE dhandle							//!< Handle created by MxdColorBalanceCreate()
	);

//!	Destroy previously created color balance dialog.
//!
//! Using this function will call any callbacks selected by the MxdCR_Destroy reason.
int MxdColorBalanceDestroy (
	MXDHANDLE dhandle							//!< Handle created by MxdColorBalanceCreate()
	);

//! Get the color balance parmameters.
//!
//! @return the color balance parameters.
int MxdColorBalanceGetParms (
	MXDHANDLE dhandle,						//!< Dialog handle  
	CBPARMS *cbparms, 						//!< Parms to fill in
	UINT32 flags								//!< None yet.  Just for future expansion.
	);

//! Set the color balance parmameters.
//!
//! Note, if you passed a non-NULL pointer for cbparms to MxdColorBalanceCreate,
//!	this function will set the structure that pointer pointed to. 
int MxdColorBalanceSetParms (
	MXDHANDLE dhandle, 						//!< Dialog handle
	CBPARMS *cbparms, 						//!< Parms to use
	UINT32 flags								//!< None yet.  Just for future expansion.
	);

//!	Add callback for color balance dialog.
int MxdColorBalanceAddCallback (
	MXDHANDLE dhandle, 						//!< Handle created by MxdColorBalanceCreate()
	MICALLBACKLIST::CALLBACKFUNC cbfunc, 				//!< Callback function
	void *cbdata, 								//!< Callback "hook" data
	UINT32 reason								//!< Reasons for which cbfunc() should get called
	);

//!	Remove callback for color balance dialog.
int MxdColorBalanceRemoveCallback (
	MXDHANDLE dhandle, 						//!< Handle created by MxdColorBalanceCreate()
	MICALLBACKLIST::CALLBACKFUNC cbfunc, 				//!< Callback function
	void *cbdata, 								//!< Callback "hook" data, must be same as when added
	UINT32 reason								//!< Reasons for which cbfunc() should get called, must be same as when added
	);

#endif

//! This routine computes the lookup table for Hue.
void HISTransCompHueTable (
	short *htrans,								//!< Array of 360 integers 
	const double *sval, 						//!< Array of 6 slider values in the range -1.0 to 1.0
	const double *hsval
	);

//! Computes the lookup table for Hue/Amp.
void HISTransCompHueAmpTable (
	signed char *atrans, 
	const double *sval						//!< Array of 6 slider values in the range -1.0 to 1.0
	);

//!	Setup color balance conversion functions.
//!
//! This function sets up a color balance conversion handle using the given
//!	parameters.  The color balance functions do color conversion in-place and
//!	are optimized to do nothing if the color balance is "identity".
int McbalInit (
	CBPARMS *parms,							//!< Color balance parameters 
	void **handle								//!< Handle returned
	);

//! Free color balance conversion handle.
void McbalStop (
	void *handle								//!< Handle from McbalInit()
	);

//! Do color balancing on RGB buffers (3 buffers).
//!
//! Do color balance in-place on the given buffers.  If the color balance handle is
//!	set up such that nothing needs to be done, then nothing will be done.
void McbalTransBuffers (
	void *handle,								//!< Handle from McbalInit()
	UINT8 *rbuf, 								//!< Buffer of red values
	UINT8 *gbuf, 								//!< Buffer of green values
	UINT8 *bbuf, 								//!< Buffer of blue values
	int count									//!< Number of cells to convert
	);

//! Do color balancing on an array of COLORs.
//! Modifies the colors through the color balance handle.
void McbalTransCOLORs (
	void *handle, 								//!< Handle from McbalInit()
	COLOR *colors, 							//!<	Array of color values
	int count									//!<	Number of cells to convert
	);

//! Do color balancing on integer RGB buffers.
//!
//! Do color balance in-place on the given buffers.  If the color balance handle is
//!	set up such that nothing needs to be done, then nothing will be done.  This is
//!	the same as McbalTransBuffers() but the buffers are (int*) instead of (UBYTE *)
//!	I have no idea why this exists, but something must have used it at one time.
void McbalTransInt8 (
	void *handle, 								//!< Handle from McbalInit()
	int *rbuf, 									//!< Buffer of red values
	int *gbuf, 									//!< Buffer of green values
	int *bbuf, 									//!< Buffer of blue values
	int count									//!<	Number of cells to convert
	);

//!	This routine computes the lookup table for Hue.
void McbalCompHueTable (
	short *htrans, 							//!< Array of 360 integers
	double *sval, 								//!< Array of 6 slider values in the range -1.0 to 1.0
	double *hsval
	);

//!	Computes the lookup table for Hue/Amp.
void McbalCompHueAmpTable (
	signed char *atrans, 
	double *sval								//!< Array of 6 slider values in the range -1.0 to 1.0
	);

//! 
int McbalParmIsIdentity(CBPARMS *);

//! Check to see if a CBPARM is set to default settings.
//!
//! This returns true if the given color balance does nothing (all
//!	values set to default).  Note:  this is not a very fast check, so
//!	don't do it in a loop.  
int McbalIsIdentity(
	void *parms
	);

}	// extern "C"

#endif
