/**
 * \file mi32/datetime.h
 * \brief Definitions for date/time functions
 *
 * \if NODOC
 * $Id: datetime.h_v 1.49 2013/02/26 15:41:30 mju Exp $
 *
 * $Log: datetime.h_v $
 * Revision 1.49  2013/02/26 15:41:30  mju
 * Add getFormatted using char* format specifier.
 *
 * Revision 1.48  2011/10/24 20:31:45  mju
 * Fix dox and remove redundant hdr.
 *
 * Revision 1.47  2011/09/02 15:25:42  mju
 * Add format_yyyymmdd_hhmmss and make format_job equivalent.
 *
 * Revision 1.46  2009/10/29 16:12:22  mju
 * Add clear().
 *
 * Revision 1.45  2009/07/29 13:54:34  mju
 * Add isSet and isValid methods.
 *
 * Revision 1.44  2008/10/01 22:30:09  bcolombe
 * Added FORMAT_Job
 *
 * Revision 1.43  2007/01/17 23:30:42  dwilliss
 * Added a new format enum value
 *
 * Revision 1.42  2007/01/03 18:46:35  scowan
 * Changed names and added some code to setFormatted.
 *
 * Revision 1.38  2006/12/27 21:20:25  scowan
 * Added Get and Set datetimeformat methods.
 *
 * Revision 1.37  2005/03/31 16:57:10  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.36  2004/05/07 15:34:22  dwilliss
 * Added an enum for initial values and constructor to take one.  So you can
 * now construct a DATETIME and initialize it to local time all in one line.
 *
 * Revision 1.31  2003/12/02 23:11:53  dwilliss
 * Added a Set method to set from a string like srtptime()
 *
 * Revision 1.28  2003/03/13 17:41:53  dwilliss
 * Added methods to get/set julian date as a double with frational days
 *
 * Revision 1.15  2000/07/11 19:57:31  dwilliss
 * Added ConvertToLocal() and ConvertToUTC() methods
 *
 * Revision 1.11  2000/07/11 17:15:51  dwilliss
 * \endif
**/

#ifndef INC_MI32_DATETIME_H
#define INC_MI32_DATETIME_H

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_TIME_H
#include <time.h>
#define	INC_TIME_H
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
struct RVCDATETIME;
#ifdef WIN32
struct _SYSTEMTIME;
struct _FILETIME;
#endif
#endif


#if defined(MISYSTEMDLL)
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//! Container for date/time.
//! The date/time represented can be either UTC (a.k.a. GMT) or the local time zone.
class CLASSLIBEXPORT DATETIME {
	public:
		enum FORMAT {
			FORMAT_ISO8601_Basic =		0,		//!< Format is YYYYMMDDTHHMMSS.S+ZZ where 'T' is the separator, and ZZ is the time zone offset
			FORMAT_ISO8601_Extended =	1,		//!< Format is YYYY-MM-DDTHH:MM:SS.S+ZZ:ZZ where 'T' is the separator, and ZZ is the time zone offset
			FORMAT_XML_dateTime =		2,		//!< Format is defined by http://www.w3.org/TR/2004/REC-xmlschema-2-20041028/datatypes.html#dateTime
			FORMAT_XML_date =				3,		//!< Format is defined by http://www.w3.org/TR/2004/REC-xmlschema-2-20041028/datatypes.html#date
			FORMAT_XML_time =				4,		//!< Format is defined by http://www.w3.org/TR/2004/REC-xmlschema-2-20041028/datatypes.html#time
			FORMAT_RFC822 =				5,		//!< Format is dd mm yy hh:mm:ss zzz (SMTP format)
			FORMAT_Simple =				6,		//!< Format is YYYY-MM-DD HH:MM:SS with no time zone specified.  
			FORMAT_YYYYMMDD_HHMMSS =	7,		//!< Format is YYYYMMDD_HHMMSS for job files
			FORMAT_Job =					FORMAT_YYYYMMDD_HHMMSS,		//!< Format is YYYYMMDD_HHMMSS for job files
			};
			
		enum TIME {
			TIME_Uninitialized = 0,
			TIME_CurrentLocal = 1,
			TIME_CurrentUTC = 2
			};

		//! Default constructor.
		DATETIME (
			);

		//! Copy constructor.
		DATETIME (
			const DATETIME& rhs
			);

		//! Constructor for a specific time.
		//!
		DATETIME (
			TIME value
			);

		//! Construct for a given date.
		//! Note, this one doesn't have the IsLocal parameter
		//! since no time is set.
		DATETIME (
			int year,			//!< Year, must include centurary
			int month,			//!< Month (Jan == 1)
			int day				//!< Day of month (1 - 31)
			);

		//! Construct for a given date/time.
		DATETIME (
			int year,			//!< Year, must include centurary
			int month,			//!< Month (Jan == 1)
			int day,				//!< Day of month (1 - 31)
			int hour,
			int min,
			int sec,
			bool bIsLocal		//!< True if time is local, false if UTC
			);

		//! Construct from an RVCDATETIME.
		//! Note: RVCDATETIMEs are always "local" time
		DATETIME (
			const RVCDATETIME& rhs
			);

		//! construct from a struct tm
		DATETIME (
			const struct tm& rhs,
			bool bIsLocal			//!< True if time is local, false if UTC
			);

	#ifdef WIN32
		//! Construct from a SYSTEMTIME (Win32 only)
		DATETIME (
			const struct _SYSTEMTIME& rhs,
			bool bIsLocal			//!< True if time is local, false if UTC
			);
	#endif

	#ifdef WIN32
		//! Construct from a FILETIME (Win32 only)
		DATETIME (
			const struct _FILETIME& rhs,
			bool bIsLocal			//!< True if time is local, false if UTC
			);
	#endif

		//! construct from a time_t.
		//! Note, by definition, a time_t is always \e supposed to be UTC,
		explicit DATETIME (
			time_t value
			);

		//! Destructor
		~DATETIME (
			);

		////////////////////////////////////////////////////////////////////////!
		// Assignment operators

		//! Assignment operator from DATETIME
		DATETIME& operator= (
			const DATETIME& rhs
			);

		//! Assignment from an RVCDATETIME
		DATETIME& operator= (
			const RVCDATETIME& rhs
			);

		//! assignment from time_t (assumed to be UTC (a.k.a. GMT))
		DATETIME& operator= (
			const time_t rhs
			);

		//! assignment from time_t (assumed to be UTC (a.k.a. GMT))
		DATETIME& operator= (
			DATETIME::TIME rhs
			);

		////////////////////////////////////////////////////////////////////////!
		// Comparison operators

		bool operator== (
			const DATETIME& rhs
			) const;

		bool operator!= (
			const DATETIME& rhs
			) const;

		bool operator< (
			const DATETIME& rhs
			) const;

		bool operator> (
			const DATETIME& rhs
			) const;

		bool operator<= (
			const DATETIME& rhs
			) const;

		bool operator>= (
			const DATETIME& rhs
			) const;


		//! Clear.
		void Clear (
			);
		
		//! Change internal representation from UTC to local time zone.
		//! Does nothing if internal representation is already local.
		void ConvertToLocal (
			);

		//! Change internal representation local time zone to UTC.
		//! Does nothing if internal representation is already UTC.
		void ConvertToUTC (
			);

		//! Get the date/time as a (struct tm).
		//! Note, the tm_wday and tm_yday fields are not guarenteed to be
		//! filled out.
		void Get (
			struct tm& tm,
			bool bLocal = false		//!< true to get local time, false to get UTC
			) const;

		//! Get the date/time as an RVCDATETIME.  
		//! Sets the count field to 0.
		//! RVCDATETIMEs are always be local time for historical reasons
		void Get (
			RVCDATETIME& dt
			) const;

	#ifdef WIN32
		//! Get the date/time as a SYSTEMTIME (Win32 only).
		void Get (
			struct _SYSTEMTIME& st,
			bool bLocal = false		//!< true to get local time, false to get UTC
			) const;
	#endif

		//! Get the year, including century.
		INT32 GetYear () const;

		//! Get the month number (1 - 12)
		UINT32 GetMonth () const;

		//! Get the day of the month (1 - 31)
		UINT32 GetDayOfMonth () const;

		//! Get the Hour value of the time.
		UINT32 GetHour () const;

		//! Get the Minutes value of the time.
		UINT32 GetMin () const;

		//! Get the Seconds value of the time.
		UINT32 GetSec () const;

		//! Return the Julian date.
		INT32 GetDateJulian () const;
			
		//! Return the Julian date.
		//! Returns Julian date as a double with the time represented as a fraction of a day
		double GetDateTimeJulian () const;
			
		//! Get the date as an integer in YYYYMMDD format.
		//! This is the form RVC database DBFIELDTYPE_Date uses.
		INT32 GetDateYYYYMMDD () const;

		//! Get the string representation of the date / time according to the standard 'format' specified.
		//! The return value can be used in a MISTRING << operation.
		//! Return the string representation of the date / time.
		MISTRING GetFormatted (
			FORMAT format,
			bool IncludeLocal				//!< Append local time zone information if the 'format' supports it
			) const;

		//! Get formatted string.
		MISTRING GetFormatted (
			const char *format,			//!< Format specifier (see StrfTime)
			bool bLocal = true			//!< True to show local time, false for UTC
			) const;

		//! Return the date given a 100 nanosecond interval from 1-jan-1601 (Windows FILETIME structure)
		//! This is the form RVC database DBFIELDTYPE_DateTime uses.
		UINT64 GetNanoTime () const;

		//! Get time in seconds past midnight
		UINT32 GetTimeOfDay (
			bool bLocal		//!< true to get local time, false for UTC
			) const;

		//! Get the current value as a time_t.
		//! time_t values are always UTC
		time_t GetTimeT () const;

		//! Get day of the week (1 = Sunday, 7 = Saturday)
		UINT32 GetDayOfWeek () const;

		//! Returns the day of the year (1 - 366)
		UINT32 GetDayOfYear () const;

		//! Returns true if the time represented by the local time zone.
		bool IsLocalTime () const;

		//! Determine if value has been set.
		bool IsSet () const;

		//! Returns true if the time represented by the class is in UTC.
		//! Note, this is just !IsLocalTime()
		bool IsUTCTime () const;

		//! Determine if contains valid date and time.
		bool IsValid () const;

		//! Set from time_t.
		//! time_t values \e should always be UTC
		void Set (
			const time_t rhs
			);
			
		//! Set from (struct tm)
		void Set (
			const struct tm& rhs,
			bool bIsLocal				//!< True if time is local, false if UTC
			);

		//! Set value from an RVCDATETIME.
		//! Note, RVCDATETIMEs are stored as local, not UTC
		void Set (
			const RVCDATETIME& rhs
			);

		//! Set the time from a string
		//!
		//! See the man page for strptime() on most Unix systems for details
		//!
		//! @return NULL if the conversion fails, else returns a pointer to the
		//! first character in the input string which is not part of a date/time
		//! string.
		const MIUNICODE* Set (
			const MIUNICODE* str,
			const char *format = "%c"	//!< See StrfTime
			);

	#ifdef WIN32
		//! Set from a SYSTEMTIME (Win32 only).
		//! Note, I'd prototype this as taking (const SYSTEMTIME&) but I'd have to
		//! include windows.h.  I can forward declare struct _SYSTEMTIME
		void Set (
			const struct _SYSTEMTIME& rhs,
			bool bIsLocal				//!< True if time is local, false if UTC
			);
	#endif

	#ifdef WIN32
		//! Set from a FILETIME (Win32 only).
		void Set (
			const struct _FILETIME& rhs,
			bool bIsLocal 				//!< True if time is local, false if UTC
			);
	#endif

		//! Set to 'build' date.  Time will be 12:00 (noon) UTC.
		void SetBuild (
			);

		//! Set to the current date/time
		void SetCurrent (
			bool bLocalTime = true	//!< True to set to local time, false for UTC
			);

		//! Set the date (Leaves the time alone)
		void SetDate (
			INT32 year,		//!< must include century
			UINT32 month,		//!< Jan == 1
			UINT32 day			//!< 1 - 31
			);

		//! Set the date given a julian date.
		//! Leaves time fields alone.
		void SetDateJulian (
			INT32 juliandate
			);
		
		//! Set the date given a julian date.
		//! time is set from the fractional part of the double
		void SetDateTimeJulian (
			double juliandate
			);

		//! Set 'this' from the string representation of the date / time according to the standard 'format' specified.
		//! The time value obtained from the formatted string will be converted to UTC time if the string includes the
		//! time zone offset.
		void SetFormatted (
			const MISTRING& dtstr, 
			FORMAT format
			);
			
		//! Set the date given a 100 nanosecond interval from 1-jan-1601 (Windows FILETIME structure)
		//! This is the form RVC database DBFIELDTYPE_DateTime uses
		//! Not called Set() due to ambiguity
		void SetNanoTime (
			UINT64 rhs
			);

		//! Set date given date in YYYYMMDD format
		//! Leaves time fields alone.
		void SetDateYYYYMMDD (
			int yyyymmdd
			);

		//! Set the time in hours, minutes and seconds.
		//! Leaves date fields alone.
		void SetTimeOfDay (
			UINT32 hour,
			UINT32 min,
			UINT32 sec,
			bool bIsLocal			//!< True if time is local, false if UTC
			);

		//! Set the time in seconds since midnight.
		//! Leaves date fields alone.
		void SetTimeOfDay (
			UINT32 SecondsSinceMidnight,
			bool bIsLocal 			//!< True if time is local, false if UTC
			);


		//! Print a time/date to a MISTRING using strftime() formatting.
		//! Does not clear the MISTRING first.  The time/date string is
		//! appended to the current contents.
		//!
		//!	The format string must only use punctuation, whitespace and
		//! the following things...
		//!
		//!	  \li <b>%%</b>	A literal "%" character.
		//!	  \li <b>%a</b>	The abbreviated weekday according to the current locale
		//!	  \li <b>%b</b>	The abbreviated month name according to the current locale
		//!	  \li <b>%B</b>	The full month name according to the current locale
		//!	  \li <b>%c</b>	The preferred date and time representation for the current locale.
		//!	  \li <b>%C</b>	The century number (year/100) as a 2-digit integer
		//!   \li <b>%d</b>	The day of the month as a decimal number (01 - 31)
		//!   \li <b>%e</b>  Like %d, but leading zero is replaced by a space.
		//!	  \li <b>%h</b>	Same as %b
		//!	  \li <b>%H</b>	The hour as a decimal number using a 24-hour clock (00 - 23)
		//!	  \li <b>%I</b>	The hour as a decimal number using a 12-hour clock (00 - 12)
		//!	  \li <b>%j</b>	The day of the year as a decimal number (001 - 366)
		//!	  \li <b>%k</b>	The hour (24-hour clock) Single digits preceded by a blank
		//!	  \li <b>%l</b>	The hour (12-hour clock) Single digits preceded by a blank
		//!	  \li <b>%m</b>	The month as a decimal number (01 - 12)
		//!	  \li <b>%M</b>	The minute as a decimal number (00 - 59)
		//!	  \li <b>%n</b>	A newline character (same as \n)
		//!	  \li <b>%p</b>	Either "AM" or "PM" in the current locale (noon=="PM", midnight=="AM")
		//!	  \li <b>%P</b>	Either "am" or "pm" in the current locale. Yes, %p is uppercase and
		//!						%P is lowercase.  That's POSIX's fault!
		//!	  \li <b>%r</b>	The time in AM or PM notation. In POSIX locale, this is
		//!						"%I:%M:%S %p"
		//!	  \li <b>%r</b>	The time in 24-hour notation. In POSIX locale, this is
		//!						"%H:%M".  For a version including seconds, see %T
		//!	  \li <b>%s</b>	The number of seconds since the Epoch (1970-01-01 00:00:00 UTC)
		//!						except that on Macintoshes, the Epoch started in 1974.
		//!	  \li <b>%S</b>	The seconds as a decimal number (00 - 59)
		//!	  \li <b>%t</b>	A tab character 
		//!	  \li <b>%T</b>	The time in 24-hour notation. (%H:%M:%S)
		//!	  \li <b>%u</b>	The day of the week as a decimal (1 - 7, Monday == 1)
		//!	  \li <b>%U</b>	The week number of the current year as a decimal number (00 - 53)
		//!						starting with the first Sunday as the first day of week 01.  See
		//!						also %V and %W
		//!	  \li <b>%V</b>	The ISO 8601:1988 week number of the current year (01 - 53)
		//!						where week 1 is the first week that has at least 4 days in the
		//!						current year and Monday is the first day of the week.
		//!	  \li <b>%w</b>	The day of hte week as a decimal (0 - 6, Sunday == 0)
		//!	  \li <b>%W</b>	The week number of the current year as a decimal number (00 - 53),
		//!						starting with the first Monday as the first day of week 01
		//!	  \li <b>%x</b>	The preferred date representation for the current locale without the time.
		//!	  \li <b>%X</b>	The preferred time representation for the current locale without the date.
		//!	  \li <b>%y</b>	The year as a decimal number without a century (00 - 99)
		//!	  \li <b>%Y</b>	The year as a decimal number including the century
		//!	  \li <b>%z</b>	The time-zone as hour offset from GMT.  Required to emit RFC822-conformant
		//!						dates.
		//!	  \li <b>%Z</b>	The time-zone name or abbreviation
		//!	  
		void StrfTime (
			MISTRING& str,				//!< String returned (contents will not be cleared)
			const char *format,		//!< Format specifier
			bool bLocal = true		//!< True to show local time, false for UTC
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! The following "Set" methods were declared "bad" by the rest of the
		//! group at the programmer meeting.  They are still used internally
		//! by SetDate() and SetTimeOFDay().

		//! Set the Hour value of the time.
		void SetHour (
			UINT32 val
			);

		//! sets flag telling whether time is local or UTC.
		//! Default assumes UTC.  Note that setting this flag does not
		//! actually change the time represented or set any time zone
		//! information.  Use ConvertToLocal() or ConvertToUTC() to do 
		//!	that.
		void SetIsLocal (
			bool bIsLocal
			);

		//! Set the day of the month (1 - 31).
		void SetDayOfMonth (
			UINT32 val
			);

		//! Set the Minutes value of the time.
		void SetMin (
			UINT32 val
			);

		//! Set the month number (1 - 12).
		void SetMonth (
			UINT32 val
			);

		//! Set the Seconds value of the time.
		void SetSec (
			UINT32 val
			);
		
		//! Set the Year.  Must include century
		void SetYear (
			INT32 val
			);

		//! Converts a struct tm to a time_t
		//! Same as mktime(), but not all platforms have it.
		static time_t TimeTFromTM (
			const struct tm*const timeval
			);
	
		struct tm m_tm;
		bool m_bIsLocal;		//!< True if time is local, false if UTC
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef CLASSLIBEXPORT

#endif
