/**
 *	\file milist.h <mi32/milist.h>
 *	\brief Definition for MILIST template
 *
 * \if NODOC
 * $Log: milist.h_v $
 * Revision 1.54  2012/07/26 14:54:31  mju
 * Add "at" methods.
 *
 * Revision 1.53  2011/10/11 13:26:32  mju
 * Remove unneeded hdr.
 *
 * Revision 1.52  2010/01/18 20:56:24  mju
 * Add extract method.
 *
 * Revision 1.51  2010/01/14 17:13:19  mju
 * Add insertAfter/Before.
 *
 * Revision 1.50  2009/04/24 16:09:05  mju
 * Add exchange method.
 *
 * Revision 1.49  2008/02/11 15:04:45  mju
 * Add removeMatching.
 *
 * Revision 1.48  2007/02/09 18:26:45  mju
 * Add copyIterator.
 *
 * Revision 1.47  2007/02/09 17:04:56  mju
 * Remove unsafe listitem::swap method.
 * (was unused).
 *
 * Revision 1.46  2006/12/12 22:41:37  scowan
 * Added hasSameValues.
 *
 * Revision 1.41  2005/08/12 22:16:52  scowan
 * Added reverseBegin and reverseEnd methods.
 *
 * Revision 1.39  2005/04/14 17:28:37  mju
 * Add Find() methods.
 *
 * Revision 1.38  2005/04/05 15:49:54  scowan
 * Fixed problem of getIndexed of not decrementing 'index' when advancing thought the list.
 *
 * Revision 1.37  2005/03/24 14:28:38  mju
 * Add getIndexed.
 *
 * Revision 1.35  2004/08/17 17:43:23  mju
 * Change getfront/back to not use iterator.
 *
 * Revision 1.34  2004/06/10 15:36:32  mju
 * Add Reverse() method.
 *
 * Revision 1.32  2004/04/16 20:47:03  mju
 * Add IsEqual method.
 *
 * Revision 1.31  2003/12/22 16:28:05  dwilliss
 * Added RCS log
 * \endif
**/

#ifndef INC_MI32_MILIST_H
#define INC_MI32_MILIST_H

#ifndef	INC_MI32_MEMALLOC_H
	#include <mi32/memalloc.h>
#endif

#include <functional>

//! List container template.
template <typename _LIT> class MILIST {

	protected:

		struct LISTITEM {					//! Internal list item holder structure
			_LIT m_Item;						//!< Actual template item
			LISTITEM *m_Next, *m_Prev;

			LISTITEM (					//! Constructor
				const _LIT& item
				) : m_Item(item), m_Prev(0), m_Next(0) {}
				
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
			
				LISTITEM (const LISTITEM&);
				LISTITEM& operator= (const LISTITEM&);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

	public:

		// Forward declarations.

		class ITERATOR;					
		class CONST_ITERATOR;
		friend class CONST_ITERATOR;

		//! Const iterator class
		class CONST_ITERATOR {
			public:

				CONST_ITERATOR (				//! Default constructor
					) {}

				CONST_ITERATOR (				//! Internal constructor
					LISTITEM *item
					) : m_Ptr(item) {}

				CONST_ITERATOR (				//! Copy constructor
					const ITERATOR& item
					) : m_Ptr(item.m_Ptr) {}

				const _LIT& operator* (		//! Dereference operator, returns reference to templated item
					) const { return (CONST_ITERATOR::m_Ptr->m_Item); }

				const _LIT* operator-> (	//! Dereference operator, returns pointer to templated item
					) const { return (&**this); }

				//! Pre-increment operator.
				CONST_ITERATOR& operator++ (
					) {
					CONST_ITERATOR::m_Ptr = CONST_ITERATOR::m_Ptr->m_Next;
					return (*this);
					}

				//! Post-increment operator.
				CONST_ITERATOR operator++(int
					) {
					CONST_ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				//! Pre-decrement operator.
				CONST_ITERATOR& operator-- (
					) {
					CONST_ITERATOR::m_Ptr = CONST_ITERATOR::m_Ptr->m_Prev;
					return (*this);
					}

				//! Post-decrement operator.
				CONST_ITERATOR operator--(int
					) {
					CONST_ITERATOR temp = *this;
					--*this;
					return (temp);
					}

				bool operator== (				//! Equality operator
					const CONST_ITERATOR& rhs
					) const { return (CONST_ITERATOR::m_Ptr == rhs.m_Ptr); }

				bool operator!= (				//! Inequality operator
					const CONST_ITERATOR& rhs
					) const { return (!(*this == rhs)); }

				LISTITEM *GetNode (			//! Internal, get a LISTITEM*
					) const { return (m_Ptr); }

			protected:
				LISTITEM *m_Ptr;
			};

		friend class ITERATOR;			//!< Allow access to LISTITEM

		class ITERATOR : public CONST_ITERATOR {
			public:

				ITERATOR (						//! Default constructor
					) {}

				ITERATOR (						//! Internal constructor
					LISTITEM *item
					) : CONST_ITERATOR(item) {}
					
				//! Dereference operator, returns reference to templated item
				_LIT& operator* (				
					) const { return (CONST_ITERATOR::m_Ptr->m_Item); }

				//! Dereference operator, returns pointer to templated item
				_LIT* operator-> (
					) const { return (&**this); }

				//! Pre-increment operator.
				ITERATOR& operator++ (
					) {
					CONST_ITERATOR::m_Ptr = CONST_ITERATOR::m_Ptr->m_Next;
					return (*this);
					}

				//! Post-increment operator.
				ITERATOR operator++(int
					) {
					ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				//! Pre-decrement operator.
				ITERATOR& operator-- (
					) {
					CONST_ITERATOR::m_Ptr = CONST_ITERATOR::m_Ptr->m_Prev;
					return (*this);
					}

				//! Post-increment operator.
				ITERATOR operator--(int
					) {
					ITERATOR temp = *this;
					--*this;
					return (temp);
					}

				//! Equality operator
				bool operator== (
					const ITERATOR& rhs
					) const { return (CONST_ITERATOR::m_Ptr == rhs.m_Ptr); }

				//! Inequality operator
				bool operator!= (
					const ITERATOR& rhs
					) const { return (!(*this == rhs)); }

			};

		//! Constructor for an empty list.
		MILIST (								
			) : m_Head(0), m_NumItems(0) {
			MmAllocCExc((void **)&m_Head, sizeof(LISTITEM));		//! Deliberate MmAllocC instead of new
			m_Head->m_Next = m_Head->m_Prev = m_Head;
			}

		//! Copy constructor.
		MILIST (								
			const MILIST<_LIT>& rhs
			) : m_Head(0), m_NumItems(0) {
			MmAllocCExc((void **)&m_Head, sizeof(LISTITEM));		//! Deliberate MmAllocC instead of new
			m_Head->m_Next = m_Head->m_Prev = m_Head;
			for (CONST_ITERATOR it = rhs.Begin(); (it != rhs.End()); ++it) {
				Insert(End(), *it);
				}
			}

		//! Destructor.
		~MILIST (							
			) {
			Clear();
			MmFree(m_Head);
			}

		//! Assignment operator.
		MILIST<_LIT>& operator= (		
			const MILIST<_LIT>& rhs
			) {
			if (this != &rhs) {
				Clear();
				for (CONST_ITERATOR it = rhs.Begin(); (it != rhs.End()); ++it) {
					Insert(End(), *it);
					}
				}
			return (*this);
			}

		//! Return ITERATOR that refers to specified position in the list.
		ITERATOR At (
			INT32 pos
			) { ITERATOR it = Begin(); while (pos > 0 && it != End()) { ++it; --pos; } return (it); }

		//! Return CONST_ITERATOR that refers to specified position in the list.
		CONST_ITERATOR At (
			INT32 pos
			) const { CONST_ITERATOR it = Begin(); while (pos > 0 && it != End()) { ++it; --pos; } return (it); }

		//! Return an ITERATOR that refers to the beginning of the list.
		ITERATOR Begin (					
			) { return (ITERATOR(m_Head->m_Next)); }

		//! Return a CONST_ITERATOR that refers to the beginning of the list.
		CONST_ITERATOR Begin (		  
			) const { return (CONST_ITERATOR(m_Head->m_Next)); }

		//! Remove all entries from the list.
		void Clear (						
			) { Remove(Begin(), End()); }

		//! Determine if specified item is contained in list.
		//! This does a linear search of the list, so consider efficiency implications when using.
		//! List item must support equality comparison operator.
		bool Contains (
			const _LIT& item
			) const {
			for (CONST_ITERATOR it = Begin(); (it != End()); ++it) {
				if (*it == item) return (true);
				}
			return (false);
			}

		//! Copy CONST_ITERATOR to (non-const) ITERATOR.
		//! This requires caller to have non-const access to the list itself.
		ITERATOR CopyIterator (
			CONST_ITERATOR it
			) { return (ITERATOR(it.GetNode())); }

		//! Find item in list.
		//! @return CONST_ITERATOR for item, will reference End() if not found.
		CONST_ITERATOR Find (
			const _LIT& item
			) const {
			CONST_ITERATOR it = Begin();
			for (; (it != End()); ++it) {
				if (*it == item) break;
				}
			return (it);
			}

		//! Find item in list (non-const).
		//! @return ITERATOR for item, will reference End() if not found.
		ITERATOR Find (
			const _LIT& item
			) {
			ITERATOR it = Begin();
			for (; (it != End()); ++it) {
				if (*it == item) break;
				}
			return (it);
			}

		//! Return an ITERATOR that refers to the end of the list, not a valid list item.
		ITERATOR End (
			) { return (ITERATOR(m_Head)); }

		//! Return a CONST_ITERATOR that refers to the end of the list, not a valid list item.
		CONST_ITERATOR End (
			) const { return (CONST_ITERATOR(m_Head)); }

		//! Exchange list with another.
		void Exchange (
			MILIST<_LIT>& other
			) {
			LISTITEM *TempHead = m_Head;
			m_Head = other.m_Head;
			other.m_Head = TempHead;
			INT32 TempNumItems = m_NumItems;
			m_NumItems = other.m_NumItems;
			other.m_NumItems = TempNumItems;
			return;
			}

		//! Extract item from another list, placing at end of this list.
		void Extract (
			MILIST<_LIT>& other,	//!< List to extract from, must not be current list
			ITERATOR& it			//!< Item in other list to extract, must not be at end of other list.
			) {
			other.RemoveItem(it);
			InsertItem(it,End());
			}

		//! Return a reference to a const _LIT item from the front of the list
		const _LIT& GetBack (
			) const { return (m_Head->m_Prev->m_Item); }

		//! Return a reference to a _LIT item from the front of the list
		_LIT& GetBack (
			) { return (m_Head->m_Prev->m_Item); }

		//! Return a reference to a const _LIT item from the front of the list
		const _LIT& GetFront (
			) const { return (m_Head->m_Next->m_Item); }

		//! Return a reference to a _LIT item from the front of the list
		_LIT& GetFront (
			) { return (m_Head->m_Next->m_Item); }

		//! Get item at specified index.
		//! Note that this will iterate through the list to the specified
		//! index and thus should be used with care.
		const _LIT& GetIndexed (
			int index
			) const { return (*At(index)); }

		//! Get item at specified index.
		//! Note that this will iterate through the list to the specified
		//! index and thus should be used with care.
		_LIT& GetIndexed (
			int index
			) { return (*At(index)); }

		//! Return the number of items in the list.
		INT32 GetNumItems (				
			) const { return (m_NumItems); }

		//! Determine equality with another list of same type, regardless of the item order.
		bool HasSameValues (
			const MILIST<_LIT>& rhs
			) const {
			if (m_NumItems != rhs.m_NumItems) return (false);
			for (CONST_ITERATOR lit = Begin();(lit != End()); ++lit) {
				if (!rhs.Contains(*lit)) return (false);
				}
			return (true);
			}
			
		//! Insert an item into the place BEFORE the iterator.
		//! @return Reference to new item in list (not same as 'item' parameter as will be copied).
		_LIT& Insert (
			ITERATOR it,
			const _LIT& item
			) { return (InsertBefore(it,item)); }

		//! Insert an item into list after specified iterator.
		//! @return Reference to new item in list (not same as 'item' parameter as will be copied).
		_LIT& InsertAfter (						
			ITERATOR it,
			const _LIT& item
			) {
			LISTITEM *pPrev = it.GetNode();
			LISTITEM *pNew = new LISTITEM(item);
			pNew->m_Prev = pPrev;
			pNew->m_Next = pPrev->m_Next;
			pPrev->m_Next->m_Prev = pNew;
			pPrev->m_Next = pNew;
			++m_NumItems;
			return (pNew->m_Item);
			}

		//! Insert an item into list before specified iterator.
		//! @return Reference to new item in list (not same as 'item' parameter as will be copied).
		_LIT& InsertBefore (
			ITERATOR it,
			const _LIT& item
			) {
			LISTITEM *pNext = it.GetNode();
			LISTITEM *pNew = new LISTITEM(item);
			pNew->m_Next = pNext;
			pNew->m_Prev = pNext->m_Prev;
			pNext->m_Prev->m_Next = pNew;
			pNext->m_Prev = pNew;
			++m_NumItems;
			return (pNew->m_Item);
			}

		//! Determine if any list entries exist.
		bool IsEmpty (						
			) const { return (m_NumItems == 0); }

		//! Determine equality with another list of same type.
		bool IsEqual (
			const MILIST<_LIT>& rhs
			) const {
			if (m_NumItems != rhs.m_NumItems) return (false);
			for (CONST_ITERATOR lit = Begin(), rit = rhs.Begin(); (lit != End()); ++lit, ++rit) {
				if (*lit != *rit) return (false);
				}
			return (true);
			}

		//! Moves the item referenced by the iterator toward the beginning of the list.
		void Lower (
			ITERATOR it
			) {
			if (it == Begin()) return;
			ITERATOR lower = it;
			--lower;
			RemoveItem(it);
			InsertItem(it, lower);
			}

		//! Moves the item referenced by the iterator toward the beginning of the list.
		void Lower (
			int pos
			) {
			ITERATOR it;
			for (it = Begin(); (it != End() && pos > 0); --pos, ++it)
				;
			if (it != End()) Lower(it);
			}

		//! Merge two lists together, the list copied from is cleared, does not sort.
		void Merge (						
			MILIST<_LIT>& other
			) {
			if (other.IsEmpty()) return;
			if (IsEmpty()) {
				LISTITEM *temp = m_Head;
				m_Head = other.m_Head;
				other.m_Head = temp;
				m_NumItems = other.m_NumItems;
				other.m_NumItems = 0;
				}
			else {
				other.m_Head->m_Prev->m_Next = m_Head;
				other.m_Head->m_Next->m_Prev = m_Head->m_Prev;
				m_Head->m_Prev->m_Next = other.m_Head->m_Next;
				m_Head->m_Prev = other.m_Head->m_Prev;
				m_NumItems += other.m_NumItems;
				other.m_Head->m_Prev = other.m_Head->m_Next = other.m_Head;
				other.m_NumItems = 0;
				}	
			return;
			}

		//! Moves the item referenced by the iterator to the back of the list
		void MoveToBack (
			ITERATOR it
			) {
			if (it == --End()) return;
			RemoveItem(it);
			InsertItem(it, End());
			}

		//! Moves the item referenced by the iterator to the front of the list
		void MoveToFront (
			ITERATOR it
			) {
			if (it == Begin()) return;
			RemoveItem(it);
			InsertItem(it, Begin());
			}

		//! Insert an item at the beginning of the list.
		//! @return Reference to new item in list (not same as 'item' parameter as will be copied).
		_LIT& PushFront (					
			const _LIT& item
			) { return (Insert(Begin(), item)); }

		//! Remove an item from the beginning of the list.
		void PopFront (					
			) { Remove(Begin()); }

		//! Insert an item on the end of the list.
		//! @return Reference to new item in list (not same as 'item' parameter as will be copied).
		_LIT& PushBack (					
			const _LIT& item
			) { return (Insert(End(), item)); }

		//! Remove the last item from the list.
		void PopBack (						
			) { Remove(--End()); }
			
		//! Moves the item referenced by the iterator toward the end of the list.
		void Raise (
			ITERATOR it
			) {
			if (it == --End()) return;
			ITERATOR upper = it;
			++upper;
			RemoveItem(it);
			InsertItem(it, ++upper);
			}

		//! Moves the item referenced by the iterator toward the beginning of the list.
		void Raise (
			int pos
			) {
			ITERATOR it;
			for (it = Begin(); (it != End() && pos > 0); --pos, ++it)
				;
			if (it != End()) Raise(it);
			}

		//! Remove item specified by ITERATOR from the list.
		//! Do not use the specified iterator after using this method as it will be invalid.
		void Remove (
			ITERATOR it
			) {
			RemoveItem(it);
			delete it.GetNode();
			}

		//! Remove a range of items from the list.
		void Remove (
			ITERATOR start,						//!< Iterator for first item to remove
			ITERATOR end							//!< Ending item, exclusive, will not be removed
			) {
			for (ITERATOR it = start; (it != end); ) {
				Remove(it++);
				}
			}

		//! Remove duplicate entries.
		void RemoveDuplicates (			
			) {
			if (m_NumItems < 2) return;
			for (ITERATOR check = Begin(); (check != End()); ++check) {
				ITERATOR it = check;
				for (++it; (it != End()); ++it) {
					if (*check == *it) {
						ITERATOR prev = it;
						--prev;
						Remove(it);
						it = prev;
						}
					}
				}
			return;
			}

		//! Remove matching item(s).
		void RemoveMatching (
			const _LIT& item
			) {
			for (ITERATOR it = Begin(); (it != End()); ) {
				if (*it == item) {
					Remove(it++);
					}
				else {
					++it;
					}
				}
			return;
			}

		//! Remove item at specified position.
		void RemovePos (
			int pos									//!< Position starting at 0 for beginning of list
			) {
			ITERATOR it;
			for (it = Begin(); (it != End() && pos > 0); --pos, ++it)
				;
			if (it != End()) Remove(it);
			}

		//! Reverse order of items in list.
		void Reverse (
			) {
			LISTITEM *pItem = m_Head;
			do {
				LISTITEM *pNext = pItem->m_Next;
				pItem->m_Next = pItem->m_Prev;
				pItem->m_Prev = pNext;
				pItem = pNext;
				} while (pItem != m_Head);
			return;
			}

		//! Return an ITERATOR that refers to the end of the list.
		ITERATOR ReverseBegin (					
			) { return (ITERATOR(m_Head->m_Prev)); }

		//! Return a CONST_ITERATOR that refers to the end of the list.
		CONST_ITERATOR ReverseBegin (
			) const { return (CONST_ITERATOR(m_Head->m_Prev)); }

		//! Return an ITERATOR that refers to the end of the list, not a valid list item.
		ITERATOR ReverseEnd (
			) { return (ITERATOR(m_Head)); }

		//! Return a CONST_ITERATOR that refers to the end of the list, not a valid list item.
		CONST_ITERATOR ReverseEnd (
			) const { return (CONST_ITERATOR(m_Head)); }

		//! Sort the list in operator< () order.
		void Sort (
			) { SortItems(); }

		//! Sort the list in operator< () order.
		template<class _Pr1>
		void Sort (
			_Pr1 _Pr
			) { 
			if (m_NumItems < 2) return;
			MILIST<_LIT> NewList;
			ITERATOR *ItArray = new ITERATOR[m_NumItems+1];
			ItArray[0] = NewList.End();

			while (m_NumItems > 0) {
				ITERATOR item = Begin();
				RemoveItem(item);

				INT32 index = 0;
				if (NewList.m_NumItems) {
					INT32 Left = 0, Right = NewList.m_NumItems-1;
					do {
						index = (Left+Right) / 2;
						if (_Pr(*item, *ItArray[index])) {
							Right = index - 1;
							}
						else {
							Left = ++index;			// The insert method is a place behind, add one to index in case Left > Right
							}
						} while (Left <= Right);
					}
				
				ITERATOR place = ItArray[index];
				for (int i = NewList.m_NumItems+1; (i > index); --i) ItArray[i] = ItArray[i-1];
				ItArray[index] = item;
				NewList.InsertItem(item, place);
				}
			
			delete [] ItArray;
			LISTITEM *temp = m_Head;		// Switch headers
			m_Head = NewList.m_Head;
			NewList.m_Head = temp;
			m_NumItems = NewList.m_NumItems;
			return;
			}
		
		//! Sort the list in operator< () order.
		void Sort (							
			bool (*PredFunc)(const _LIT& lhs, const _LIT& rhs)
			) { SortItems(PredFunc); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		void InsertItem (
			ITERATOR it,
			ITERATOR place
			) {
			LISTITEM *q = it.GetNode();
			LISTITEM *after = place.GetNode();

			q->m_Next = after;
			q->m_Prev = after->m_Prev;
			after->m_Prev->m_Next = q;
			after->m_Prev = q;
			m_NumItems ++;
			return;
			}

		void RemoveItem (
			ITERATOR it
			) {
			LISTITEM *q = it.GetNode();
			q->m_Next->m_Prev = q->m_Prev;
			q->m_Prev->m_Next = q->m_Next;
			m_NumItems --;
			return;
			}
			
		void SortItems (							//! Sort the list in operator< () order
			) {
			if (m_NumItems < 2) return;
			MILIST<_LIT> NewList;
			ITERATOR *ItArray = new ITERATOR[m_NumItems+1];
			ItArray[0] = NewList.End();

			while (m_NumItems > 0) {
				ITERATOR item = Begin();
				RemoveItem(item);

				INT32 index = 0;
				if (NewList.m_NumItems) {
					INT32 Left = 0, Right = NewList.m_NumItems-1;
					do {
						index = (Left+Right) / 2;
						if (*item < *ItArray[index]) {
							Right = index - 1;
							}
						else {
							Left = ++index;			// The insert method is a place behind, add one to index in case Left > Right
							}
						} while (Left <= Right);
					}
				
				ITERATOR place = ItArray[index];
				for (int i = NewList.m_NumItems+1; (i > index); --i) ItArray[i] = ItArray[i-1];
				ItArray[index] = item;
				NewList.InsertItem(item, place);
				}
			
			delete [] ItArray;
			LISTITEM *temp = m_Head;		// Switch headers
			m_Head = NewList.m_Head;
			NewList.m_Head = temp;
			m_NumItems = NewList.m_NumItems;
			return;
			}

		void SortItems (							//! Sort the list in operator< () order
			bool (*PredFunc)(const _LIT& lhs, const _LIT& rhs)
			) {
			if (m_NumItems < 2) return;
			MILIST<_LIT> NewList;
			ITERATOR *ItArray = new ITERATOR[m_NumItems+1];
			ItArray[0] = NewList.End();

			while (m_NumItems > 0) {
				ITERATOR item = Begin();
				RemoveItem(item);

				INT32 index = 0;
				if (NewList.m_NumItems) {
					INT32 Left = 0, Right = NewList.m_NumItems-1;
					do {
						index = (Left+Right) / 2;
						if (PredFunc(*item, *ItArray[index])) {
							Right = index - 1;
							}
						else {
							Left = ++index;			// The insert method is a place behind, add one to index in case Left > Right
							}
						} while (Left <= Right);
					}

				ITERATOR place = ItArray[index];
				for (int i = NewList.m_NumItems + 1; (i > index); --i) ItArray[i] = ItArray[i-1];
				ItArray[index] = item;
				NewList.InsertItem(item, place);
				}
			
			delete [] ItArray;
			LISTITEM *temp = m_Head;		// Switch headers
			m_Head = NewList.m_Head;
			NewList.m_Head = temp;
			m_NumItems = NewList.m_NumItems;
			return;
			}

		LISTITEM *m_Head;
		INT32 m_NumItems;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif		// INC_MI32_MILIST_H
