/**
 * \file minetres.h <mi32/minetres.h>
 * \brief MINETRES class definition.
 *
 * \if NODOC
 * $Id: minetres.h_v 1.7 2009/11/20 18:51:37 mju Exp $
 *
 * $Log: minetres.h_v $
 * Revision 1.7  2009/11/20 18:51:37  mju
 * Add buildServerList and resetServerList.
 *
 * Revision 1.6  2009/11/13 18:54:53  mju
 * Add constructFromHost and getRemoteName.
 *
 * Revision 1.5  2007/05/09 15:06:16  mju
 * Use milist hdr.
 *
 * Revision 1.4  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.3  2001/05/08 21:16:49  mju
 * Redesign again to just keep item info and build lists when needed.
 *
 * Revision 1.1  2001/05/08 17:00:11  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MINETRES_H
#define	INC_MI32_MINETRES_H

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

//! MicroImages Network Resource class.
//!
//! To use this class, start by calling MINETRES::BuildMainList().  The returned
//! list of items may then be iterated through to determine their type and usage
//! and to build their child lists using BuildChildList().
//!
//! Use of this class in Windows (X or native) requires the 'mpr.lib' library.
class MINETRES {
	public:

		//! Item "display type" values.
		enum DISPLAYTYPE {
			DISPLAYTYPE_Invalid = 0,
			DISPLAYTYPE_Network,
			DISPLAYTYPE_Domain,
			DISPLAYTYPE_Server,
			DISPLAYTYPE_Share,
			DISPLAYTYPE_Directory,
			DISPLAYTYPE_Generic
			};

		//! Item "usage" values.
		enum USAGE {
			USAGE_Invalid = 0,
			USAGE_Container,
			USAGE_Connectable
			};

		//! Build main item list from "root" of "network neighborhood".
		static ERRVALUE BuildMainList (
			MILIST<MINETRES>& MainList			//!< List of network resource items returned
			);

		//! Build list of server names.
		static ERRVALUE BuildServerList (
			MISTRINGLIST& ServerList			//!< Names of servers/hosts returned, will be in form of \\hostname
			);

		//! Construct from server host path.
		static ERRVALUE ConstructFromHost (
			const MISTRING& HostPath,			//!< Host path, may be just name or \\hostname
			MINETRES *& pNetResRet				//!< MINETRES instance returned if successful, NULL if not
			);

		//! Reset known list of servers.
		static void ResetServerList (
			);

		//! Destructor.
		~MINETRES (
			);

		//! Build list of child items for this item.
		ERRVALUE BuildChildList (
			MILIST<MINETRES>& ChildList		//!< List of child items returned
			);

		//! Get description for display in list.
		const MISTRING& GetDescription (
			) const { return (m_lpComment); }

		//! Get name to display in list.
		const MISTRING& GetDisplayName (
			) const { return (m_DisplayName); }

		//! Get display type for resource.
		DISPLAYTYPE GetDisplayType (
			) const;

		const MISTRING& GetRemoteName (
			) const { return (m_lpRemoteName); }

		//! Get resource usage.
		USAGE GetUsage (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		UINT32 m_dwScope;
		UINT32 m_dwType;
		UINT32 m_dwDisplayType;
		UINT32 m_dwUsage;
		MISTRING m_lpLocalName;
		MISTRING m_lpRemoteName;
		MISTRING m_lpComment;
		MISTRING m_lpProvider;
		MISTRING m_DisplayName;

		//! Default constructor.
		MINETRES (
			);

	#ifdef WIN32
		static ERRVALUE EnumChildren (void*, MILIST<MINETRES>&);
		static ERRVALUE ReportError (UINT32 retval);
		void SetValues (const void *vpWinNetRes);
	#endif

		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};

typedef MILIST<MINETRES> MINETRESLIST;


#endif	//!<  INC_MI32_MINETRES_H 
