/**
 *	\file mi32/planeconic.h
 * \brief PLANECONIC class definition
 *
 * \if NODOC
 * $Id: planeconic.h_v 1.11 2011/11/09 15:14:31 mju Exp $
 *
 * $Log: planeconic.h_v $
 * Revision 1.11  2011/11/09 15:14:31  mju
 * Fix dox warning.
 *
 * Revision 1.10  2011/11/04 13:06:44  mju
 * Remove redundant hdr.
 *
 * Revision 1.9  2006/06/08 20:08:26  scowan
 * Added rect to ellipse and back methods.
 *
 * Revision 1.7  2005/06/02 19:59:46  scowan
 * Added generating arc using two points and a bulge factor.
 *
 * Revision 1.6  2005/06/01 20:15:59  scowan
 * Added method to generate three points defining the arc or circle.
 *
 * Revision 1.3  2004/05/06 20:57:00  scowan
 * Added coordinate conversion methods.
 *
 * Revision 1.1  2004/04/15 16:24:51  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_PLANECONIC_H
#define	INC_MI32_PLANECONIC_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct DPOINT3D;
class POLYLINE;
class TRANS2D_AFFINE;
class TRANS2D_MAPGEN;
class TRANS3D;
#endif

class GEOMLIBCLASSEXPORT PLANECONIC {
	public:
		enum CLOSURE {
			CLOSURE_Open,				//!< Arc or Elliptical arc, no polygon formed
			CLOSURE_Chord,				//!< Arc or Elliptical arc with closure point as a segment from end angle point to start angle point
			CLOSURE_Wedge,				//!< Arc or Elliptical arc with closure point as a line from end angle point to the center point and ending at the start angle point
			CLOSURE_Complete			//!< Circle or ellipse
			};
			
		//! Default ctor
		PLANECONIC (
			);
			
		//! Copy ctor
		PLANECONIC (
			const PLANECONIC& rhs
			);
			
		//! Assigment
		PLANECONIC& operator= (
			const PLANECONIC& rhs
			);

		//! Clear conic to initial state (no conic defined)
		void Clear (
			);

		//! Compute arc based on three points			
		//! @return 'True' if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeArc (
			const DPOINT2D& p1, 
			const DPOINT2D& p2, 
			const DPOINT2D& p3
			);
			
		//! Compute arc based on three points
		//! Extrusion vertex calculated based on the plane defined by the three points
		//! @return 'True' if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeArc (
			const DPOINT3D& p1, 
			const DPOINT3D& p2, 
			const DPOINT3D& p3
			);
			
		//! Compute arc based on three points			
		//! @return 'True' if successful, false if not (bulge == 0 or 2 points are the same)
		bool ComputeArcByBulge (
			const DPOINT2D& p1, 
			const DPOINT2D& p2, 
			double bulge					//!< If positive, generates the arc on the left side of the line, if negative, on the right side
			);
			
		//! Compute arc based on three points			
		//! @return 'True' if successful, false if not (bulge == 0 or 2 points are the same)
		bool ComputeArcByBulge (
			const DPOINT3D& p1, 
			const DPOINT3D& p2, 
			double bulge,
			const DPOINT3D& Normal 
			);
			
		//! Compute circle based on two points
		//! @return 'True' if successful, false if not (Both points are the same)
		bool ComputeCircle (
			const DPOINT2D& p1, 
			const DPOINT2D& p2
			);

		//! Compute circle based on three points			
		//! @return 'True' if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeCircle (
			const DPOINT2D& p1, 
			const DPOINT2D& p2, 
			const DPOINT2D& p3
			);
			
		//! Compute circle based on three points
		//! Extrusion vertex calculated based on the plane defined by the three points
		//! @return 'True' if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeCircle (
			const DPOINT3D& p1, 
			const DPOINT3D& p2, 
			const DPOINT3D& p3
			);

		//! Compute distance from point given to edge of conic
		//! @return Distance from point to edge of conic
		double ComputeDistanceFrom (
			const DPOINT2D& pt
			) const;
			
		//! Compute distance from point given to edge of conic
		//! @return Distance from point to edge of conic
		double ComputeDistanceFrom (
			const DPOINT3D& pt
			) const;
			

		//! Compute 2D ellipse based on inscribed rectangle given
		//! @return 'True' if successful, false if not (More than 5 points or area of zero)
		bool ComputeEllipseFromRect (
			const POLYLINE& PolyLine
			);
			
		//! Compute extents of conic
		void ComputeExtents (
			DRECT3D& Extents
			) const;
			
		//! Compute circumscribed rectangle from 2D ellipse
		//! @return 'True' if successful, false if not (Not an ellipse)
		bool ComputeRectFromEllipse (
			POLYLINE& PolyLine				//!< Place to put the computed rectangle to
			) const;
			
		//! Convert conic using affine transformation
		//! @return 'True' if conversion valid, 'false' if approximated
		bool ConvertForward (
			const TRANS2D_AFFINE& taf
			);
		
		//! Convert conic using generic transformation		
		//! @return 'True' if conversion valid, 'false' if approximated
		bool ConvertForward (
			const TRANS2D_MAPGEN& tmg
			);

		//! Convert conic using affine transformation
		//! @return 'True' if conversion valid, 'false' if approximated
		bool ConvertInverse (
			const TRANS2D_AFFINE& taf
			);
		
		//! Convert conic using generic transformation		
		//! @return 'True' if conversion valid, 'false' if approximated
		bool ConvertInverse (
			const TRANS2D_MAPGEN& tmg
			);

		//! Given a circular conic (arc or circle), generate a set of three points that will define the arc or circle.
		bool GenerateArc3Points (
			DPOINT2D& p1, 
			DPOINT2D& p2, 
			DPOINT2D& p3
			) const;

		//! Generate a POLYLINE from 'this' using the 'MinChordDist' to determine the number of points used to create 
		//! the line or polygon.
		ERRVALUE GenerateConicPoints (
			POLYLINE& PolyLine,
			double MinChordDist = 0.0					//!< Default is to use %5 of the Radius length
			) const;
			
		//! Get center of conic in absolute coordinates.
		//! @return Center point of the conic
		const DPOINT3D& GetCenterPoint (
			) const { return (m_CenterPoint); }
			
		//! Get the closure of the conic
		//! @return Closure of the conic
		CLOSURE GetClosure (
			) const { return (m_Closure); }
			
		//! Get the end angle of the arc or elliptical arc in radians.
		//! @return End angle in radians
		double GetEndAngle (
			) const { return (m_EndAngle); }
			
		//! Get extrusion point of conic in absolute coordinates.
		//! This point, along with the center point, define the normal vertex to the plane that this conic resides.
		//! @return Extrusion point of the conic
		const DPOINT3D& GetExtrusionPoint (
			) const { return (m_Extrusion); }
			
 		//! Rotation of conic with respect to the X axis.  Not used if conic is circle or arc.
		//! @return Rotation of conic in radians.
		double GetRotation (
			) const { return (m_Rotation); }

		//! Get the start angle of the arc or elliptical arc in radians.
		//! @return Start angle in radians
		double GetStartAngle (
			) const { return (m_StartAngle); }

		//! Get the X radius of conic.			
		//! Conic is a circle or arc if m_XRadius == m_YRadius or m_YRadius == 0.0.
		//! @return X radius of conic.			
		double GetXRadius (
			) const { return (m_XRadius); }

		//! Get the Y radius of conic.			
		//! Conic is a circle or arc if m_XRadius == m_YRadius or m_YRadius == 0.0.
		//! @return Y radius of conic.			
		double GetYRadius (
			) const { return (m_YRadius); }
			
		//! Is extrusion vertex valid?
		//! @return 'True' if extrusion vertex is valid, 'false' if not
		bool HasExtrusion (
			) const { return (m_HasExtrusion); }
			
		//! Is the conic element circular?
		//! @return 'True' if circular, 'false' if elliptical
		bool IsCircular (
			) const { return (m_XRadius == m_YRadius || m_YRadius == 0); }
		
		//! Is the conic element defined?
		bool IsDefined (
			) const { return (m_XRadius > 0.0); }
			
		//! Is the point inside the conic
		//! If the conic is extruded, it will determine if the point lies inside the projected conic to the XY plane
		bool IsPointInside (
			const DPOINT2D& pt
			) const;
			
		//! Does the point lie inside the conic defined on the plane
		bool IsPointInside (
			const DPOINT3D& pt
			) const;
			
		//! Get center of conic in absolute coordinates.
		void SetCenterPoint (
			const DPOINT3D& point
			) { m_CenterPoint = point; }
			
		//! Set the closure of the conic
		void SetClosure (
			CLOSURE Closure
			) { m_Closure = Closure; }
			
		//! Set the end angle of the arc or elliptical arc in radians.
		void SetEndAngle (
			double angle
			) { m_EndAngle = angle; }
			
		//! Set extrusion point of conic in absolute coordinates.
		//! This point, along with the center point, define the normal vertex to the plane that this conic resides.
		void SetExtrusionPoint (
			const DPOINT3D& Extrusion
			);
			
		//! Set the start angle of the arc or elliptical arc in radians.
		void SetStartAngle (
			double angle
			) { m_StartAngle = angle; }

		//! Set the X radius of conic.			
		//! Conic is a circle or arc if m_XRadius == m_YRadius or m_YRadius == 0.0.
		void SetXRadius (
			double radius
			) { m_XRadius = radius; }

		//! Set the Y radius of conic.			
		//! Conic is a circle or arc if m_XRadius == m_YRadius or m_YRadius == 0.0.
		void SetYRadius (
			double radius
			) { m_YRadius = radius; }
			
		//! Rotation of conic with respect to the X axis.  Not used if conic is circle or arc.
		void SetRotation (
			double rotation
			) { m_Rotation = rotation; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DPOINT3D m_CenterPoint;					// Center of conic in absolute coordinates.
		double m_StartAngle;						// Start angle of arc in radians.
		double m_EndAngle;						// End angle of arc in radians.  Conic is a circle or ellipse if m_EndAngle == m_StartAngle || m_EndAngle > m_StartAngle + 2*PI
		double m_XRadius;							// X radius of conic.
		double m_YRadius;							// Y radius of conic.  Conic is a circle or arc if m_XRadius == m_YRadius or m_YRadius == 0.0.
		double m_Rotation;						// Rotation of conic with respect to the X axis.  Not used if conic is circle or arc.
		DPOINT3D m_Extrusion;					// This point, along with the center point, define the normal vertex to the plane that this conic resides.
		bool m_HasExtrusion;						// Extrusion vertex is valid
		CLOSURE m_Closure;

		void ComputeArcEndPoints (DPOINT2D& StartPt, DPOINT2D& EndPt) const;
		double ComputeAngleRadius (double angle) const;
		bool ComputeCircleLow (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3);
		void Generate3dTrans (TRANS3D& t3d) const;
		void GenerateStartEndAngle (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3);
		#endif		// GENERATING_DOXYGEN_OUTPUT
			
	};

#undef GEOMLIBEXPORT
#undef GEOMLIBCLASSEXPORT

#endif	// INC_MI32_PLANECONIC_H 
