/**
 * \file mi32/renderbuffer.h
 * \brief RENDERBUFFER class definition
 *
 * \if NODOC
 * $Id: renderbuffer.h_v 1.19 2011/11/10 22:27:14 mju Exp $
 *
 * $Log: renderbuffer.h_v $
 * Revision 1.19  2011/11/10 22:27:14  mju
 * Add parm to copyRect.
 *
 * Revision 1.18  2011/10/28 20:06:23  mju
 * Add overlayOnDevice using frame buffer.
 *
 * Revision 1.17  2011/10/28 17:36:20  mju
 * Add copyFromDevice variant using single frame buffer.
 *
 * Revision 1.16  2011/10/28 16:24:25  mju
 * Support multiple frame buffers, remove stereo-specific setup.
 *
 * Revision 1.15  2011/10/20 17:12:36  mju
 * Add setupStereoSideBySide.
 *
 * Revision 1.14  2011/09/28 21:26:41  mju
 * Use stereodevice class.
 *
 * Revision 1.13  2008/08/08 16:48:47  mju
 * Add erase method.
 *
 * Revision 1.12  2008/06/19 21:45:37  mju
 * Add rect variant for overlay.
 *
 * Revision 1.11  2008/05/15 17:52:52  mju
 * Add copyFromBuffer.
 *
 * Revision 1.10  2008/04/11 16:17:05  mju
 * Make overlay method const with mask parms.
 * Remove methods to set rgb mask.
 *
 * Revision 1.9  2008/03/10 20:55:21  mju
 * Add target point parm.
 * Add ability to specify tile size when setup device.
 *
 * Revision 1.8  2007/04/18 17:30:04  mju
 * Use stereodevice hdr.
 *
 * Revision 1.7  2005/08/29 20:45:26  scowan
 * Fixed g++ warning about shadow variable duplication.
 *
 * Revision 1.5  2005/06/15 22:07:10  mju
 * Add interlaceFromDevice.
 *
 * Revision 1.4  2005/05/24 21:29:18  mju
 * Unify overlayOnDevice with optional cliprect.
 *
 * Revision 1.3  2005/05/24 17:54:37  mju
 * Add ability to set plane masks for use when overlay.
 *
 * Revision 1.2  2005/05/03 20:29:45  mju
 * Add overlayOnDevice variant using rectangle.
 *
 * Revision 1.1  2005/04/22 17:34:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_RENDERBUFFER_H
#define	INC_MI32_RENDERBUFFER_H

#ifndef INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef INC_MI32_COLORSPC_H
#include <mi32/colorspc.h>
#endif

#ifndef INC_MI32_STEREODEVICE_H
#include <mi32/stereodevice.h>
#endif


namespace MGD {
	class DEVICE;
	class DEVICE_MEM32RGBA;
	}

//===================================================================================================================

//! General memory-based rendering buffer.
//! Supports creation/copying from and overlay on other MGD devices.
class RENDERBUFFER {
	public:

		//! Get target colorspace suitable for overlay.
		static COLORSPACE GetTargetColorSpace (
			const MGD::DEVICE *pDeviceRef					//!< Reference device
			);

		//! Constructor.
		RENDERBUFFER ();

		//! Destructor.
		~RENDERBUFFER ();

		//! Copy rendering from specified source device using all frame buffers.
		//! Created device will have same (virtual) size, number of frame buffers and colorspace as source.
		//! If successful will set as having been rendered.
		ERRVALUE CopyFromDevice (
			const MGD::DEVICE_MEM32RGBA *pDevSource,	//!< Source device to copy from
			bool UseMinimalRectangle,						//!< Use minimal rectangle
			bool CopyDepth = false
			);

		//! Copy rendering from specified source device using specified frame buffer.
		//! Created device will have same (virtual) size and colorspace as source with single frame buffer.
		//! If successful will set as having been rendered.
		ERRVALUE CopyFromDevice (
			const MGD::DEVICE_MEM32RGBA *pDevSource,	//!< Source device to copy from
			UINT16 FrameBuffer,								//!< Source frame buffer to copy
			bool UseMinimalRectangle,						//!< Use minimal rectangle
			bool CopyDepth = false
			);

		//! Destroy device buffer if any.
		void DestroyDevice ();

		//! Erase buffer contents.
		void Erase ();

		//! Get rendering device (non-const).
		//! @return Device pointer, NULL if not yet created.
		MGD::DEVICE* GetDevice ();

		//! Get rendering device (const).
		//! @return Device pointer, NULL if not yet created.
		const MGD::DEVICE* GetDevice () const;

		//! Determine if has been rendered.
		bool HasRendered (
			) const { return (m_pDevice != 0 && m_HasRendered); }

		//! Interlace left/right stereo views.
		ERRVALUE InterlaceFromDevice (
			MGD::DEVICE_MEM32RGBA *pDevSource,			//!< Source device to copy from
			const LRECT2D& SrcRectLeft,					//!< Source rectangle for left view
			const LRECT2D& SrcRectRight,					//!< Source rectangle for right view
			STEREODEVICE::METHOD StereoMode,				//!< Stereo mode
			bool UseMinimalRectangle = true				//!< Use minimal rectangle
			);

		//! Overlay rendered buffer on specified device using rectangle.
		//! Will copy all frame buffers if target and buffer have same number, otherwise will only
		//! copy the first frame buffer.
		void OverlayOnDevice (
			MGD::DEVICE *pDevTarget,					//!< Target device to overlay on
			const LRECT2D *pClipRectTgt = 0,			//!< Optional limiting rectangle in target device coordinates
			const LPOINT2D *pPointTgt = 0,			//!< Upper left target point
			UINT8 MaskRed = 255,
			UINT8 MaskGreen = 255,
			UINT8 MaskBlue = 255
			) const;

		//! Overlay rendered buffer on specified device using rectangle.
		//! Will copy all frame buffers if target and buffer have same number, otherwise will only
		//! copy the first frame buffer.
		void OverlayOnDevice (
			MGD::DEVICE *pDevTarget,					//!< Target device to overlay on
			UINT16 FrameBufferTgt,						//!< Frame buffer on target device to copy to
			const LRECT2D *pClipRectTgt = 0,			//!< Optional limiting rectangle in target device coordinates
			const LPOINT2D *pPointTgt = 0,			//!< Upper left target point
			UINT8 MaskRed = 255,
			UINT8 MaskGreen = 255,
			UINT8 MaskBlue = 255
			) const;

		//! Set whether current rendering is valid or not.
		void SetRendered (
			bool bHasRendered								//!< True to mark as valid, false to mark as invalid
			) { m_HasRendered = bHasRendered; }

		//! Setup device.
		//! Existing rendering will be lost.
		//! Created device will have same size and colorspace as specified reference.
		ERRVALUE SetupDevice (
			const MGD::DEVICE *pDeviceRef,			//!< Reference device for determining size and colorspace
			INT32 width = 0,								//!< Device width override if nonzero
			INT32 height = 0,								//!< Device height override if nonzero
			UINT16 NumFrameBuffers = 1					//!< Number of frame buffers, 0 to match reference device, typically 2 for stereo
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGD::DEVICE_MEM32RGBA *m_pDevice;
		LRECT2D m_ValidRect;								// Portion of virtual rendering space which is valid
		bool m_HasRendered;								// Has been rendered

		void CopyRect (
			MGD::DEVICE *pDevTarget,
			const MGD::DEVICE *pDevSource,
			UINT16 FrameBuffer,
			bool UseTransp,
			bool UseDepth,
			const LRECT2D *pClipRectTgt = 0,
			const LPOINT2D *pPointTgt = 0,
			UINT8 MaskRed = 255,
			UINT8 MaskGreen = 255,
			UINT8 MaskBlue = 255
			) const;

		// Unimplemented
		RENDERBUFFER (const RENDERBUFFER&);
		RENDERBUFFER& operator= (const RENDERBUFFER&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================


#endif	// INC_MI32_RENDERBUFFER_H
