/**
 * \file rvc/binary.h 
 * \brief interface for the RVC::BINARY class.
 *
 * \if NODOC
 * $Id: binary.h_v 1.17 2011/12/06 22:10:52 mju Exp $
 *
 * $Log: binary.h_v $
 * Revision 1.17  2011/12/06 22:10:52  mju
 * Remove redundant hdr.
 *
 * Revision 1.16  2006/06/28 15:55:51  scowan
 * Changed make to take a non-const objitem.
 *
 * Revision 1.15  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.14  2005/12/09 17:29:10  scowan
 * Updated to use credentials.
 *
 * Revision 1.13  2005/07/11 19:44:45  scowan
 * Changed id type to binary handle.
 *
 * Revision 1.11  2005/01/19 22:19:56  scowan
 * Added support for descriptor.
 *
 * Revision 1.10  2004/04/12 20:19:32  scowan
 * Added is inverted method.
 *
 * Revision 1.4  2003/05/13 17:58:46  scowan
 * Modified to be able to inherit from it.
 *
 * Revision 1.1  2003/05/13 16:23:55  scowan
 * Initial revision
 *
 * \endif
 **/

#ifndef INC_RVC_BINARY_H
#define INC_RVC_BINARY_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class BINARYHANDLE;
	class OBJMAKEPARMS;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_GenericRef RVC objects.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_GenericRef.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_GenericRef.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
class BINARY : public RVC::OBJECT {  
	public:

		//! Default constructor
		BINARY ();

		//! Copy constructor
		BINARY (
			const RVC::BINARY& obj
			);

		//! Destructor
		virtual ~BINARY ();

		//! Assignment
		RVC::BINARY& operator= (
			const RVC::BINARY& rhs
			);

		//! Determine whether or not object is byte order inverted
		//! @return "True' if the object need to be inverted (SwapBytes())			
		bool IsInverted (
			) const;
		
		//! Make a child rvc binary object from a given parent with a name and description
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a child rvc binary object from a given RVCOBJITEM
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const char* Source = 0,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read binary object into a simple array			
		ERRVALUE Read (
			SIMPLE_ARRAY<UINT8>& data
			) const;

		//! Write binary object from a simple array
		//! Will overwrite existing data and resize object to 'data.GetNumItems()'
		ERRVALUE Write (
			const SIMPLE_ARRAY<UINT8>& data
			);

		//! Write binary object from a void pointer and number of bytes
		//! Will overwrite existing data and resize object to 'NumBytes'
		ERRVALUE Write (
			const void* const data,
			INT32 NumBytes
			);
			
	protected:

		//! Used for objects that inherit from this object
		ERRVALUE Make (const OBJECT& parent, OBJMAKEPARMS& ObjMakeParms);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		BINARYHANDLE* m_BinaryHandle;

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;		//!< Get the default object type
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of RVC namespace

#endif	// INC_RVC_BINARY_H
