/**
 * \file rvc/histogrm.h 
 * \brief interface for the RVC::HISTOGRAM class.
 *
 * \if NODOC
 * $Id: histogrm.h_v 1.29 2011/12/09 22:57:01 mju Exp $
 *
 * $Log: histogrm.h_v $
 * Revision 1.29  2011/12/09 22:57:01  mju
 * Remove redundant hdr.
 *
 * Revision 1.28  2010/07/23 17:07:45  mju
 * Add countScale member and method to get.
 *
 * Revision 1.19  2005/12/19 17:46:59  dwilliss
 * adjusted credentials to account for prior default parameters
 *
 * Revision 1.18  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.14  2004/08/31 13:40:24  mju
 * Add getBinRange.
 *
 * Revision 1.8  2001/10/26 21:54:08  mju
 * Rename methods so clear whether getting "bin" or "cell" ranges etc.
 * Add GetBinIndexFromValue().
 *
 * Revision 1.1  2000/09/29 15:50:16  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_HISTOGRM_H
#define INC_RVC_HISTOGRM_H

#ifndef INC_RVC_RASTER_H
	#include <rvc/raster.h>
#endif

#ifndef	INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class REGION2D;
class STAT_UNIVARIATE;
namespace RVC {
	class CREDENTIALS;
	class FILEHANDLE;
	struct HISTINFO;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_Histogram objects. 
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Histogram.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
class HISTOGRAM : public RVC::OBJECT {
	public:
	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FILTER_COMPONENT;
		#endif
		
		HISTOGRAM ();

		HISTOGRAM (
			const RVC::HISTOGRAM& rhs
			);

		virtual ~HISTOGRAM ();

		RVC::HISTOGRAM& operator= (
			const RVC::HISTOGRAM& rhs
			);

		//! Compute histogram from raster object inside the region boundaries
		ERRVALUE ComputeFromRegion (
			const RVC::RASTER& RastObj,
			const REGION2D& Region,
#ifdef RVC_CREDENTIALS_MANDATORY
			RVC::RASTER::COMPONENT Component,
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			RVC::RASTER::COMPONENT Component = RVC::RASTER::COMPONENT_Default,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Get histogram bin index corresponding to specified cell value.
		//! @return Index, may not correspond to valid bin.
		int GetBinIndexFromValue (
			double CellValue
			) const;

		//! Get histogram bin interval.
		//! @return Bin interval
		double GetBinInterval (
			) const;

		//! Get maximum histogram bin value.
		//! This returns the highest cell value which could be counted in last histogram bin.
		//! @return Maximum value, may be less than maximum cell value.
		double GetBinMaxValue (
			) const;

		//! Get minimum histogram bin value.
		//! This returns the lowest cell value which could be counted in first histogram bin.
		//! @return Minimum value, may be greater than minimum cell value.
		double GetBinMinValue (
			) const;

		//! Get range of histogram bin values.
		const DOUBLE_RANGE& GetBinRange (
			) const { return (m_BinRange); }

		//! Get cell count depending on cell value
		//! @return Cell count of cell value
		UINT32 GetCellCount (
			double CellValue
			) const;

		//! Get array of histogram cell counts
		//! @return Cell count list
		const SIMPLE_ARRAY<UINT32>& GetCellCounts (
			) const;

		//! Get maximum cell value.
		//! @return Cell value, may be less than minimum bin value.
		double GetCellMaxValue (
			) const { return (m_CellRange.GetMaximum()); }

		//! Get minimum cell value.
		double GetCellMinValue (
			) const { return (m_CellRange.GetMinimum()); }

		//! Get cell value range.
		void GetCellRange (
			DOUBLE_RANGE& CellRange
			) const { CellRange = m_CellRange; }

		//! Get column sampling interval
		//! @return Column sampling interval
		UINT16 GetColumnSampleInterval (
			) const;
			
		//! Get raster component that this histogram was generated for
		//! @return Raster Component (RVC::RASTER::COMPONENT)
		RVC::RASTER::COMPONENT GetComponent (
			) const;

		//! Get scale for "count" values.
		UINT16 GetCountScale (
			) const { return (m_CountScale); }

		//! Get number of cells with values greater than maximum bin value.
		//! @return Cell count.
		UINT32 GetGreaterThanCount (
			) const;
			
		//! Get number of cells with values less than minimum bin value.
		//! @return Cell count.
		UINT32 GetLessThanCount (
			) const;
			
		//! Get line sampling interval
		//! @return Line sampling interval
		UINT16 GetLineSampleInterval (
			) const;

		//! Get the computed mean histogram value
		//! @return Mean value
		double GetMean (
			) const;

		//! Get number of null cells in the raster
		//! @return Number of raster null cells
		UINT32 GetNullCellCount (
			) const;

		//! Get statistics as STAT_UNIVARIATE.
		ERRVALUE GetStatistics (
			STAT_UNIVARIATE& stats				//!< Statistics returned
			) const;
			
		//! Get the computed standard deviation histogram value
		//! @return Standard Deviation value
		double GetStdDev (
			) const;

		//! Create a histogram.
		ERRVALUE Make (
			const RVC::RASTER& parent,
#ifdef RVC_CREDENTIALS_MANDATORY
			RVC::RASTER::COMPONENT Component,
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			RVC::RASTER::COMPONENT Component = RVC::RASTER::COMPONENT_Default,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a histogram under the raster object.  If the existing histogram is old or does
		//! not exist, a histogram is created.
		ERRVALUE Read (
			const RVC::RASTER& parent,
#ifdef RVC_CREDENTIALS_MANDATORY
			RVC::RASTER::COMPONENT Component,
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			RVC::RASTER::COMPONENT Component = RVC::RASTER::COMPONENT_Default,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Set bin range.
		void SetBinRange (
			const DOUBLE_RANGE& range
			);

		//! Set sampling intervals.
		void SetSampleIntervals (
			UINT16 LineInterval,
			UINT16 ColumnInterval
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<UINT32> m_CellCount;		//!< Array of cell counts
		DOUBLE_RANGE m_BinRange;					//!< Overall bin range
		double m_BinInterval;						//!< Bin interval
		double m_Mean;	   							//!< Mean value within selected bin range
		double m_StdDev;   							//!< Standard Deviation within selected bin range
		double m_Sum;	   							//!< Sum of values
		double m_SumOfSquares;						//!< Sum of squares of values
		double m_MeanFull;							//!< Mean value for entire data range
		UINT32 m_CountLtMin;							//!< Number of cells with values less minimum bin
		UINT32 m_CountGtMax;							//!< Number of cells with values greater maximum bin
		UINT32 m_CountNull;							//!< Number of NULL cells in the raster
		UINT16 m_LineSample;							//!< Line sampling interval
		UINT16 m_ColumnSample;						//!< Column sampling interval
		DOUBLE_RANGE m_CellRange;					//!< Overall cell range
		LRECT2D m_HistExtents;						//!< Extents that histogram was calculated, in raster cell coordinates
		RASTER::COMPONENT m_Component;
		UINT16 m_CountScale;							// Scale for "count" values

		ERRVALUE ComputeDefaultHistogram (const RASTER& RastObj, const CREDENTIALS* Credentials);
		ERRVALUE ComputeHistogram (const OBJITEM& SrcObjItem, const CREDENTIALS* Credentials);
		void GetDftDescriptor (RVC::RASTER::COMPONENT Component, RVC::DESCRIPTOR& Descriptor) const;
		void GetInfo (HISTINFO& histinfo) const;
		static ERRVALUE GetInfo (FILEHANDLE* FileHandle, const OBJECT& obj, HISTINFO& histinfo);
		ERRVALUE SaveHistogram (const RASTER& RastObj, const DESCRIPTOR& Descriptor, const CREDENTIALS* Credentials);
		void SetInfo (const HISTINFO& histinfo, IMAGE::CELLTYPE CellType);
			
		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		class COMPHIST;
		friend class COMPHIST;
		friend class FILTER_COMPONENT;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Filter to determine which histogram object to select based on the RASTER::COMPONENT needed.
class HISTOGRAM::FILTER_COMPONENT : public OBJECT::FILTER {
	public:

		//! Constructor with parent raster object and RASTER::COMPONENT to filter with.
		FILTER_COMPONENT (
			const RASTER& ParentRastObj,
			RASTER::COMPONENT Component
			);

		//! Destructor
		virtual ~FILTER_COMPONENT ();
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RASTER::COMPONENT m_Component;
		IMAGE::CELLTYPE m_CellType;
		FILEHANDLE* m_FileHandle;
		
		virtual bool Filter (const RVC::OBJECT& obj);
		#endif		// GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif	// INC_RVC_HISTOGRM_H
