/**
 * \file soa/wmscapabilities.h
 * \brief SOA::WMS "capabilities" class definitions.
 *
 * \if NODOC
 * $Id: wmscapabilities.h_v 1.10 2011/02/01 21:34:58 bcolombe Exp $
 *
 * $Log: wmscapabilities.h_v $
 * Revision 1.10  2011/02/01 21:34:58  bcolombe
 * Add OnlineResource to REQUEST
 *
 * Revision 1.9  2007/05/07 14:15:44  mju
 * Use mistringlist hdr.
 *
 * Revision 1.8  2007/03/29 16:37:11  mju
 * Make getGeographicBoundingBox non-inline.
 *
 * Revision 1.7  2007/02/16 21:36:23  mju
 * Nest exception class inside capability.
 *
 * Revision 1.6  2007/02/12 21:13:56  mju
 * Add getOperation methods.
 *
 * Revision 1.5  2007/02/02 22:25:15  mju
 * *** empty log message ***
 *
 * Revision 1.4  2007/01/25 22:11:56  mju
 * Add findLayer and findStyle.
 *
 * Revision 1.3  2007/01/19 22:25:49  mju
 * Add more value retrieval methods for layer.
 *
 * Revision 1.2  2007/01/19 22:15:05  mju
 * Change main parser to use httpmessage so can retain xmldoc for later access.
 *
 * Revision 1.1  2007/01/19 18:39:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_WMSCAPABILITIES_H
#define	INC_SOA_WMSCAPABILITIES_H

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MI32_SPATREF_H
	#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif

class XMLDOC;
class XMLNODE;
class HTTPMESSAGE;
class SUBREGION2D;

namespace SOA {
namespace WMS {

//=====================================================================================================================

class ONLINERESOURCE {
	public:
		//! Constructor.
		ONLINERESOURCE ();

		//! Destructor.
		~ONLINERESOURCE ();

		const MISTRING& GetHref (
			) const { return (m_href); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		MISTRING m_href;
		#endif
	};

//=====================================================================================================================

class BASEURL {
	public:

		//! Get "Format" (MIME type) string.
		const MISTRING& GetFormat (
			) const { return (m_Format); }

		//! Get online resouce.
		const ONLINERESOURCE& GetOnlineResource (
			) const { return (m_OnlineResource); }

	protected:

		//! Constructor.
		BASEURL ();

		//! Destructor.
		~BASEURL ();

		//! Parse item from XMLNODE.
		//! To be called by derived class ParseXML method when child item not handled.
		ERRVALUE ParseItem (
			const XMLNODE *itemnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		ONLINERESOURCE m_OnlineResource;
		MISTRING m_Format;
		#endif
	};

//=====================================================================================================================

class DATAURL : public BASEURL {
	public:

		//! Constructor.
		DATAURL ();

		//! Destructor.
		~DATAURL ();

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);
	};
typedef MILIST<DATAURL> DATAURLLIST;

//=====================================================================================================================

class SIZEDATAURL : public BASEURL {
	public:

		//! Constructor.
		SIZEDATAURL ();

		//! Destructor.
		~SIZEDATAURL ();

		//! Get height.
		UINT32 GetHeight (
			) const { return (m_Height); }

		//! Get width.
		UINT32 GetWidth (
			) const { return (m_Width); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Attributes
		UINT32 m_Width;
		UINT32 m_Height;
		#endif
	};
typedef MILIST<SIZEDATAURL> SIZEDATAURLLIST;

//=====================================================================================================================

class CONTACTPERSON {
	public:

		//! Constructor.
		CONTACTPERSON ();

		//! Destructor.
		~CONTACTPERSON ();

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

		//! Get "Organization" element.
		const MISTRING& GetOrganization (
			) const { return (m_Organization); }

		//! Get "Person" element.
		const MISTRING& GetPerson (
			) const { return (m_Person); }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		MISTRING m_Person;
		MISTRING m_Organization;
		#endif
	};

//=====================================================================================================================

class CONTACTADDRESS {
	public:
		//! Constructor.
		CONTACTADDRESS ();

		//! Destructor.
		~CONTACTADDRESS ();

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

		//! Get "Address" element.
		const MISTRING& GetAddress (
			) const { return (m_Address); }

		//! Get "AddressType" element.
		const MISTRING& GetAddressType (
			) const { return (m_AddressType); }

		//! Get "City" element.
		const MISTRING& GetCity (
			) const { return (m_City); }

		//! Get "Country" element.
		const MISTRING& GetCountry (
			) const { return (m_Country); }

		//! Get "PostCode" element.
		const MISTRING& GetPostCode (
			) const { return (m_PostCode); }

		//! Get "StateOrProvince" element.
		const MISTRING& GetStateOrProvince (
			) const { return (m_StateOrProvince); }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		MISTRING m_AddressType;
		MISTRING m_Address;
		MISTRING m_City;
		MISTRING m_StateOrProvince;
		MISTRING m_PostCode;
		MISTRING m_Country;
		#endif
	};

//=====================================================================================================================

class CONTACTINFORMATION {
	public:
		//! Constructor.
		CONTACTINFORMATION ();

		//! Destructor.
		~CONTACTINFORMATION ();

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

		//! Get "ContactAddress" element.
		const WMS::CONTACTADDRESS& GetAddress (
			) const { return (m_ContactAddress); }

		//! Get "ContactPosition" element.
		const MISTRING& GetPosition (
			) const { return (m_ContactPosition); }

		//! Get "ContactElectronicMailAddress" element.
		const MISTRING& GetEmail (
			) const { return (m_ContactEmail); }

		//! Get "ContactFaxTelephone" element.
		const MISTRING& GetFaxPhone (
			) const { return (m_ContactFaxPhone); }

		//! Get "ContactPersonPrimary" element.
		const WMS::CONTACTPERSON& GetPersonPrimary (
			) const { return (m_ContactPersonPrimary); }

		//! Get "ContactVoiceTelephone" element.
		const MISTRING& GetVoicePhone (
			) const { return (m_ContactVoicePhone); }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		WMS::CONTACTPERSON m_ContactPersonPrimary;
		MISTRING m_ContactPosition;
		WMS::CONTACTADDRESS m_ContactAddress;
		MISTRING m_ContactVoicePhone;
		MISTRING m_ContactFaxPhone;
		MISTRING m_ContactEmail;
		#endif
	};

//=====================================================================================================================

//! WMS "Service" element.
class SERVICE {
	public:
		//! Constructor.
		SERVICE ();

		//! Destructor.
		~SERVICE ();

		//! Get Abstract.
		const MISTRING& GetAbstract (
			) const { return (m_Abstract); }

		//! Get "Access Constraints" value.
		const MISTRING& GetAccessConstraints (
			) const { return (m_AccessConstraints); }

		//! Get contact information.
		const WMS::CONTACTINFORMATION& GetContactInformation (
			) const { return (m_ContactInformation); }

		//! Get "Fees" value.
		const MISTRING& GetFees (
			) const { return (m_Fees); }

		//! Get Keyword list.
		const MISTRINGLIST& GetKeywordList (
			) const { return (m_KeywordList); }

		//! Get layer limit.
		UINT32 GetLayerLimit (
			) const { return (m_LayerLimit); }

		//! Get maximum allowed height for request.
		UINT32 GetMaxHeight (
			) const { return (m_MaxHeight); }

		//! Get maximum allowed width for request.
		UINT32 GetMaxWidth (
			) const { return (m_MaxWidth); }

		//! Get online resource.
		const WMS::ONLINERESOURCE& GetOnlineResource (
			) const { return (m_OnlineResource); }

		//! Get Title.
		const MISTRING& GetTitle (
			) const { return (m_Title); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		MISTRING m_Title;
		MISTRING m_Abstract;
		MISTRINGLIST m_KeywordList;
		WMS::ONLINERESOURCE m_OnlineResource;
		WMS::CONTACTINFORMATION m_ContactInformation;
		MISTRING m_Fees;
		MISTRING m_AccessConstraints;
		UINT32 m_LayerLimit;
		UINT32 m_MaxWidth;
		UINT32 m_MaxHeight;
		#endif
	};

//=====================================================================================================================

class OPERATION {
	public:
		//! Constructor.
		OPERATION ();

		//! Destructor.
		~OPERATION ();

		//! Get format list.
		const MISTRINGLIST& GetFormatList (
			) const { return (m_FormatList); }

		//! Get online resouce.
		const ONLINERESOURCE& GetOnlineResource (
			) const { return (m_OnlineResource); }

		//! Determine if is available.
		bool IsAvailable (
			) const { return (m_IsAvailable); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		MISTRINGLIST m_FormatList;
		WMS::ONLINERESOURCE m_OnlineResource;
		// Internal
		bool m_IsAvailable;
		#endif
	};

//=====================================================================================================================

class REQUEST {
	public:
		//! Constructor.
		REQUEST ();

		//! Destructor.
		~REQUEST ();

		//! Get operation information for GetCapabilities request.
		const WMS::OPERATION& GetOperationGetCapabilities (
			) const { return (m_OpGetCapabilities); }

		//! Get operation information for GetFeatureInfo request.
		const WMS::OPERATION& GetOperationGetFeatureInfo (
			) const { return (m_OpGetFeatureInfo); }

		//! Get operation information for GetMap request.
		const WMS::OPERATION& GetOperationGetMap (
			) const { return (m_OpGetMap); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		WMS::OPERATION m_OpGetCapabilities;
		WMS::OPERATION m_OpGetMap;
		WMS::OPERATION m_OpGetFeatureInfo;
		#endif
	};

//=====================================================================================================================

class EXTENDEDCAPABILITIES {
	public:
	private:
	};

//=====================================================================================================================

class BOUNDINGBOX {
	public:
		//! Constructor.
		BOUNDINGBOX ();

		//! Destructor.
		~BOUNDINGBOX ();

		//! Get Coordinate Reference System.
		const SPATREF::COORDREFSYS& GetCRS (
			) const { return (m_CRS); }

		//! Get extent.
		const DRECT2D& GetExtent (
			) const { return (m_Extent); }

		//! Get resolution.
		const DPOINT2D& GetResolution (
			) const { return (m_Resolution); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Attributes
		SPATREF::COORDREFSYS m_CRS;
		DRECT2D m_Extent;
		DPOINT2D m_Resolution;
		#endif
	};
typedef MILIST<BOUNDINGBOX> BOUNDINGBOXLIST;

//=====================================================================================================================

class DIMENSION {
	public:
		//! Constructor.
		DIMENSION ();

		//! Destructor.
		~DIMENSION ();

		//! Get Default.
		const MISTRING& GetDefault (
			) const { return (m_Default); }

		//! Get Name.
		const MISTRING& GetName (
			) const { return (m_Name); }

		//! Get Units.
		const MISTRING& GetUnits (
			) const { return (m_Units); }

		//! Get UnitSymbol.
		const MISTRING& GetUnitSymbol (
			) const { return (m_UnitSymbol); }

		//! Get Value.
		const MISTRING& GetValue (
			) const { return (m_Value); }

		bool HasMultipleValues (
			) const { return (m_MultipleValues); }

		bool IsCurrent (
			) const { return (m_Current); }

		bool IsNearestValue (
			) const { return (m_NearestValue); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Attributes
		MISTRING m_Name;
		MISTRING m_Units;
		MISTRING m_UnitSymbol;
		MISTRING m_Default;
		MISTRING m_Value;
		bool m_MultipleValues;
		bool m_NearestValue;
		bool m_Current;
		#endif
	};
typedef MILIST<DIMENSION> DIMENSIONLIST;

//=====================================================================================================================

class ATTRIBUTION {
	public:
		//! Constructor.
		ATTRIBUTION ();

		//! Destructor.
		~ATTRIBUTION ();

		//! Get Logo URL.
		const WMS::SIZEDATAURL& GetLogoURL (
			) const { return (m_LogoURL); }

		//! Get online resouce.
		const ONLINERESOURCE& GetOnlineResource (
			) const { return (m_OnlineResource); }

		//! Get Title.
		const MISTRING& GetTitle (
			) const { return (m_Title); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		MISTRING m_Title;
		WMS::ONLINERESOURCE m_OnlineResource;
		WMS::SIZEDATAURL m_LogoURL;
		#endif
	};

//=====================================================================================================================

class METADATAURL : BASEURL {
	public:
		enum TYPE {
			TYPE_Unknown,
			TYPE_ISO_19115_2003,		// ISO 19115:2003, XML type="ISO19115:2003"
			TYPE_FGDC_1998,			// FGDC-STD-001-1998, XML type="FGDC:1998"
			TYPE_COUNT
			};

		//! Constructor.
		METADATAURL ();

		//! Destructor.
		~METADATAURL ();

		//! Get metadata Type.
		TYPE GetType (
			) const { return (m_Type); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Attributes
		TYPE m_Type;
		#endif
	};
typedef MILIST<METADATAURL> METADATAURLLIST;

//=====================================================================================================================

class STYLE {
	public:
		//! Constructor.
		STYLE ();

		//! Destructor.
		~STYLE ();

		//! Get Abstract.
		const MISTRING& GetAbstract (
			) const { return (m_Abstract); }

		//! Get Legend URL list.
		const WMS::SIZEDATAURLLIST& GetLegendURLList (
			) const { return (m_LegendURLList); }

		//! Get Name.
		const MISTRING& GetName (
			) const { return (m_Name); }

		//! Get Style Sheet URL.
		const WMS::DATAURL& GetStyleSheetURL (
			) const { return (m_StyleSheetURL); }

		//! Get Style URL.
		const WMS::DATAURL& GetStyleURL (
			) const { return (m_StyleURL); }

		//! Get Title.
		const MISTRING& GetTitle (
			) const { return (m_Title); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		MISTRING m_Name;
		MISTRING m_Title;
		MISTRING m_Abstract;
		WMS::SIZEDATAURLLIST m_LegendURLList;
		DATAURL m_StyleSheetURL;
		DATAURL m_StyleURL;
		#endif
	};
typedef MILIST<STYLE> STYLELIST;

//=====================================================================================================================

class LAYER {
	public:
		//! Constructor.
		LAYER ();

		//! Destructor.
		~LAYER ();

		//! Compute subregion for this layer in specified CRS.
		ERRVALUE ComputeSubRegion (
			const SPATREF::COORDREFSYS& CoordRefSys,
			SUBREGION2D& SubRegion
			) const;

		//! Find bounding box matching specified CRS.
		//! If no CRS is specified will find first non-local bounding box.
		//! To find local bounding box, specify appropriate local CRS.
		const WMS::BOUNDINGBOX* FindBoundingBox (
			const SPATREF::COORDREFSYS *pCRS
			) const;

		//! Find layer by name.
		//! @return Pointer to layer found, NULL if not
		const WMS::LAYER* FindLayer (
			const MISTRING& LayerName				//! Name of layer to search for
			) const;

		//! Find style by name.
		//! @return Pointer to style found, NULL if not
		const WMS::STYLE* FindStyle (
			const MISTRING& StyleName				//! Name to search for
			) const;

		//! Get Abstract.
		const MISTRING& GetAbstract (
			) const { return (m_Abstract); }

		//! Get attribution.
		const WMS::ATTRIBUTION& GetAttribution (
			) const { return (m_Attribution); }

		//! Get bounding box list.
		const WMS::BOUNDINGBOXLIST& GetBoundingBoxList (
			) const { return (m_BoundingBoxList); }

		//! Get cascaded value.
		int GetCascaded (
			) const { return (m_Cascaded); }

		//! Get CRS id string (to use in request).
		//! @return true if found, false if not.
		bool GetCoordRefSysId (
			const SPATREF::COORDREFSYS& CoordRefSys,
			MISTRING& IdStr										//!< CRS ID string returned
			) const;

		//! Get CRS list.
		//! Will include inherited CRS's from ancestor layers.
		void GetCoordRefSysList (
			SPATREF::COORDREFSYSLIST& CoordRefSysList,	//!< CRS list returned
			bool IncludeTopLayer									//!< Include entries from topmost layer
			) const;

		//! Get fixed height value.
		int GetFixedHeight (
			) const { return (m_FixedHeight); }

		//! Get fixed width value.
		int GetFixedWidth (
			) const { return (m_FixedWidth); }

		//! Get geographic bounding box.
		const DRECT2D& GetGeographicBoundingBox (
			) const;

		//! Get Keyword list.
		const MISTRINGLIST& GetKeywordList (
			) const { return (m_KeywordList); }

		//! Get list of layers.
		const MILIST<LAYER>& GetLayerList (
			) const { return (m_LayerList); }

		//! Get maximum scale denominator.
		double GetMaxScaleDenominator (
			) const { return (m_MaxScaleDenominator); }

		//! Get minimum scale denominator.
		double GetMinScaleDenominator (
			) const { return (m_MinScaleDenominator); }

		//! Get Name.
		//! Name will be empty if layer cannot be requested separately.
		const MISTRING& GetName (
			) const { return (m_Name); }

		//! Get parent layer.
		//! @return Pointer to parent, NULL if topmost layer.
		const LAYER* GetParent (
			) const { return (m_pParent); }

		//! Get style list.
		const STYLELIST& GetStyleList (
			) const { return (m_StyleList); }

		//! Get Title.
		const MISTRING& GetTitle (
			) const { return (m_Title); }

		//! Get "no subsets" attribute.
		bool IsNoSubsets (
			) const { return (m_NoSubsets); }

		//! Get opaque attribute.
		bool IsOpaque (
			) const { return (m_Opaque); }

		//! Get queryable attribute.
		bool IsQueryable (
			) const { return (m_Queryable); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const LAYER *pParent,
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		const WMS::LAYER *m_pParent;
		// Attributes
		bool m_Queryable;
		int m_Cascaded;
		bool m_Opaque;
		bool m_NoSubsets;
		int m_FixedWidth;
		int m_FixedHeight;
		// Elements
		MISTRING m_Name;
		MISTRING m_Title;
		MISTRING m_Abstract;
		MISTRINGLIST m_KeywordList;
		SPATREF::COORDREFSYSLIST m_CoordRefSysList;
		MISTRINGLIST m_CoordRefSysIdList;
		DRECT2D m_GeographicBoundingBox;
		WMS::BOUNDINGBOXLIST m_BoundingBoxList;
		WMS::DIMENSIONLIST m_DimensionList;
		WMS::ATTRIBUTION m_Attribution;
		MISTRINGLIST m_AuthorityURLList;
		MISTRINGLIST m_IdentifierList;
		WMS::METADATAURLLIST m_MetadataURLList;
		WMS::DATAURLLIST m_DataURLList;
		WMS::DATAURLLIST m_FeatureListURLList;
		STYLELIST m_StyleList;
		double m_MinScaleDenominator;
		double m_MaxScaleDenominator;
		MILIST<LAYER> m_LayerList;

		void GetCoordRefSysListLow (SPATREF::COORDREFSYSLIST& CoordRefSysList, bool IncludeTopLayer) const;
		ERRVALUE ParseGeographicBoundingBox (const XMLNODE *xmlnode);
		ERRVALUE ParseLatLonBoundingBox (const XMLNODE *xmlnode);

		#endif
	};
typedef MILIST<LAYER> LAYERLIST;

//=====================================================================================================================
//! WMS "Capability" element.
class CAPABILITY {
	public:
		
		//! Container for exception format list.
		// Nested to avoid conflict with actual exception container.
		class EXCEPTION {
			public:
				//! Constructor.
				EXCEPTION ();

				//! Destructor.
				~EXCEPTION ();

				//! Get format list.
				const MISTRINGLIST& GetFormatList (
					) const { return (m_FormatList); }

				//! Parse values from XMLNODE.
				ERRVALUE ParseXML (
					const XMLNODE *xmlnode
					);

			private:
				#ifndef	GENERATING_DOXYGEN_OUTPUT
				MISTRINGLIST m_FormatList;
				#endif
			};

		//! Constructor.
		CAPABILITY ();

		//! Destructor.
		~CAPABILITY ();

		//! Find layer by name.
		//! @return Pointer to layer found, NULL if not
		const WMS::LAYER* FindLayer (
			const MISTRING& LayerName				//! Name of layer to search for
			) const;

		//! Get CRS id string (to use in request).
		//! @return true if found, false if not.
		bool GetCoordRefSysId (
			const SPATREF::COORDREFSYS& CoordRefSys,
			MISTRING& IdStr										//!< CRS ID string returned
			) const;

		//! Get CRS list common to all specified layers.
		//! Will always include all CRS's from main layer as inherited.
		void GetCommonCoordRefSysList (
			const MISTRINGLIST& LayerNameList,				//!< List of layer names, may be empty
			SPATREF::COORDREFSYSLIST& CoordRefSysList		//!< CRS list returned
			) const;

		//! Get "Exception" element.
		const WMS::CAPABILITY::EXCEPTION& GetException (
			) const { return (m_Exception); }

		//! Get "Layer" element.
		const WMS::LAYER& GetLayer (
			) const { return (m_Layer); }

		//! Get "Request" element.
		const WMS::REQUEST& GetRequest (
			) const { return (m_Request); }

		//! Parse values from XMLNODE.
		ERRVALUE ParseXML (
			const XMLNODE *xmlnode
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		// Elements
		WMS::REQUEST m_Request;
		WMS::CAPABILITY::EXCEPTION m_Exception;
		WMS::LAYER m_Layer;
		#endif
	};

//=====================================================================================================================
//! WMS Capabilities element.
class CAPABILITIES {
	public:

		//! Constructor.
		CAPABILITIES ();

		//! Destructor.
		~CAPABILITIES ();

		//! Increment reference count.
		void AddRef (
			) { ++m_RefCount; }

		//! Get "Capability" element.
		const WMS::CAPABILITY& GetCapability (
			) const { return (m_Capability); }

		//! Get "Service" element.
		const WMS::SERVICE& GetService (
			) const { return (m_Service); }

		//! Get "version" attribute.
		const MISTRING& GetVersion (
			) const { return (m_Version); }

		//! Get "updateSequence" attribute.
		const MISTRING& GetUpdateSequence (
			) const { return (m_UpdateSequence); }

		//! Get XMLDOC instance used.
		const XMLDOC& GetXML (
			) const { return (*m_pDocReply); }

		//! Parse message from server.
		ERRVALUE ParseMessage (
			const HTTPMESSAGE& message
			);

		//! Decrement reference count, destroy if no more references.
		void Release (
			) {
			if ((--m_RefCount) == 0) delete this;
			}

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		UINT32 m_RefCount;
		XMLDOC *m_pDocReply;
		// Elements
		WMS::SERVICE m_Service;
		WMS::CAPABILITY m_Capability;
		// Attributes
		MISTRING m_Version;
		MISTRING m_UpdateSequence;
		// Unimplemented
		CAPABILITIES (const CAPABILITIES& rhs);
		CAPABILITIES& operator= (const CAPABILITIES& rhs);
		#endif
	};

//=====================================================================================================================

}	// End namespace WMS
}	// End namespace SOA

#endif	// INC_SOA_WMSCAPABILITIES_H
