/**
 * \file imagesourcehttp.h <spatmod/imagesourcehttp.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_HTTP definition.
 *
 * \if NODOC
 * $Id: imagesourcehttp.h_v 1.3 2007/04/09 13:35:31 mju Exp $
 *
 * $Log: imagesourcehttp.h_v $
 * Revision 1.3  2007/04/09 13:35:31  mju
 * Add v_getDimensions.
 *
 * Revision 1.2  2007/01/26 18:45:39  mju
 * Make onNotifyReply take non-const parm so can detach.
 *
 * Revision 1.1  2007/01/22 18:39:58  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEHTTP_H
#define	INC_SPATMOD_IMAGESOURCEHTTP_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

class HTTPMESSAGE;
class TRANS2D_MAPGEN;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source to send HTTP request and manage reply.
//! Will use the MIME content type to determine the actual image format.
//! Unlike most other sources, the dimensions and pixel properties of this source might change, which if
//! happens will occur when the notification of the reply is sent.  Normally this occurs during UI idle processing.
//! For this reason, care must be taken when using this class, and typically should avoid having later stages in
//! the pipeline take ownership, as most stages will determine their own dimensions and pixel properties during
//! construction or initialization and will not handle subsequent changes.
class SOURCE_HTTP : public SOURCE {
	public:

		//! Status enumeration.
		enum STATUS {
			STATUS_Waiting,									//!< Waiting for reply, placeholder content available
			STATUS_Complete,									//!< Request complete, final content available
			STATUS_Failed										//!< Failed, use GetErrValue to get error code
			};

		//! Constructor.
		SOURCE_HTTP (
			const MISTRING& url,								//!< URL string
			const PIXEL& PixelProp,							//!< Pixel properties for placeholder
			UINT32 TotalColumns,								//!< Total columns for placeholder
			UINT32 TotalRows,									//!< Total rows for placeholder
			const TRANS2D_MAPGEN *pTransToCRS = 0		//!< Transformation from device to map CRS
			);

		// Destructor.
		virtual ~SOURCE_HTTP ();

		//! Get error value (for STATUS_Failed).
		ERRVALUE GetErrValue (
			) const { return (m_ErrValue); }

		//! Get current status.
		STATUS GetStatus (
			) const { return (m_Status); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MISTRING m_url;
		SOURCE *m_pSource;
		STATUS m_Status;
		ERRVALUE m_ErrValue;

		void OnNotifyReply (
			HTTPMESSAGE& reply			// Will have data detached on return if successful
			);

		void SetStatusError (
			ERRVALUE err
			);

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual const PIXELTABLE* v_GetPixelLookupTable () const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_HTTP (const SOURCE_HTTP&);
		SOURCE_HTTP& operator= (const SOURCE_HTTP&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEHTTP_H

